
CastParty.Debug(4, "loading CastParty.lua")

-----------------------
-- Binding handlers

function CastParty.Event.OnSlashCmd(msg)
   msg = string.lower(msg or "");
   
   local modLoaded
   local modReason
   
   if msg == "" then
      -- have the options party load on demand!
      modLoaded, modReason = LoadAddOn('CastPartyOptions');
    	if (not modLoaded) then
    		note(RED..'<CastParty><Options><Module load error> '..modReason);
    	else
    	   CastPartyOptions_SetOptionValues();  -- this is to optimize load times.
         CastPartyOptions:Show();
    	end
   end

  if msg == 'kb' or msg == 'keybindings'  then
  	 	-- have the options party load on demand!
      modLoaded, modReason = LoadAddOn('CastPartyOptions');
    	if (not modLoaded) then
    		note(RED..'<CastParty><Options><Module load error> '..modReason);
    	else
         CastPartyOptions_SetOptionValuesBindings(); -- this is to optimize load times.
         CastPartyKeyBindingsFrame:Show();
      end
   end
   
   -- dynamic debugger enable/disable
   -- if string.find(msg, 'debug',1,true) then
      -- local opt = string.sub(msg, 6)
      -- CastParty.DebugEnabled[opt] = not CastParty.DebugEnabled[opt]
      -- note('option changed: ', opt, CastParty.DebugEnabled[opt])
   -- end
end

function CastParty.Event.OnEvent(_event, ...)
   CastParty.Debug('event ', _event);
   CastParty.Profiler.Start(_event);
   
   local eventFunc = CastParty.Event[_event]
   if eventFunc then
     CastParty.Debug('event', "using CastParty.Event." .. _event)
     eventFunc(_event, unpack(arg))
   end

   CastParty.Profiler.Stop(_event);
end

function CastParty.Event.PLAYER_TARGET_CHANGED(event, ...)
   --CastParty.Debug(1, BLU.."target changed".. UnitName('target'))
   --we have clickedon, or updated the moused over unit!
   --I need to figure out how to get other buttons to work!
   --Right now, the button can't be recognized, so i default to the leftbutton.

   if CastPartyConfig.useBattlefieldHeal
   and not CastParty.State.usingBattlefieldHeal
   and not CastParty.State.casting_now
   and (IsControlKeyDown() or IsShiftKeyDown() or IsAltKeyDown())
   and UnitExists('target') then
      CastParty.State.usingBattlefieldHeal = true;
      CastParty.Debug( 1,  RED..'Battlefield heal', CastParty.State.usingBattlefieldHeal)
      -- for friendly and neutral
      if UnitIsFriend('target', 'player') then
         CastParty.Event.OnClickByUnit('LeftButton', 'target');
      -- accounts for enemy targets
      elseif not UnitIsDead('target') then
         CastParty.Healomatic.DispellTarget('Magic');
      end
    end
end

--[[***************************************************************************]]--
function CastParty.Event.PLAYER_ENTERING_WORLD(event, ...)
   -- so we only load the spells when we enter the world the first time (but not for instances)
   if not CastParty.State.are_spells_loaded then
      CastParty.State.are_spells_loaded = CastParty_LoadSpellData();
   end
   CastParty.State.isMeleeAttacking = false;

   --optimizations for the player loading times in world.
   this:RegisterEvent('PLAYER_AURAS_CHANGED');     -- required to update the player auras.
   this:RegisterEvent('UNIT_HEALTH');              -- required for spell canceling, may be changing though
   this:RegisterEvent('UNIT_AURA');                -- required to update the auras of units.
   this:RegisterEvent('PARTY_MEMBERS_CHANGED');    --party/raid member update stuff
   
   -- This one must be here so CP loads properly on UI reload
   CastParty.GUI.Init()
end

--[[***************************************************************************]]--
function CastParty.Event.PLAYER_LEAVING_WORLD(event, ...)
   --optimizations for the player loading times in world.
   this:UnregisterEvent('PLAYER_AURAS_CHANGED');     -- required to update the player auras.
   this:UnregisterEvent('UNIT_HEALTH');              -- required for spell canceling, may be changing though
   this:UnregisterEvent('UNIT_AURA');                -- required to update the auras of units.
   this:UnregisterEvent('PARTY_MEMBERS_CHANGED');    --party/raid member update stuff
end

function CastParty.Event.LEARNED_SPELL_IN_TAB(event, ...)
   CastParty_Debug( 'event', event )
   CastParty.State.are_spells_loaded = CastParty_LoadSpellData();
end

--[[***************************************************************************]]--
function CastParty.Event.VARIABLES_LOADED(event, ...)
   -- if not CastParty.Brain then
      -- CastParty.Debug(4, RED..'Brain did not properly load before '..event..' event, initializing brain then player config');
      -- if CastParty.BrainFactory then
        -- CastParty.Brain = CastParty.BrainFactory.new(className)
      -- else
        -- note('CastParty has no brain factory /cry')
      -- end
   -- end
   CastParty.Config.Load();

   if myAddOnsFrame_Register then
      myAddOnsFrame_Register(CastPartyMyAddOnsData)
   end

   --set the minimap button stuff!!
   CastParty_MinimapButton_SetPositionAngle(CastPartyConfig.MinimapButtonAngle);
   CastParty_UpdateFlashy();

   --hooks to control the escape press functionality for the control panel GUI's.
   CP_ToggleGameMenu      = ToggleGameMenu;
   ToggleGameMenu         = CastParty_ToggleGameMenu;

   --hooks to control when to show/hide the party frames
   CP_ShowPartyFrame      = ShowPartyFrame;
   CP_HidePartyFrame      = HidePartyFrame;
   ShowPartyFrame         = CastParty_ShowPartyFrame;
   HidePartyFrame         = CastParty_HidePartyFrame;

   --hooks to control when to show/hide the target frame
   CP_TargetFrame_Update      = TargetFrame_Update;
   TargetFrame_Update         = CastParty_TargetFrame_Update;

   --hooks to control when to show/hide the player frame
   CP_PlayerFrame_Update      = PlayerFrame_Update;
   PlayerFrame_Update         = CastParty_PlayerFrame_Update;

   --hooks to control custom click functionaility
   CP_Squeaky_CustomOnClickFunction = Squeaky_CustomOnClickFunction;
   CP_Nurfed_Unit_OnClick           = Nurfed_Unit_OnClick;

   --blizz default frames
   CP_TargetFrame_OnClick           = TargetFrame_OnClick;
   CP_PlayerFrame_OnClick           = PlayerFrame_OnClick;
   CP_PetFrame_OnClick              = PetFrame_OnClick;
   CP_PartyMemberFrame_OnClick      = PartyMemberFrame_OnClick;
   CP_PartyMemberPetFrame_OnClick   = PartyMemberPetFrame_OnClick;
   CP_BuffButton_OnClick            = BuffButton_OnClick;
   CP_TargetofTarget_OnClick        = TargetofTarget_OnClick;
   
   -- CTmod assist (target of target) frame
   CastParty.StoredHooks.CT_AssistFrame_OnClick = CT_AssistFrame_OnClick
end

--[[***************************************************************************]]--
function CastParty.Event.ADDON_LOADED(event, ...)
   CastParty.Debug(3, GRN.."addon loaded:"..arg[1])
   -- Copy saved variables to ther appropriate places
   if CastPartySave and CastPartySave.DebugLevel then
      CastParty.DebugLevel = CastPartySave.DebugLevel
   end

   if arg[1] == 'Blizzard_RaidUI' or arg[1] == 'CT_RaidAssist' then
      --hooks for click behavior in raid frames.
      if not CastParty.StoredHooks.RaidPulloutButton_OnClick then
         CastParty.StoredHooks.RaidPulloutButton_OnClick     = RaidPulloutButton_OnClick;
      end
      if not CastParty.StoredHooks.CT_RA_CustomOnClickFunction then
         CastParty.StoredHooks.CT_RA_CustomOnClickFunction   = CT_RA_CustomOnClickFunction;
      end
      if not CastParty.StoredHooks.CT_RA_MemberFrame_OnClick then
         CastParty.StoredHooks.CT_RA_MemberFrame_OnClick     = CT_RA_MemberFrame_OnClick;
      end

      -- Make sure to set this before reloading CP GUI
      CastParty.State.BlizzRaidEventLoaded = true;

      -- This must be here to fix CT_RA clickery problem on world/raid/instance enter
      CastParty.GUI.Init();
   end
end

--[[***************************************************************************]]--
function CastParty.TriggerTrinket(spell)
   -- Send a message to debug out put when we check.
   CastParty.Debug(4, "Checking Trinkets", spell);

   -- Return if we haven't got a defined table for TC.
   if tc_ActionListCast == nil then return end

   -- Trinket Cast is loaded, we can try to trigger the trinket
   -- This code is lifted from TrinketCast_UseAction and modified to
   -- use the spell name we are casting.
   tc_ActionName = spell;

   -- Get the number of spells in the action list table   
   tc_SpellCount = tonumber(tc_ActionListCast[1]);

   -- For every spell in the table, check to see if it's the one passed in.  If it is, then trigger it.
   for tc_Counter = 2, tc_SpellCount do
      if ((tc_ActionListCast[tc_Counter] == tc_ActionName) and (tc_SpellUse[tc_Counter] ~= 0)) then
         CastParty.Debug(1, "tc_ActionListCast[tc_counter]", tc_counter, tc_ActionListCast[tc_Counter] );
         if (((tc_PlayerClass == "Druid")  and ((tc_Counter == 3) or (tc_Counter == 4) or (tc_Counter == 11) or (tc_Counter == 12) or (tc_Counter == 17))) or 
            ((tc_PlayerClass == "Mage")    and ((tc_Counter == 2) or (tc_Counter == 4) or (tc_Counter == 5) or (tc_Counter == 6) or (tc_Counter == 9) or (tc_Counter == 11))) or 
            ((tc_PlayerClass == "Paladin") and ((tc_Counter == 2) or (tc_Counter == 4) or (tc_Counter == 6) or (tc_Counter == 7) or (tc_Counter == 8))) or 
            ((tc_PlayerClass == "Priest")  and ((tc_Counter == 2) or (tc_Counter == 4) or (tc_Counter == 5) or (tc_Counter == 7) or (tc_Counter == 9) or (tc_Counter == 10))) or 
            ((tc_PlayerClass == "Shaman")  and ((tc_Counter == 2) or (tc_Counter == 7) or (tc_Counter == 8))) or 
            ((tc_PlayerClass == "Warlock") and ((tc_Counter == 5) or (tc_Counter == 7)))) then
               CastParty.Debug(4, "Trinket EnactHeal");
               TrinketCast_EnactHeal(tc_SpellUse[tc_Counter]);
            break;
         else
               CastParty.Debug(4, "Trinket Enact");
               TrinketCast_Enact(slot, tc_SpellUse[tc_Counter]);
            break;
         end
      end
   end
end

--[[***************************************************************************]]--
function CastParty.Event.PLAYER_AURAS_CHANGED(event, ...)
   --update me!!!
   --if i'm in a raid, auras events are LAGGED because of blizz 'optimizations', so i MUST listen to PLAYER_AURAS_CHANGED
   CastParty_UpdateAuras(0);
   --now display those auras!
   if CastParty_DisplayAuras then
      CastParty_DisplayAuras(0);
   end
end

--[[***************************************************************************]]--
function CastParty.Event.PLAYER_ENTER_COMBAT(event, ...)
   CastParty.Debug('event', 'auto attack ON')
   CastParty.State.isMeleeAttacking = true;
end

--[[***************************************************************************]]--
function CastParty.Event.PLAYER_LEAVE_COMBAT(event, ...)
   CastParty.Debug('event', 'auto attack OFF')
   CastParty.State.isMeleeAttacking = false;
end

--[[***************************************************************************]]--
function CastParty.Event.START_AUTOREPEAT_SPELL(event, ...)
   CastParty.Debug('event', 'autocast ON')
   CastParty.State.isWandCasting = true;
end

function CastParty.Event.STOP_AUTOREPEAT_SPELL(event, ...)
   CastParty.Debug('event', 'autocast OFF')
   CastParty.State.isWandCasting = false;
end

--[[***************************************************************************]]--
function CastParty.Event.SPELLCAST_START(event, ...)
   local spellName      = arg[1];
   local spellDuration  = arg[2];
   local spellNameCheck

   CastParty.Debug('event', 'Starting Cast of', spellName, 'spellDuration', spellDuration, CastParty.State.spell_target_unit )
   CastParty.State.casting_now = true;

   CastParty.Debug( 2, 'Battlefield heal', CastParty.State.usingBattlefieldHeal)

   --***************************************************************************
   -- DONT TOUCH!!
   if CastParty.State.spell_being_cast and CastParty.State.spell_being_cast.name then
      spellName      = CastParty.State.spell_being_cast;
      spellNameCheck = CastParty.State.spell_being_cast.name;
   else
      spellNameCheck = spellName;
      spellName      = CastParty.Brain.best_spell_ids[spellNameCheck];
   end

   --this is so we can announce who is healing what!
   if not CastParty.State.usingBattlefieldHeal
   and CastPartyConfig.showMonitor
   and CastParty.Brain.heal_types then
      if CastParty.Brain.heal_types[spellNameCheck] then
         if not CastParty.State.spell_target_unit then
            if UnitExists('mouseover') or (GameTooltipTextLeft1 and GameTooltipTextLeft1:GetText() and string.find(GameTooltipTextLeft1:GetText(), UnitName('player'))) then
               CastParty.State.spell_target_unit = 'mouseover';
            elseif UnitExists('target') then
               CastParty.State.spell_target_unit = 'target';
            end
         end
         CastParty_SpeakSpellPsycic(CastParty.State.spell_target_unit, spellName, spellDuration);
      end
   end

   if CastPartyConfig.message_res and
      (spellNameCheck == CASTPARTY_RES_ANCESTRAL_SPIRIT or
       spellNameCheck == CASTPARTY_RES_RESURRECTION or
       spellNameCheck == CASTPARTY_RES_REBIRTH or
       spellNameCheck == CASTPARTY_RES_REDEMPTION) then
         if GameTooltipTextRight1:GetText() then
            local spellTextParse = string.gsub(GameTooltipTextRight1:GetText(),'%D', "");
            if not (CastParty.State.spell_target_unit and spellTextParse) then --and UnitIsDead(GameTooltipTextLeft1:GetText()) then
               CastParty.State.spell_target_unit = 'mouseover';
            end
         end

         
         CastParty_SpeakSpell(CastParty.State.spell_target_unit, CastParty.Brain.best_spell_ids['Resurrection'], spellDuration);
         return;
   end
   -- DONT TOUCH
   --***************************************************************************

   if not ( CastPartyConfig.message_spell_text and CastParty.State.spell_target_unit and CastParty.State.spell_being_cast ) then
      return;
   end

   if CastPartyConfig.message_non_heals and not CastParty.Brain.heal_types[CastParty.State.spell_being_cast.name] then
      CastParty_SpeakSpell(CastParty.State.spell_target_unit, CastParty.State.spell_being_cast, spellDuration);
      return;
   end

   local healthCurrent  = CP_UnitHealth(CastParty.State.spell_target_unit)
   local healthMax      = CP_UnitHealthMax(CastParty.State.spell_target_unit)
   CastParty.Debug('event', 'Starting Cast of', spellNameCheck, 'spellDuration', spellDuration )
   CT_RA_SpellTarget = CastParty.State.spell_target_unit

   if ( healthMax > 0
      and ( healthCurrent / healthMax ) <= CastPartyConfig.message_health_percent
      and UnitAffectingCombat(CastParty.State.spell_target_unit))
   or CastParty.State.spell_target_unit == 'party' then
      if (CastPartyConfig.message_heals and CastParty.Brain.heal_types[CastParty.State.spell_being_cast.name]) then
         CastParty_SpeakSpell(CastParty.State.spell_target_unit, CastParty.State.spell_being_cast, spellDuration);
      end
   end
end

--[[***************************************************************************]]--
function CastParty.Event.SPELLCAST_STOP(event, ...)
   CastParty.State.spell_target_unit    = nil;
   CastParty.State.spell_being_cast     = nil;
   CastParty.State.casting_now = false;
end

--[[***************************************************************************]]--
function CastParty.Event.SPELLCAST_FAILED(event, ...)
   CastParty.State.spell_target_unit    = nil;
   CastParty.State.spell_being_cast     = nil;
   CastParty.State.casting_now = false;
end

--[[***************************************************************************]]--
function CastParty.Event.SPELLCAST_INTERRUPTED(event, ...)
   -- were we trying to cast a healing spell?
   if CastParty.State.spell_being_cast then
      -- are we in combat?
      if UnitAffectingCombat('player')
      and CastParty.State.last_player_damage
      and GetTime() - CastParty.State.last_player_damage < CastPartyConfig.emergency_damage_window then
         CastParty_SpeakFailure(CastParty.State.spell_target_unit, 'INT');
      end
   end
   CastParty.State.spell_target_unit    = nil;
   CastParty.State.spell_being_cast     = nil;
   CastParty.State.casting_now = false;
end

--[[***************************************************************************]]--
function CastParty.Event.UI_ERROR_MESSAGE(event, ...)
   local msg = arg[1];

   -- range indicator stuff
   if not UnitExists(CastParty.State.spell_target_unit) and UnitExists('target') then
      CastParty.State.spell_target_unit = 'target';
   end
   if strfind(msg, ERR_OUT_OF_RANGE ) or string.find(msg, SPELL_FAILED_OUT_OF_RANGE ) then
      if CastParty_UpdateUnitPing then
         CastParty_UpdateUnitPing(2, CP_ids[CastParty.State.spell_target_unit]);
      end
   elseif string.find(msg, SPELL_FAILED_LINE_OF_SIGHT ) then
      if CastParty_UpdateUnitPing then
         CastParty_UpdateUnitPing(3, CP_ids[CastParty.State.spell_target_unit]);
      end
   end

   --autocast cancel stuff...
   if ( CastPartyConfig.stop_autocast and CastParty_auto_casting ) then
      if string.find(msg, ERR_SPELL_COOLDOWN) or string.ind(msg, SPELL_FAILED_ITEM_NOT_READY) then
         SpellStopCasting();
         if CastParty_Debug then CastParty_Debug( 3, 'Stoping autocasting') end;
         UIErrorsFrame:AddMessage('Autocasting has been interrupted!', 1.0, 0.1, 0.1, 1.0, 1);
      end
   end

   --cancel if we have no target
   if not CastParty.State.spell_target_unit then
      return;
   end

   --cancel if our options said to.
   if not CastPartyConfig.message_failures then
      return;
   end

   local healthCurrent  = CP_UnitHealth(CastParty.State.spell_target_unit);
   local healthMax      = CP_UnitHealthMax(CastParty.State.spell_target_unit);

   -- can the person still be healed?
   if not CastParty_UnitIsHealable(CastParty.State.spell_target_unit, healthCurrent, healthMax ) then
      if CastParty_Debug then CastParty_Debug( 2,  'Unit is not healable' ) end;
      return;
   end

   if (healthMax > 0 
      and (( healthCurrent / healthMax ) <= CastPartyConfig.message_failure_percent
      and UnitAffectingCombat(CastParty.State.spell_target_unit)))
   or UnitIsDead(CastParty.State.spell_target_unit) then
      -- are we out of range?
      if string.find(msg, ERR_OUT_OF_RANGE ) or strfind(msg, SPELL_FAILED_OUT_OF_RANGE ) then
         if CastParty_Debug then CastParty_Debug( 2,  'they are out of range' ) end;
         CastParty_SpeakFailure(CastParty.State.spell_target_unit, 'OOR');

      -- did we lose line of sight?
      elseif strfind(msg, SPELL_FAILED_LINE_OF_SIGHT ) then
         if CastParty_Debug then CastParty_Debug( 2,  'line of sight. ha, ha!' ) end;
         CastParty_SpeakFailure(CastParty.State.spell_target_unit, 'LOS');

      -- are we stunned?
      elseif strfind(msg, SPELL_FAILED_STUNNED ) then
         if CastParty_Debug then CastParty_Debug( 2,  'we are stunned' ) end;
         CastParty_SpeakFailure(CastParty.State.spell_target_unit, 'STU');

      elseif strfind(msg, SPELL_FAILED_SILENCED ) then
         if CastParty_Debug then CastParty_Debug( 2,  'you are silenced' ) end;
         CastParty_SpeakFailure(CastParty.State.spell_target_unit, 'SIL');
      end
   end
   if CastParty_Debug then CastParty_Debug( 1,  msg ) end;
end

--[[***************************************************************************]]--
function CastParty.Event.CHAT_MSG_CHANNEL(event, ...)
   if CastPartyConfig.showMonitor then
      CastPart_Channel_Parse(unpack(arg));
   end
end

--[[***************************************************************************]]--
function CastParty.Event.PARTY_MEMBERS_CHANGED(event, ...)
   --to is to solve some weird issues with the party frames mis-behaving.
   CastParty_ShowPartyFrame();
   CastParty_HidePartyFrame();
end

--[[***************************************************************************]]--
function CastParty.Event.UNIT_AURA(event, ...)
   --this was moved here, because this is necessary to increase the speed of healomatic.
   --if i'm in a raid, auras events are LAGGED because of blizz 'optimizations', so i MUST listen to PLAYER_AURAS_CHANGED (for me...)
   local id = CP_ids[arg[1]];

   if id and id ~= 0 then
      CastParty_UpdateAuras(id);
      --now display those auras!
      if CastParty_DisplayAuras then
         CastParty_DisplayAuras(id);
      end
   end
end

--[[***************************************************************************]]--
function CastParty.Event.UNIT_HEALTH(event, ...)
   -- make note of the last time the player took damage
   -- this is an overly simple way to see what the health trend is
   if arg[1] == 'player' then
      local healthCurrent = CP_UnitHealth('player');
      if not CastParty.State.last_player_health then
         CastParty.State.last_player_health = 0
      end
      if healthCurrent < CastParty.State.last_player_health then
         CastParty.State.last_player_damage = GetTime();
      end
      CastParty.State.last_player_health = healthCurrent;
   end
end

--[[***************************************************************************]]--
function CastParty.Event.OnLoad()
   DEFAULT_CHAT_FRAME:AddMessage(CASTPARTY_ONLOAD_MESSAGE)
   
   -- Hook into the addon itself to prevent garbage collection
   this.CastParty = CastParty;

   -- Options frame
   SLASH_CASTPARTY1 = '/cp'
   SLASH_CASTPARTY2 = '/castparty'
   SlashCmdList['CASTPARTY'] = CastParty.Event.OnSlashCmd
   
   CastParty.Event.Register()

   --for the onupdate stuff
   CastParty.State.lastUpdateTime = GetTime();

   -- hooks occur when VARIABLES_LOADED is called!!!

   local _, className = UnitClass('player');
   
   if CastParty.BrainFactory then
     CastParty.Brain = CastParty.BrainFactory.new(className)
     
     -- We have a brain, destroy the factory and the other unused brains
     CastParty.BrainFactory = nil;
   else
     note('CastParty has no brain factory /cry')
   end

   CastParty.Debug(1, '<CastParty> OnLoad', CastParty.Brain.className)
end

function CastParty.Event.Register()
   --needed for CP logic.
   this:RegisterEvent('PLAYER_ENTERING_WORLD');
   this:RegisterEvent('PLAYER_LEAVING_WORLD');

   this:RegisterEvent('PARTY_MEMBERS_CHANGED');    --party/raid member update stuff

   this:RegisterEvent('SPELLCAST_START');          -- for spell announcing
   this:RegisterEvent('SPELLCAST_STOP');           -- for clearing current spell info
   this:RegisterEvent('SPELLCAST_INTERRUPTED');    -- for annoucing of interruptions
   this:RegisterEvent('SPELLCAST_FAILED');         -- for clearing current spell info
   this:RegisterEvent('UI_ERROR_MESSAGE');         -- for reporting spell failures
   this:RegisterEvent('LEARNED_SPELL_IN_TAB');     -- to keep up with current spells
   this:RegisterEvent('START_AUTOREPEAT_SPELL')    -- for knowing about autocasting state
   this:RegisterEvent('STOP_AUTOREPEAT_SPELL')     -- for clearing autocasting state
   this:RegisterEvent('VARIABLES_LOADED');         -- required so we know when to load additional parts of CastParty
   this:RegisterEvent('ADDON_LOADED');             -- required to load the raid clicker buttons.
   this:RegisterEvent('CHAT_MSG_CHANNEL');         -- to monitor when we other people are healing someone one (psycic healing!).
   this:RegisterEvent('PLAYER_ENTER_COMBAT');      -- to monitor when we start attcking someone (melee)
   this:RegisterEvent('PLAYER_LEAVE_COMBAT');      -- to monitor when we stop attcking someone (melee)
   this:RegisterEvent('PLAYER_TARGET_CHANGED');    -- when i change a target (i.e... click...), this is the event fired!!!
end

--[[***************************************************************************]]--
function CastParty_HidePartyFrame()
   --note('hiding party frame')
   if CP_HidePartyFrame and ((CastPartyConfig and CastPartyConfig.hide_party_frame) or ((HIDE_PARTY_INTERFACE == '1') and (GetNumRaidMembers() > 0))) then
      CP_HidePartyFrame();
   elseif CP_HidePartyFrame and (not CastPartyConfig) then
      CP_HidePartyFrame();
   end
end

--[[***************************************************************************]]--
function CastParty_ShowPartyFrame()
   --note('showing party frame')
   if CP_ShowPartyFrame and ((CastPartyConfig and not CastPartyConfig.hide_party_frame))  then
      CP_ShowPartyFrame();
   elseif CP_ShowPartyFrame and (not CastPartyConfig) then
      CP_ShowPartyFrame();
   end
end

--[[***************************************************************************]]--
function CastParty_TargetFrame_Update()
   if CP_TargetFrame_Update and (CastPartyConfig and not CastPartyConfig.hide_target_frame)then
      CP_TargetFrame_Update();
   elseif CP_TargetFrame_Update and (not CastPartyConfig) then
      CP_TargetFrame_Update();
   else
      TargetFrame:Hide();
      TargetofTargetFrame:Hide();
   end
end

--[[***************************************************************************]]--
function CastParty_PlayerFrame_Update()
   if CP_PlayerFrame_Update and (CastPartyConfig and not CastPartyConfig.hide_player_frame) then
      PlayerFrame:Show();
      CP_PlayerFrame_Update();
   elseif CP_PlayerFrame_Update and (not CastPartyConfig) then
      PlayerFrame:Show();
      CP_PlayerFrame_Update();
   else
      PlayerFrame:Hide();
   end
end

--[[***************************************************************************]]--
function CastParty_ToggleGameMenu(clicked)
   if ( EzDismount_Toggle and EzDismount_Menu:IsVisible() ) then
      EzDismount_Menu:Hide()

   elseif ( CastPartyChangeOption and (CastPartyOptions:IsVisible() or CastPartyKeyBindingsFrame:IsVisible()) ) then
		CastPartyOptionsCloseButton:Click();
		CastPartyKeyBindingsFrameCloseButton:Click();

   else
      CP_ToggleGameMenu(clicked)
   end
end

--[[***************************************************************************]]--
function CastParty_StopSpell()

   if not CastParty.State.spell_target_unit then
      CastParty.State.spell_target_unit = 'target';
   end

   -- don't bother with health check until you push a button!
   if CastParty.State.casting_now
   and CastParty.State.spell_target_unit
   and CastParty.State.spell_being_cast
   and CastParty.State.spell_being_cast.name then
      -- only cancel approved spells
      if CastParty.Brain.heal_types[CastParty.State.spell_being_cast.name] and
         (CastParty.Brain.heal_types[CastParty.State.spell_being_cast.name] == 'efficient' or
          CastParty.Brain.heal_types[CastParty.State.spell_being_cast.name] == 'emergency') then
         -- check cast time
         if not CastParty.State.cast_time then
            CastParty.State.cast_time = 0
         end
         if CastParty.State.cast_time < GetTime() and CastParty.State.spell_being_cast.max_heal then
            local healthCurrent = CP_UnitHealth(CastParty.State.spell_target_unit);
            local healthMax     = CP_UnitHealthMax(CastParty.State.spell_target_unit);
            local healAmount    = healthMax - healthCurrent;
            if healthMax > 100 and healAmount / CastParty.State.spell_being_cast.max_heal < CastPartyConfig.heal_cancel_threshold then
               CastParty.Debug( 3, 'spell stopped', 'heal amount', healAmount, 'max heal', CastParty.State.spell_being_cast.max_heal)
               CastParty.Debug( 3, 'spell would have healed for', (healAmount / CastParty.State.spell_being_cast.max_heal), 'and is less then cancel threshold', CastPartyConfig.heal_cancel_threshold )
               SpellStopCasting();
            end
         end
      end
   end

end

--[[***************************************************************************]]--
function CastParty.Event.OnUpdate()
   local currentTime = GetTime();

   --timer to check the auto melee attack status every 1 seconds.
   if (currentTime > CastParty.State.lastUpdateTime) then
      if CastParty.State.wasMeleeAttack and not CastParty.State.isMeleeAttacking then
         AttackTarget();
         CastParty.State.wasMeleeAttack = false;
      end
      if CastParty.State.usingBattlefieldHeal then
         CastParty.State.usingBattlefieldHeal = false;
      end
      CastParty.State.lastUpdateTime = currentTime + 1;
   end
end
