--[[
This file deals with CastParty talking to other players in the party using CastParty through
a private comm channel.


\n    %n - unit name
\n    %h - current health\n    %Mh - max health\n    %Dh - health deficit\n    %Ph - health percent\n    %DPh - health deficit percent
\n    %m - current mana/rage/energy\n    %Mm - max m/r/e\n    %Dm - m/r/e deficit\n    %Pm - m/r/e percent\n    %DPm - m/r/e deficit percent
\n    %l - level\n    %L - don't show lvl 60
\n    %c - class\n    %C - class short name

Specify the format of the text on your target bar\n
%n - unit name\n    %m - max health\n    %h - current health\n    %H - health deficit\n
%p - health percent\n    %l - level\n    %L - don't show lvl 60\n
%c - class\n    %C - short class name

--] ]

-- This function was a start at a way to rework how formatting is done, but became too cumbersome.
-- Might end up doing it later, but for now it's commented. --pb
local function CastParty_LoadFormats()
   local formats = {}
   
   formats = {
      ['%n']  = { desc = 'unit name',
                  f = function(u) if not u then return 'Unknown' end return UnitName(u) or 'Unknown' end },
      ['%c']  = { desc = 'unit class',
                  f = function(u) if not u then return '??' end return UnitClass(u) or '??' end },
      ['%C']  = { desc = 'unit class short name',
                  f = function(u) if not u then return '??' end return UnitClass(u) or '??' end },
      ['%l']  = { desc = 'unit level',
                  f = function(u) if not u then return 0 end return UnitLevel(u) or 0 end },
      ['%L']  = { desc = "don't show level 60",
                  f = function(u) if not u then return 100 end return UnitManaMax(u) or 100 end },
      ['%Mm'] = { desc = 'max m/r/e',
                  f = function(u) if not u then return 100 end return UnitManaMax(u) or 100 end },
      ['%h']  = { desc = 'current health',
                  f = function(u) if not u then return 1 end return UnitHealth(u) or 1 end },
      ['%Mh'] = { desc = 'max health',
                  f = function(u) if not u then return 100 end return UnitHealthMax(u) or 100 end },
      ['%Dh'] = { desc = 'health deficit',
                  f = function(u)
                         return -1 * ( formats['%Mh'].f(u) - formats['%h'].f(u))
                      end },
      ['%Ph'] = { desc = 'health percent',
                  f = function(u)
                         local pct = formats['%h'].f(u) / formats['%Mh'].f(u)
                         return string.format('%d', pct * 100)..'@pct' -- string format to int will truncate
                      end },
      ['%DPh'] = { desc = 'health deficit percent',
                  f = function(u)
                         local pct = formats['%h'].f(u) / formats['%Mh'].f(u)
                         return string.format('%d', -100 * (1- pct))..'@pct' -- string format to int will truncate
                      end },
      ['%m']  = { desc = 'current mana/rage/energy',
                  f = function(u) if not u then return 1 end return UnitMana(u) or 1 end },
      ['%Mm'] = { desc = 'max m/r/e',
                  f = function(u) if not u then return 100 end return UnitManaMax(u) or 100 end },
      ['%Dm'] = { desc = 'm/r/e deficit',
                  f = function(u)
                         return -1 * ( formats['%Mm'].f(u) - formats['%m'].f(u))
                      end },
      ['%Pm'] = { desc = 'm/r/e percent',
                  f = function(u)
                         local pct = formats['%m'].f(u) / formats['%Mm'].f(u)
                         return string.format('%d', pct * 100)..'@pct' -- string format to int will truncate
                      end },
      ['%DPm'] = { desc = 'm/r/e deficit percent',
                  f = function(u)
                         local pct = formats['%m'].f(u) / formats['%Mm'].f(u)
                         return string.format('%d', -100 * (1- pct))..'@pct' -- string format to int will truncate
                      end },
   }
   
   return formats
end
--]]

function CastParty.FormatString( _msgTranslate, _unit, _supressColors )
   if _msgTranslate == nil then
      return nil
   end
   
   CastParty.Debug(2, GRN..'formatting str, unit', _unit)

   -- some data is not always ready when a player 1st joins a group, or high lag areas
   -- so we spoof the data unit it's ready...
   local unitName   = UnitName(_unit);
   local health     = CP_UnitHealth(_unit);
   local healthMax  = CP_UnitHealthMax(_unit);
   local healthFraction = health / healthMax;
   local healthPercent   = '0%';
   local healthPercent_d = '0%';
   local mana     = CP_UnitMana(_unit);
   local manaMax  = CP_UnitManaMax(_unit);
   local manaFraction = mana / manaMax;
   local manaPercent   = '0%';
   local manaPercent_d = '0%';
   local unitClass = UnitClass(_unit) or '??';
   local unitLevel = UnitLevel(_unit) or '0';

   if healthMax ~= 0 then
      healthPercent = string.format('%d', 100 * healthFraction)..'@pct' -- formatting to int truncates
      healthPercent_d = string.format('%d', -100 * (1 - healthFraction) )..'@pct'
   end

   if manaMax ~= 0 then
      manaPercent = string.format('%d', 100 * manaFraction)..'@pct'
      manaPercent_d = string.format('%d', -100 * (1 - manaFraction) )..'@pct'
   end

   --***************************************************************************
   -- DONT TOUCH!! ONLY DO THIS FOR REZZING!!!
   if CastParty.State.spell_being_cast 
      and CastParty.Brain.heal_types[CastParty.State.spell_being_cast.name] == 'resurrection'
      and (_unit == 'mouseover' or _unit == 'target')
      and GameTooltipTextLeft1:GetText()
   then
      unitName = string.gsub(GameTooltipTextLeft1:GetText(), CASTPARTY_CORPSEOF, "");
      if unitName then
         local index;
         for parsing in string.gfind(unitName, '%s') do
            index = string.find(unitName, ' ') or 0;
            unitName = string.sub(unitName, index+1, string.len(unitName));
         end
         if string.find(_msgTranslate, unitName) then
            unitName = UnitName(_unit);
         end
      end
   end

   if not unitName then
      unitName = 'Unknown';
   end
   -- DONT TOUCH
   --***************************************************************************

   --do the level color thing!
   if CastPartyConfig.color_level and not _supressColors then
      unitLevel = CastParty_GetDifficultyColor(unitLevel)..unitLevel..'|r';
   end

   --issues with mobs...
   if _unit == 'target' then
        if not UnitInGroup('target') then
         healthMax = CastParty.MobHealth.GetTargetMaxHP();
         if healthMax > 0 then
            health = CastParty.MobHealth.GetTargetCurHP();
         else
            _msgTranslate = string.gsub(_msgTranslate, '|', "");
            _msgTranslate = string.gsub(_msgTranslate, '%%h', "");
            _msgTranslate = string.gsub(_msgTranslate, '%%Mh', "" );
            _msgTranslate = string.gsub(_msgTranslate, '%%DPh', "" );
            _msgTranslate = string.gsub(_msgTranslate, '%%Ph', "" );
            _msgTranslate = string.gsub(_msgTranslate, '%%Dh', "" );
            _msgTranslate = string.gsub(_msgTranslate, '%%m', "");
            _msgTranslate = string.gsub(_msgTranslate, '%%Mm', "" );
            _msgTranslate = string.gsub(_msgTranslate, '%%DPm', "" );
            _msgTranslate = string.gsub(_msgTranslate, '%%Pm', "" );
            _msgTranslate = string.gsub(_msgTranslate, '%%Dm', "" );
            health = 0
         end
      end
   end

   _msgTranslate = string.gsub(_msgTranslate, '%%c', unitClass );

   if (unitClass == 'Warlock') then
      _msgTranslate = string.gsub(_msgTranslate, '%%C', 'Loc');
   else
      _msgTranslate = string.gsub(_msgTranslate, '%%C', string.sub(unitClass, 1, 3));
   end
   
   _msgTranslate = string.gsub(_msgTranslate, '%%NL', '\n')

   _msgTranslate = string.gsub(_msgTranslate, '%%h', health);
   _msgTranslate = string.gsub(_msgTranslate, '%%Dh', -1 * ( healthMax - health ) );
   _msgTranslate = string.gsub(_msgTranslate, '%%Mh', healthMax );
   _msgTranslate = string.gsub(_msgTranslate, '%%Ph', healthPercent );
   _msgTranslate = string.gsub(_msgTranslate, '%%DPh', healthPercent_d );
   
   local clr = nil
   if _unit and not _supressColors then
      local item = nil
      local ptype = UnitPowerType(_unit)
      if     ptype == 0 then item = CastPartyConfig.text_mana_rgb
      elseif ptype == 1 then item = CastPartyConfig.text_rage_rgb
      elseif ptype == 2 then item = CastPartyConfig.text_energy_rgb --focus
      elseif ptype == 3 then item = CastPartyConfig.text_energy_rgb --energy
      elseif ptype == 4 then item = CastPartyConfig.text_energy_rgb --happiness
      end
      
      if item then
         --local str = string.lower(string.format("%02X%02X%02X", item.r*255, item.g*255, item.b*255))
               --.. ' '..ptype..' '..item.r..' '.. item.g..' '.. item.b..' '..
         --CastParty.Debug(2, str..'|cff'..str.. 'using|r |cff20ff20abc|r')
         clr =  '|cff'..string.lower(string.format("%02X%02X%02X", item.r*255, item.g*255, item.b*255))
      end
      
      --CastParty.Debug(2, clr..'using color:|r')
   end
   
   if clr then
      _msgTranslate = string.gsub(_msgTranslate, '%%m', clr..tostring(mana)..'|r');
      _msgTranslate = string.gsub(_msgTranslate, '%%Dm', clr..tostring(-1 * ( manaMax - mana ))..'|r' );
      _msgTranslate = string.gsub(_msgTranslate, '%%Mm', clr..tostring(manaMax)..'|r' );
      _msgTranslate = string.gsub(_msgTranslate, '%%Pm', clr..tostring(manaPercent)..'|r' );
      _msgTranslate = string.gsub(_msgTranslate, '%%DPm', clr..tostring(manaPercent_d)..'|r' );
   else
      _msgTranslate = string.gsub(_msgTranslate, '%%m',   mana);
      _msgTranslate = string.gsub(_msgTranslate, '%%Dm',  -1 * ( manaMax - mana ));
      _msgTranslate = string.gsub(_msgTranslate, '%%Mm',  manaMax);
      _msgTranslate = string.gsub(_msgTranslate, '%%Pm',  manaPercent);
      _msgTranslate = string.gsub(_msgTranslate, '%%DPm', manaPercent_d);
   end
   
   _msgTranslate = string.gsub(_msgTranslate, '%%l', unitLevel );

   if (unitLevel == 60) then
      -- A.S. This is needed twice to take out the extra space if the user
      --     puts one in. Notice the diff between '%%L ' and '%%L'
      _msgTranslate = string.gsub(_msgTranslate, '%%L ', "" );
      _msgTranslate = string.gsub(_msgTranslate, '%%L', "" );
   else
      _msgTranslate = string.gsub(_msgTranslate, '%%L', unitLevel );
   end

   if _unit == 'party' then
      _msgTranslate = string.gsub(_msgTranslate, '%%n', CASTPARTY_PARTY );
   else
      if string.len(unitName) > 13 and _unit ~= 'mouseover' then
         unitName = string.sub(unitName, 1, 13) .. '..';
      end
      --do the name color thing!
      if CastPartyConfig.color_names and not _supressColors then
         local color = CastParty_classColor(_unit);
         unitName = '|cff'..CastParty_FloatToHex(color.r, color.g, color.b)..unitName..'|r';
      end
      _msgTranslate = string.gsub(_msgTranslate, '%%n', unitName );
   end

   -- custom messages
   if healthFraction > 0.9 then
      _msgTranslate = string.gsub(_msgTranslate, '%%t', CastParty_message_health_100);
   elseif healthFraction > 0.8 then
      _msgTranslate = string.gsub(_msgTranslate, '%%t', CastParty_message_health_90);
   elseif healthFraction > 0.7 then
      _msgTranslate = string.gsub(_msgTranslate, '%%t', CastParty_message_health_80);
   elseif healthFraction > 0.6 then
      _msgTranslate = string.gsub(_msgTranslate, '%%t', CastParty_message_health_70);
   elseif healthFraction > 0.5 then
      _msgTranslate = string.gsub(_msgTranslate, '%%t', CastParty_message_health_60);
   elseif healthFraction > 0.4 then
      _msgTranslate = string.gsub(_msgTranslate, '%%t', CastParty_message_health_50);
   elseif healthFraction > 0.3 then
      _msgTranslate = string.gsub(_msgTranslate, '%%t', CastParty_message_health_40);
   elseif healthFraction > 0.2 then
      _msgTranslate = string.gsub(_msgTranslate, '%%t', CastParty_message_health_30);
   elseif healthFraction > 0.1 then
      _msgTranslate = string.gsub(_msgTranslate, '%%t', CastParty_message_health_20);
   else
      _msgTranslate = string.gsub(_msgTranslate, '%%t', CastParty_message_health_10);
   end
   
   -- replace percent tag with the percent sign
   _msgTranslate = string.gsub(_msgTranslate, '@pct', '%%')

   return _msgTranslate

end

--***************************************************************************
--***************************************************************************
-- This function is used to announce spells, and is formated
-- according to the configuration found in PlayerConfig.lua
--
function CastParty_SpeakSpell(_unit, _spell, _duration)
   -- goofy nil errors possible??
   if not _spell then
     CastParty.Debug(3, 'Invalid arg to SpeakSpell: _spell', _spell)
     return
   end

   CastParty.Debug(3, 'Reached speak spell!!', _unit, _spell.name, _duration)

   -- for instant cast spells
   if not _duration then _duration = 0 end

   -- convert duration to sec
   _duration = string.format('%.1f', _duration / 1000)

   -- spell message found in Player Config
   local messageText = CastPartyConfig.message_spell_text

   -- format message accorrding to player config
   messageText = string.gsub(messageText, '%%s', _spell.name)
   messageText = string.gsub(messageText, '%%d', _duration)

   if _spell.rank then
      messageText = string.gsub(messageText, '%%r', _spell.rank)
   else
      messageText = string.gsub(messageText, '%%r', CASTPARTY_RANK + ' ?')
   end

   if _spell.max_heal and not UnitIsUnit('player', _unit) and UnitIsFriend('player', _unit) then
      messageText = string.gsub(messageText, '%%e', _spell.max_heal)
   else
      messageText = string.gsub(messageText, '%%e', '??')
   end

   messageText = CastParty.FormatString( messageText, _unit, true)
   CastParty_SendMessage(messageText, _unit)
end


--***************************************************************************
--***************************************************************************
-- This function is used to broadcast spells on the custom channel, so that
-- other players will know if/when a heal is happening. This is to provide
-- visual data for when heals occur.
--
function CastParty_SpeakSpellPsycic(_unit, _spell, _duration)

   if CastParty_Debug then CastParty_Debug(3, 'Reached speak spell psycic!!', _unit) end;

   local unitName
   local unitID
   
   -- goofy nil errors possible??
   if (_spell == nil or _unit == nil) then
      return;
   end

   -- for instant cast spells
   if not _duration then _duration = 0 end

   -- convert duration to sec
   _duration = string.format('%.1f', _duration / 1000)

   local ttinfo = GameTooltipTextLeft1:GetText();
   if _unit == 'mouseover' then
      unitName = UnitName(_unit);
      if not unitName and ttinfo then
         local index;
         for parsing in string.gfind(ttinfo, '%s') do
            index = string.find(ttinfo, ' ');
            unitName = string.sub(ttinfo, index+1, string.len(ttinfo));
         end
      end
      unitID = 91;
   else
      if _unit then
         unitName = UnitName(_unit);
      end
      if CP_ids[_unit] then
         unitID = CP_ids[_unit];
      else
         unitID = 91;
      end
   end

   if not unitName then
      unitName = '??';
   end


   local spellName
   local spellRank
   local spellAmount
   
   if type(_spell) == 'string' then
      spellName = _spell;
      if (GameTooltipTextRight1:GetText()) then
         spellRank = GameTooltipTextRight1:GetText();
      else
         spellRank = CASTPARTY_RANK .. ' ?';
      end
   else
      if _spell.name then
         spellName = _spell.name;
      end
      if _spell.rank then
         spellRank = _spell.rank;
      end
   end

   if type(_spell) == 'table' and _spell.max_heal and UnitIsFriend('player', _unit) then
      spellAmount = _spell.max_heal;
   else
      spellAmount = '??';
   end

   -- spell message
   CastParty_Channel_Comms(unitID .. '#' .. unitName .. '#' .. spellName .. '#' .. spellRank .. '#' .. _duration .. '#' .. spellAmount);
end


--***************************************************************************
--***************************************************************************
--
-- this function is used to address failures
--
function CastParty_SpeakFailure(_unit, _state)

   --cancel if our options said to.
   if not CastPartyConfig.message_failures then
      return;
   end

   local mana = UnitMana('player');
   local manaFraction = mana / UnitManaMax('player');

   -- do we have any mana, if not who cares
   if manaFraction < CastPartyConfig.message_disable_mana then
      return
   end

   -- can the person still be healed?
   local health    = CP_UnitHealth(_unit);
   local healthMax = CP_UnitHealthMax(_unit);

   if CastParty_UnitIsHealable(CastParty.State.spell_target_unit, health, healthMax) == false then
      return
   end

   --check to make sure we are under the health percent required
   if (( health / healthMax ) >= CastPartyConfig.message_failure_percent) then
      return
   end
   
   local messageText

   -- Out of range
   if _state == 'OOR' then
      messageText = CastParty_message_range

   -- Line of sight
   elseif _state == 'LOS' then
      messageText = CastParty_message_sight

   -- interrupted
   elseif _state == 'INT' then
      messageText = CastParty_message_interrupt

   -- healer under attack
   elseif _state == 'HUA' then
      messageText = CastParty_message_under_attack

   -- Stunned
   elseif _state == 'STU' then
      messageText = CastParty_message_stunned

   -- silenced
   elseif _state == 'SIL' then
      messageText = CastParty_message_silenced
   end

   --CastParty_Debug( 2, 'messageText', messageText )
   -- "" check for disabled messages
   if messageText == "" then
      return
   end

   messageText = CastParty.FormatString( messageText, _unit, true );
   CastParty_SendMessage(messageText, _unit);
end

--***************************************************************************
--***************************************************************************
--
-- Send the message to the chat channel
--
function CastParty_SendMessage(_messageText, _unit)

   local mana = UnitMana('player')
   local manaFraction = mana / UnitManaMax('player')

   -- is mana to low to care?
   -- don't spam if you ain't got the mana to heal
   if manaFraction < CastPartyConfig.message_disable_mana then
      return
   end

   --
   -- some tricky logic here
   --
   -- Raid:
   -- if your in a raid, tell the raid, but if the player is in your local
   -- party then only tell the party, in the player is not in the raid or
   -- the party then whisper.
   --
   -- Party:
   -- if the player is in a raid and not in your party, do nothing
   -- if the player is in your party, then spam away, or whisper(if not
   -- in the raid that is)
   --
   -- Self:
   -- Announce messages to your self only
   --
   -- Whisper:
   -- Announce messages to the trageted player only
   --
   -- Hope to add channel support in here in the future
   --


   --***************************************************************************
   -- DONT TOUCH!!
   local unitName;

   unitName = UnitName(_unit);


   if not unitName and UnitName('mouseover') and not UnitName('target') then
      _unit = 'mouseover';
   end

   if _unit == 'target' and not UnitIsFriend('player', 'target') and UnitIsFriend('player', 'mouseover') then
      _unit = 'mouseover';
   end

   if _unit == 'mouseover' and GameTooltipTextLeft1:GetText() then
      unitName = string.gsub(GameTooltipTextLeft1:GetText(), CASTPARTY_CORPSEOF, "");
      local index;
      for parsing in string.gfind(unitName, '%s') do
         index = string.find(unitName, ' ');
         unitName = string.sub(unitName, index+1, string.len(unitName));
      end
   end
   -- DONT TOUCH!!
   --***************************************************************************

   local isRaid
   local isParty
   local notRaid
   local notParty
   local doWhisper
   
   if GetNumRaidMembers() > 0 then
      isRaid = true;
   else
      isRaid = false;
   end

   if GetNumPartyMembers() > 0 then
      isParty = true;
   else
      isParty = false;
   end


   -- we are obviously NOT in a raid, but want to send it there.
   if (CastPartyConfig.message_channel == 'raid' and not isRaid) then
      notRaid = true;
   else
      notRaid = false;
   end

   -- we are obviously NOT in a party, but want to send it there.
   if (CastPartyConfig.message_channel == 'party' and not isParty) then
      notParty = true;
   else
      notParty = false;
   end


   -- if we're not really sure what it is... but it's a player of sorts.
   if (UnitIsPlayer(_unit) and not UnitIsUnit(_unit, 'player')) or _unit == 'mouseover' then
      doWhisper = true;
   else
      doWhisper = false;
   end


   if not unitName then
      unitName = UnitName('player');
   end

   if CastPartyConfig.message_channel == 'player' and DEFAULT_CHAT_FRAME then
      DEFAULT_CHAT_FRAME:AddMessage(RED.._messageText)

   elseif (CastPartyConfig.message_channel == 'whisper' or ((notParty and not isRaid) or (notRaid and not isParty))) then
      if (LOCALE_enUS) then
         _messageText = string.gsub(_messageText, unitName, 'you');
      end
      SendChatMessage(_messageText, 'WHISPER', nil, unitName)

   elseif ((CastPartyConfig.message_channel == 'party' and isParty) or notRaid) and
      (UnitInParty(_unit) or string.find(_unit, 'pet') or _unit == 'mouseover') then
      SendChatMessage(_messageText, 'PARTY')

   elseif (CastPartyConfig.message_channel == 'raid' and isRaid) and (UnitInRaid(_unit) or _unit == 'mouseover') then
      SendChatMessage(_messageText, 'RAID')

   end
end
