--[[
This file contains functions dealing with the GUI elements of CastPartyCore

--]]

function CastParty.GUI.Exists(_string)
   if getglobal(_string) then
      return true;
   end
   
   CastParty.Debug(4, RED.._string..' doesnt exist')
   return false;
end

-- this is called by the CastParty frame loader and when an option is changed in the Options GUI
function CastParty.GUI.Init()
   CastParty.Debug(4, RED..'gui.init called')
   local modLoaded
   local modReason
   if CastPartyConfig.showPlayer or CastPartyConfig.showParty or CastPartyConfig.showTarget then
      -- have the GUI load on demand!
      modLoaded, modReason = LoadAddOn('CastPartyGUI');
    	if (not modLoaded) then
    		note('<CastParty><GUI><Module load error> '..modReason);
    		return;
    	end
   end

   if CastParty_GUI_Init then
      CastParty_GUI_Init();
   end

   CastParty.Debug( 3, YEL..'reached CastParty.GUI.Init')

   if CastParty.State.BlizzRaidEventLoaded then
      CastParty.GUI.InitBlizzardRaidGUI()
   end
   
   CastParty.GUI.InitBlizzardFrameClicks()
   
   -- override click behavior for other known mods
   CastParty.GUI.InitOtherFrameClicks()

   --do the thing for the PlayerFrame. It's a little different than
   --the target frame, as blizz always assumes it's shown.

   CastParty_PlayerFrame_Update();

   --determine whether to hide the party frames or not.
   if CastPartyConfig.hide_party_frame then
      CastParty_HidePartyFrame();
   else
      CastParty_ShowPartyFrame();
   end
   
   CastParty.GUI.InitEventRegistration();
end

function CastParty.GUI.InitBlizzardRaidGUI()
   if CastPartyConfig.cp_blizz_clicks then
      CastParty.Debug( 3, YEL..'blizz raid custom clickery '..GRN..'enabled')

      RaidPulloutButton_OnClick     = CastParty.Event.OnClickByUnit;

      --raid tabby thing when you drag a group/class onto the game window
      for i = 1,12 do
         for j=1,5 do
            if (CastParty.GUI.Exists('RaidPullout' .. i .. 'Button' .. j .. 'ClearButton')) then
               getglobal('RaidPullout' .. i .. 'Button' .. j .. 'ClearButton'):RegisterForClicks('LeftButtonUp', 'RightButtonUp', 'MiddleButtonUp', 'Button4Up', 'Button5Up');
            end
            -- this is to remind me how to do this, should i want to (add clickery to the raid buff icons.) however, it's not currently possible, since
            -- they are a frame object, and have to be a button object in order to work.
         end
      end
   else
      CastParty.Debug( 3, YEL..'bliz raid custom clickery '..ORN..'disabled')
      RaidPulloutButton_OnClick     = CastParty.StoredHooks.RaidPulloutButton_OnClick;

      --raid tabby thing when you drag a group/class onto the game window
      for i = 1,12 do
         for j=1,5 do
            if (CastParty.GUI.Exists('RaidPullout' .. i .. 'Button' .. j .. 'ClearButton')) then
               getglobal('RaidPullout' .. i .. 'Button' .. j .. 'ClearButton'):RegisterForClicks('LeftButtonUp', 'RightButtonUp');
            end
         end
      end
   end

   if CastPartyConfig.cp_clicks then
      CastParty.Debug( 3, YEL..'CT_RA/CTmod custom clickery '..GRN..'enabled')
      CT_RA_CustomOnClickFunction   = CastParty.Event.OnClickByUnit;
      CT_RA_MemberFrame_OnClick     = CastParty.Event.OnClickByUnit;
      CT_AssistFrame_OnClick        = CastParty.Event.OnClickByUnit;

      --Player/MT Targets by CT_RA
      for i = 1,10 do
         if (CastParty.GUI.Exists('CT_RAPTGroupMember' .. i .. 'CastFrame')) then
            getglobal('CT_RAPTGroupMember' .. i .. 'CastFrame'):RegisterForClicks('LeftButtonUp', 'RightButtonUp', 'MiddleButtonUp', 'Button4Up', 'Button5Up');
         end
         if (CastParty.GUI.Exists('CT_RAMTGroupMember' .. i .. 'CastFrame')) then
            getglobal('CT_RAMTGroupMember' .. i .. 'CastFrame'):RegisterForClicks('LeftButtonUp', 'RightButtonUp', 'MiddleButtonUp', 'Button4Up', 'Button5Up');
         end
      end
   else
      CastParty.Debug( 3, YEL..'CT_RA/CTmod custom clickery '..ORN..'disabled')
      CT_RA_CustomOnClickFunction   = CastParty.StoredHooks.CT_RA_CustomOnClickFunction;
      CT_RA_MemberFrame_OnClick     = CastParty.StoredHooks.CT_RA_MemberFrame_OnClick;

      --Player/MT Targets by CT_RA
      for i = 1,10 do
         if (CastParty.GUI.Exists('CT_RAPTGroupMember' .. i .. 'CastFrame')) then
            getglobal('CT_RAPTGroupMember' .. i .. 'CastFrame'):RegisterForClicks('LeftButtonUp', 'RightButtonUp');
         end
         if (CastParty.GUI.Exists('CT_RAMTGroupMember' .. i .. 'CastFrame')) then
            getglobal('CT_RAMTGroupMember' .. i .. 'CastFrame'):RegisterForClicks('LeftButtonUp', 'RightButtonUp');
         end
      end
   end
end

function CastParty.GUI.InitBlizzardFrameClicks()
   local FrameActions = {}
   FrameActions.Party = {}
   FrameActions.Party.Debuffs = {}
   FrameActions.Target = {}
   FrameActions.Target.Buffs = {}
   FrameActions.Target.Debuffs = {}
   
   local LeftRight = { 'LeftButtonUp', 'RightButtonUp' }
   local AllButtons = { 'LeftButtonUp', 'RightButtonUp', 'MiddleButtonUp', 'Button4Up', 'Button5Up' }

   -- Define what goes in each field for when blizz frames have custom clicks enabled
   if CastPartyConfig.cp_blizz_clicks then
      FrameActions.Party.Debuffs['OnClick'] = CastParty_ClickAura
      FrameActions.Party.Debuffs['clicks'] = { 'LeftButtonUp' }
      FrameActions.FrameClicks = AllButtons
      FrameActions.BuffButtonClicks = LeftRight
      FrameActions.MemberButtonClicks = AllButtons
      FrameActions.PetButtonClicks = AllButtons
      FrameActions.Target.Debuffs['OnClick'] = CastParty_ClickAura
      FrameActions.Target.Debuffs['clicks'] = LeftRight
      FrameActions.Target.Buffs['OnClick'] = CastParty_ClickAura
      FrameActions.Target.Buffs['clicks'] = LeftRight
   else
      -- Custom clicks disabled, revert to normal blizzard click handling
      FrameActions.Party.Debuffs['OnClick'] = nil
      FrameActions.Party.Debuffs['clicks'] = { }
      FrameActions.FrameClicks = LeftRight
      FrameActions.BuffButtonClicks = { 'RightButtonUp' }
      FrameActions.MemberButtonClicks = LeftRight
      FrameActions.PetButtonClicks = LeftRight
      FrameActions.Target.Debuffs['OnClick'] = nil
      FrameActions.Target.Debuffs['clicks'] = { }
      FrameActions.Target.Buffs['OnClick'] = nil
      FrameActions.Target.Buffs['clicks'] = { }
   end
   
   --player buff hoooks
   for i = 0,23 do
      getglobal('BuffButton' .. i):RegisterForClicks(unpack(FrameActions.BuffButtonClicks));
   end
   
   -- party member frame DEBUFFS
   for i = 1,4 do
      for j = 1,4 do
         getglobal('PartyMemberFrame'..i..'Debuff'..j):SetScript('OnClick', FrameActions.Party.Debuffs['OnClick'] );
         getglobal('PartyMemberFrame'..i..'Debuff'..j):RegisterForClicks(unpack(FrameActions.Party.Debuffs['clicks']));
      end
   end

   -- target member frame BUFFS
   for i = 1,5 do
      getglobal('TargetFrameBuff' .. i):SetScript('OnClick', FrameActions.Target.Buffs['OnClick'] );
      getglobal('TargetFrameBuff' .. i):RegisterForClicks(unpack(FrameActions.Target.Buffs['clicks']));
   end

   for i = 1,16 do
      getglobal('TargetFrameDebuff' .. i):SetScript('OnClick', FrameActions.Target.Debuffs['OnClick'] );
      getglobal('TargetFrameDebuff' .. i):RegisterForClicks(unpack(FrameActions.Target.Debuffs['clicks']));
   end

   -- party and pet frame click registers
   for i = 1,4 do
      getglobal('PartyMemberFrame' .. i):RegisterForClicks(unpack(FrameActions.MemberButtonClicks));
      getglobal('PartyMemberFrame' .. i .. 'PetFrame'):RegisterForClicks(unpack(FrameActions.PetButtonClicks));
   end

   --blizz player/pet frame button hooks
   getglobal('PlayerFrame'):        RegisterForClicks(unpack(FrameActions.FrameClicks));
   getglobal('TargetFrame'):        RegisterForClicks(unpack(FrameActions.FrameClicks));
   getglobal('TargetofTargetFrame'):RegisterForClicks(unpack(FrameActions.FrameClicks));
   getglobal('PetFrame'):           RegisterForClicks(unpack(FrameActions.FrameClicks));
   
   if CastPartyConfig.cp_blizz_clicks then
      CastParty.Debug( 3, YEL..'custom clickery '..GRN..'enabled'..YEL..' for default frames')

      --blizzard default frames.
      TargetFrame_OnClick           = CastParty.Event.OnClickByUnit;
      TargetofTarget_OnClick        = CastParty.Event.OnClickByUnit;
      PlayerFrame_OnClick           = CastParty.Event.OnClickByUnit;
      PetFrame_OnClick              = CastParty.Event.OnClickByUnit;
      PartyMemberFrame_OnClick      = CastParty.Event.OnClickByUnit;
      PartyMemberPetFrame_OnClick   = CastParty.Event.OnClickByUnit;
      BuffButton_OnClick            = CastParty_ClickAura;
   else
      CastParty.Debug( 3, YEL..'custom clickery '..ORN..'disabled'..YEL..' for default frames')

      --blizzard default frames.
      TargetFrame_OnClick           = CP_TargetFrame_OnClick;
      TargetofTarget_OnClick        = CP_TargetofTarget_OnClick;
      PlayerFrame_OnClick           = CP_PlayerFrame_OnClick;
      PetFrame_OnClick              = CP_PetFrame_OnClick;
      PartyMemberFrame_OnClick      = CP_PartyMemberFrame_OnClick;
      PartyMemberPetFrame_OnClick   = CP_PartyMemberPetFrame_OnClick;
      BuffButton_OnClick            = CP_BuffButton_OnClick;
   end
end

function CastParty.GUI.InitOtherFrameClicks()
   --TODO: refactor similar to InitBlizzardFrameClicks
   local FrameEvents = {}

   if CastPartyConfig.cp_clicks then
      CastParty.Debug( 3, YEL..'custom clickery '..GRN..'enabled'..YEL..' for mod frames: '..BLU..' Squeaky, Nurfed')

      --other frame hooks :)
      Squeaky_CustomOnClickFunction    = CastParty.Event.OnClickByUnit;
      Nurfed_Unit_OnClick              = CastParty.Event.OnClickByUnit;

      --nurfed frames
      if Nurfed_Utility then
         local unit
         for _,unit in pairs(CP_units) do
            if CastParty.GUI.Exists('Nurfed_'..unit) then
               getglobal('Nurfed_'..unit):RegisterForClicks('LeftButtonUp', 'RightButtonUp', 'MiddleButtonUp', 'Button4Up', 'Button5Up');
               if CastParty.GUI.Exists('Nurfed_'..unit..'target') then
                  getglobal('Nurfed_'..unit..'target'):RegisterForClicks('LeftButtonUp', 'RightButtonUp', 'MiddleButtonUp', 'Button4Up', 'Button5Up');
               end
               if CastParty.GUI.Exists('Nurfed_'..unit..'target') then
                  getglobal('Nurfed_'..unit..'targettarget'):RegisterForClicks('LeftButtonUp', 'RightButtonUp', 'MiddleButtonUp', 'Button4Up', 'Button5Up');
               end
            end
         end
      end

      --CTmod buff aura things.
      if CT_BuffFrame_OnLoad then
         for i = 0,23 do
            if CastParty.GUI.Exists('CT_BuffButton'..i) then
               getglobal('CT_BuffButton'..i):SetScript('OnClick', CastParty_ClickAura );
               getglobal('CT_BuffButton' .. i):RegisterForClicks('LeftButtonUp', 'RightButtonUp');
            end
         end
      end

   else
      CastParty.Debug( 3, YEL..'custom clickery '..ORN..'disabled'..YEL..' for mod frames: '..BLU..' Squeaky, Nurfed')

      --other frame hooks :)
      Squeaky_CustomOnClickFunction    = CP_Squeaky_CustomOnClickFunction;
      Nurfed_Unit_OnClick              = CP_Nurfed_Unit_OnClick;

      --nurfed frames
      if Nurfed_Utility then
         local unit
         for _,unit in pairs(CP_units) do
            if CastParty.GUI.Exists('Nurfed_'..unit) then
               getglobal('Nurfed_'..unit):RegisterForClicks('LeftButtonUp', 'RightButtonUp');
               if CastParty.GUI.Exists('Nurfed_'..unit..'target') then
                  getglobal('Nurfed_'..unit..'target'):RegisterForClicks('LeftButtonUp', 'RightButtonUp');
               end
               if CastParty.GUI.Exists('Nurfed_'..unit..'target') then
                  getglobal('Nurfed_'..unit..'targettarget'):RegisterForClicks('LeftButtonUp', 'RightButtonUp');
               end
            end
         end
      end

      --CTmod buff aura things.
      if CT_BuffFrame_OnLoad then
         for i = 0,23 do
            if CastParty.GUI.Exists('CT_BuffButton'..i) then
               getglobal('CT_BuffButton'..i):SetScript('OnClick', CT_BuffButton_OnClick );
               getglobal('CT_BuffButton' .. i):RegisterForClicks('RightButtonUp');
            end
         end
      end
   end
end

function CastParty.GUI.InitEventRegistration()
   -- register/unregister events
   local PF = {
      'UNIT_LEVEL', 'UNIT_COMBAT', 'UNIT_SPELLMISS', 'UNIT_PVP_UPDATE', 'UNIT_MAXMANA',
      'UNIT_NAME_UPDATE', 'UNIT_PORTRAIT_UPDATE', 'UNIT_DISPLAYPOWER',
      'PLAYER_ENTERING_WORLD', 'PLAYER_ENTER_COMBAT', 'PLAYER_LEAVE_COMBAT',
      'PLAYER_UPDATE_RESTING', 'PLAYER_REGEN_DISABLED', 'PLAYER_REGEN_ENABLED',
      'PARTY_MEMBERS_CHANGED', 'PARTY_LEADER_CHANGED', 'PARTY_LOOT_METHOD_CHANGED',
   }
   local PFHB = { 'UNIT_HEALTH', 'UNIT_MAXHEALTH', }
   local PFMB = {
      'UNIT_MANA', 'UNIT_RAGE', 'UNIT_FOCUS', 'UNIT_ENERGY', 'UNIT_HAPPINESS',
      'UNIT_MAXMANA', 'UNIT_MAXRAGE', 'UNIT_MAXFOCUS', 'UNIT_MAXENERGY', 'UNIT_MAXHAPPINESS',
      'UNIT_DISPLAYPOWER',
   }
   local evt
   if CastPartyConfig.hide_player_frame then
      for _,evt in ipairs(PF)   do PlayerFrame:UnregisterEvent(evt);          end
      for _,evt in ipairs(PFHB) do PlayerFrameHealthBar:UnregisterEvent(evt); end
      for _,evt in ipairs(PFMB) do PlayerFrameManaBar:UnregisterEvent(evt);   end
   else
      for _,evt in ipairs(PF)   do PlayerFrame:RegisterEvent(evt);          end
      for _,evt in ipairs(PFHB) do PlayerFrameHealthBar:RegisterEvent(evt); end
      for _,evt in ipairs(PFMB) do PlayerFrameManaBar:RegisterEvent(evt);   end
   end

   --target frame update stuff!
   ClearTarget();
   local Tevents = {
      'PLAYER_TARGET_CHANGED', 'PLAYER_FLAGS_CHANGED',
      'UNIT_HEALTH', 'UNIT_LEVEL', 'UNIT_FACTION', 'UNIT_CLASSIFICATION_CHANGED', 'UNIT_AURA',
      'PARTY_MEMBERS_CHANGED',
   }
   local Cevents = {
      'PLAYER_TARGET_CHANGED', 'PLAYER_COMBO_POINTS',
   }
   if CastPartyConfig.hide_target_frame then
      TargetFrame:Hide();
      TargetofTargetFrame:Hide()
      for _, evt in ipairs(Tevents) do TargetFrame:UnregisterEvent(evt); end
      for _, evt in ipairs(Cevents) do ComboFrame:UnregisterEvent(evt);  end
   else
      for _, evt in ipairs(Tevents) do TargetFrame:RegisterEvent(evt); end
      for _, evt in ipairs(Cevents) do ComboFrame:RegisterEvent(evt);  end
   end
end