-- ------------------------------------------------------------------------------------
-- DEBUG LEVEL
-- The higher the number the less debug information is printed in chat
-- the 1st number listed in a debug print out, is the debug level of that print out
-- ------------------------------------------------------------------------------------
--
-- Currently, 3 and 4 are areas we are interested in
--            1 and 2 are informational
--
-- You can change this in the advanced tab of CastPartyOptions GUI
-- Don't set it here, the first call to CastParty.Debug will figure it out
--CastPartySave.DebugLevel = 5;



-- These are debug keys for debuggers. Comment out to disable debugging on the given key
-- Whatever the key equals will be printed as the key name before the message
CastParty.DebugEnabled = {
  ['healomatic'] = GRN..'healomatic: ',
  ['brain'] = GRY..'brain: ',
  ['event'] = ORN..'event: ',
}

--***************************************************************************
--***************************************************************************
-- prints the arguments to the chatframe if the first argument is greater
-- than or equal to the global CastParty_DebugLevel

function CastParty.Debug(...)
   local msg = "";
   -- set the level from the saved variable if not yet set
   if not CastParty.DebugLevel and CastPartySave and CastPartySave.DebugLevel then
      CastParty.DebugLevel = CastPartySave.DebugLevel
   end
   
   -- default to no debug output if neither is set
   if not CastParty.DebugLevel or CastParty.DebugLevel < 1 then
      CastParty.DebugLevel = 5
   end

   if type(arg[1]) == 'number' and arg[1] < CastParty.DebugLevel then
      return;
   elseif type(arg[1]) == 'string' then
      -- only print these if global debugging is on
      if CastParty.DebugLevel >= 5 then
         return
      end
      if not CastParty.DebugEnabled[arg[1]] then
         return
      end
      msg = CastParty.DebugEnabled[arg[1]]
      -- pop the first item from args
      table.remove(arg, 1)
   end

   local str;
   for k,v in ipairs(arg) do
      str = tostring(v);
      if (str) then
         msg = msg .. str .. " : ";
      else
         msg = msg .. "(nil)" .." : ";
      end
   end

   DEFAULT_CHAT_FRAME:AddMessage(msg);
end

-- TODO: removeme when this conversion is complete
CastParty_Debug = CastParty.Debug

function CastParty.DebugPrintSpells()
   note(YEL..'Healing spells recognized by Castparty.Brain');
   if CastParty.Brain.heal_data then
      for key1, value1 in CastParty.Brain.heal_data do
         note(RED..key1);
         for key2, value2 in value1 do
            note(WHT..'   ', value2.name, value2.rank, BLU.. '  Mana:', value2.mana, GRN..'  Heal Amt:', value2.max_heal);
         end
         note("  ");
      end
   end

   note(" ");
   note(YEL..'Curing spells recognized by Castparty.Brain');
   for key1, value1 in CastParty.Brain.curables do
      note(key1 .. ':' .. value1);
   end

   note(" ");
   note(YEL..'All spells recognized by Castparty.Brain (best_spell_ids)');
   for index=1, 200 do
      for key2, value2 in CastParty.Brain.best_spell_ids do
         if value2.id == index then
            note(GRY..value2.id .. ': ' ..WHT.. value2.name .. '  ' ..DRKBLU.. value2.rank);
         end
      end
   end

   -- [[
   note(" ");
   note(YEL..'All spells recognized by Castparty.Brain (spell_ids)');
   local index=1
   for each in CastParty.Brain.spell_ids do
      for key2, value2 in CastParty.Brain.spell_ids do
         if value2.id == index then
            note(GRY..'['..key2..']' .. value2.id .. ': ' ..WHT.. value2.name .. '  ' ..DRKBLU.. value2.rank);
         end
      end
      index = index + 1;
   end
   --]]

   -- [[
   note(" ");
   note(YEL..'Class buff spells recognized by Castparty.Brain');
   for key1, value1 in CastParty.Brain.class_buffs do
      note(WHT..key1);
   end
   --]]
end

function CastParty.DebugPrintIlls(unit)
   note(YEL.. 'Printing debuffs listed on unit: ', unit, '(', CastParty.State.ills, ')')
   local key
   local value
   for key, value in CastParty.State.ills[unit] do
      note(key .. ' : ' .. value);
   end
end

function CastParty.DebugPrintBindings()
   local tmp_array = {};
   local key
   local value
   local key2
   local value2
   for key, value in pairs(CastPartyConfig.key_bindings) do
      for key2, value2 in pairs(value) do
         table.insert(tmp_array, { key..' '..GRY..key2..WHT, value2 } );
      end
   end

   table.sort(tmp_array, function(a,b) return a[1] < b[1] end);

   note(" ");
   note(YEL..'Click key bindings recognized by Castparty');
   for _, value in ipairs(tmp_array) do
      note(value[1] .. ' : ' ..value[2]);
   end
end

function note(...)
   local msg = "";
   local str;
   for k,v in ipairs(arg) do
      str = tostring(v);
      if (str) then
         msg = msg .. str .. " ";
      else
         msg = msg .. "(nil)" .." ";
      end
   end

   --handy little function to help CastParty_Debugging.
   DEFAULT_CHAT_FRAME:AddMessage(tostring(msg))
end
