-- I would like to see a CastParty raid version then many functions will be
-- independent of other peoples UI's, from what I can see the only the UI's
-- should be calling our functions not using or inserting data now that Raid
-- data is avaiable.  We should manage our data interally by our own UI,
-- even if it is hidden if this is at all possable

-- globals in use but not defined here
--
-- CP_units             = need to update this table to include pets, this should reduce a vast
--                         number of pet errors and reduce the code size plus the repeated code

-- file locals

-- Flags used to help in spells section, by checking active spells
CastParty.State.heal_states = {
   [CASTPARTY_HEAL_REJUVENATION]           = false,
   [CASTPARTY_HEAL_REGROWTH]               = false,
   [CASTPARTY_HEAL_RENEW]                  = false,
   [CASTPARTY_DEBUFF_WS]                   = false,
   [CASTPARTY_BUFF_POWER_WORD_SHIELD]      = false,
   [CASTPARTY_BUFF_INNER_FIRE]             = false,
   [CASTPARTY_BUFF_NATURES_SWIFTNESS]      = false,
   [CASTPARTY_BUFF_BLESS_OF_PROTECTION]    = false,
   [CASTPARTY_BUFF_INNER_FOCUS]            = false,
   [CASTPARTY_BUFF_SPIRIT_OF_REDEMPTION]   = false,
   [CASTPARTY_CLEARCASTING]                = false,
   [CASTPARTY_BUFF_MANA_SHIELD]            = false,
   ['shields up']                          = false,
}

-- storage for bandage data
CastParty.Healomatic.bandages =
{
   { name = CASTPARTY_BAND_LINEN,           heal =   66 },
   { name = CASTPARTY_BAND_HEAVY_LINEN,     heal =  114 },
   { name = CASTPARTY_BAND_WOOL,            heal =  161 },
   { name = CASTPARTY_BAND_HEAVY_WOOL,      heal =  301 },
   { name = CASTPARTY_BAND_SILK,            heal =  400 },
   { name = CASTPARTY_BAND_HEAVY_SILK,      heal =  640 },
   { name = CASTPARTY_BAND_MAGEWEAVE,       heal =  800 },
   { name = CASTPARTY_BAND_HEAVY_MAGEWEAVE, heal = 1104 },
   { name = CASTPARTY_BAND_RUNECLOTH,       heal = 1360 },
   { name = CASTPARTY_BAND_HEAVY_RUNECLOTH, heal = 2000 },
}


--[[***************************************************************************
*
*                  M I S C E L L A N E O U S   F U N C T I O N S
*
***************************************************************************]]--


--[[***************************************************************************
*
*                  U N I T   I N F O R M A T I O N   T E S T S
*
***************************************************************************]]--

--***************************************************************************
--***************************************************************************
-- look for active states on target, need to do this, in case the targets not in the party
-- may move this to the aura system like ills are now, and could speed things up alittle bit
-- but will not happen until have the display code re-write
--    Nature's Swiftness, Paladin Shields, have already added several from the list
-- This could also be used in the future to check for stun or silance conditions
--    on the caster to be reposted verably. instead of watching spell failure/ineterruption
--    events?
--
-- Currently used by:
--    AdvancedDRUIDHeal
--    AdvancedPRIESTHeal
--    AdvancedPALADINHeal
--    AdvandedShamanHeal
--

function CastParty.Healomatic.PrintStates()
   CastParty.Debug(3, 'Printing healomatic states')
   local state
   local val
   for state,val in pairs(CastParty.State.heal_states) do
      CastParty.Debug(2, BLU.. state .. '::'..tostring(val));
   end
end

--***************************************************************************
--***************************************************************************
function CastParty.Healomatic.CheckStates(_unit, _healType)
   -- --------------------------------------------------------------------------------------
   -- needed function for future spell cool downs like nature's swiftness, and tranquility
   -- Paladin's shields, barkskin, and others i can't think of right now, im sure
   -- start, duration, enable = GetSpellCooldown(spellID, BOOKTYPE_SPELL);
   -- --------------------------------------------------------------------------------------
   -- start, duration, enbale = GetSpellCooldown(21, SpellBookFrame.bookType)
   --
   -- can then be used like
   --
   -- DPReady = ((GetTime()-start) > duration)
   -- if DPReady then CastSpellByName(name)
   --
   -- wondering about enable, ill have to try to test it
   -- hopefully it is somting like, either or works for me
   -- false = can be cast cooldown is not enabled = flase
   -- true  = still in cooldown, cant cast, cooldown is enabled = true
   -- also we would need two flags, one for can we use it and one for it is active now
   --
   --
   -- Since we scan the auras on UNIT_AURA, and save the data in tables, it is redundant to
   -- be rescanning all the data again. It behooves us to reuse as much data as possible
   -- --------------------------------------------------------------------------------------

   -- Force a res spell if they click a dead bar and the option is set
   if CastPartyConfig.resurrect_dead and UnitIsDeadOrGhost(_unit) then
      CastParty.Debug( 1, 'Overriding heal type ', _healType, ' with resurrection' )
      return 'resurrection'
   end
   
   --testing new code
   for k in pairs(CastParty.State.heal_states) do
      CastParty.State.heal_states[k] = false;
   end

   --if there are no auras on the target unit..
   if not CP_auras[_unit] then
      CastParty.Debug( 2, ORN..'CheckStates', 'early return', _unit, _healType )
      return _healType;
   end

   --
   -- Target Data
   --

   for k, v in  pairs(CP_auras[_unit]) do
      -- check for presence of debuffs on the target.
      if v.name then
         -- check for presence of buffs on the target.
         if v.name ~= CASTPARTY_BUFF_NATURES_SWIFTNESS and
            v.name ~= CASTPARTY_BUFF_INNER_FOCUS and
            v.name ~= CASTPARTY_BUFF_SPIRIT_OF_REDEMPTION and
            v.name ~= CASTPARTY_CLEARCASTING then
               CastParty.Debug( 2, 'Target has buff', v.name )
               CastParty.State.heal_states[v.name] = true;
         end
      end
   end

   --
   -- Cooldown checks on player to go here as well
   --
   -- cooldown checks will no longer go here, they need to go in class healing
   --NR: need to tweak this part.

   CastParty_Tooltip:SetOwner(UIParent, 'ANCHOR_NONE');

   local buffTexture;
   local buffIndex;
   local buffName;

   -- buffs for ME start at 0, not 1.
   for i = 0, CastPartyConfig.max_buffs do
      buffIndex   = GetPlayerBuff(i, 'HELPFUL');
      buffTexture = GetPlayerBuffTexture(buffIndex);
      if buffTexture then
         CastParty_Tooltip:SetPlayerBuff(i)
         buffName = CastParty_TooltipTextLeft1:GetText()

         if buffName == CASTPARTY_BUFF_NATURES_SWIFTNESS then
            CastParty.State.heal_states[CASTPARTY_BUFF_NATURES_SWIFTNESS] = true;
            -- because spell becomes an instant cast.
            _healType = 'efficient';
            CastParty.Debug( 2, 'found swiftness' )

         elseif buffName == CASTPARTY_BUFF_INNER_FOCUS then
            CastParty.State.heal_states[CASTPARTY_BUFF_INNER_FOCUS] = true;
            if _healType == "" then _healType = 'emergency' end;
            CastParty.Debug( 2, 'found inner focus' )

         elseif buffName == CASTPARTY_BUFF_SPIRIT_OF_REDEMPTION then
            CastParty.State.heal_states[CASTPARTY_BUFF_SPIRIT_OF_REDEMPTION] = true;
            if _healType == "" then _healType = 'emergency' end;
            CastParty.Debug( 2, 'found SOR' )

         elseif buffName == CASTPARTY_CLEARCASTING then
            CastParty.State.heal_states[CASTPARTY_CLEARCASTING] = true;
            if _healType == "" then _healType = 'emergency' end;
            CastParty.Debug( 2, 'found clearcasting' )
         end
         CastParty.Debug( 1, GRY..'fancy player Buff', buffName )
      end
   end

   if CastParty.State.heal_states[CASTPARTY_BUFF_POWER_WORD_SHIELD] 
   or CastParty.State.heal_states[CASTPARTY_BUFF_BLESS_OF_PROTECTION]
   or CastParty.State.heal_states[CASTPARTY_BUFF_MANA_SHIELD] then
      CastParty.State.heal_states['shields up'] = true;
      CastParty.Debug( 1, GRY..'Shields are up' )
      if CastPartyConfig.shield_detection and
         _healType ~= 'efficient' and
         _healType ~= 'emergency' and
         _healType ~= 'instant'
      then
         --pop up an error message on the screen, for feedback purposes!!!
         UIErrorsFrame:AddMessage(CASTPARTY_SHIELD_HEAL, 1.0, 0.1, 0.1, 1.0, UIERRORS_HOLD_TIME);
         _healType = 'efficient';
         CastParty.Debug( 2, GRY..'Shield forced efficient' )
      end
   end
   CastParty.Debug( 2, ORN..'CheckStates', 'healType (or changed healType)', _unit, _healType )
   return _healType;
end


--[[***************************************************************************
*
*                       D E B U F F   F U N C T I O N S
*
***************************************************************************]]--


--***************************************************************************
--***************************************************************************
-- Called when a debuff of someone in the raid is removed/cured
--
-- Currently used by:
--    CastParty.Healomatic.CureAny
--    CastParty_ClickAura
--
function CastParty.Healomatic.RemoveDebuff(_unit, _debuffType)
   CastParty.Debug(1, 'RemoveDebuff', _unit, _debuffType)
   local k,v
   for k, v in pairs(CastParty.State.ills[_unit]) do
      if v == CastParty.Brain.curables[_debuffType] then
         table.remove(CastParty.State.ills[_unit], k);
         break;
      end
   end
end


--***************************************************************************
--***************************************************************************
-- Currently used by:
--    CastParty_ClickAura
--    CastParty.Healomatic.CureAny
--

-- TODO: need to modify this function to take into account the number of debuff applications
-- on the target, and cast the appropriate spell.
--
-- Returns whether the debuff was cured
--
function CastParty.Healomatic.CureDebuff(_unit, _nextCure)
   if not _nextCure then
      return false
   end
   
   CastParty.State.spell_being_cast = CastParty.Brain.best_spell_ids[_nextCure];

   CastParty_Retarget(_unit);

   CastSpell(CastParty.State.spell_being_cast.id, SpellBookFrame.bookType);
   SpellTargetUnit(_unit);

   CastParty_Retarget('SET');

   return true;
end


--***************************************************************************
--***************************************************************************
-- Currently used by:
--    CastParty.Healomatic.CureGroupMember
--
function CastParty.Healomatic.CureAny(_unit)
   if not CastParty.State.ills[_unit] then
      return;
   end

   local nextCure = CastParty.State.ills[_unit][1];  -- this is the type of debuff we have, i.e. Magic, Curse, etc.

   if CastParty.Healomatic.CureDebuff(_unit, nextCure) then
      CastParty.Healomatic.RemoveDebuff(_unit, nextCure);
   end
end


--***************************************************************************
--***************************************************************************
-- Currently used by:
--    called from config
--    called from options
--
function CastParty.Healomatic.CureGroupMember()
   local frameName, index, unit
   for index = 0, 89 do
      frameName = getglobal('CastPartyMember'..index);
      unit = CP_units[index];
      if UnitExists(unit)
      and CastParty.Healomatic.CureAny(unit)
      and (frameName.range and frameName.range < 50) then
         return true;
      end
   end

   return false;
end

function CastParty.Healomatic.DispellTarget(cureType)
   if CastParty.Brain.curables[cureType] then
      CastParty.Healomatic.CureDebuff('target', CastParty.Brain.curables[cureType]);
   end
end


--[[***************************************************************************
*
*                      A P P L Y H E A L   F U N C T I O N S
*
***************************************************************************]]--

--***************************************************************************
--***************************************************************************
-- ----------------
-- Bandage function
-- ----------------

--
-- Currently used by:
--    Config file setting
--    CastPartyOptions setting
--
function CastParty.Healomatic.ApplyBandage(_unit)
   local index
   local value
   local bagpos;
   local item;
   
   -- do we need to check for in party/raid, so we can interpolate health if needed?
   local health       = CP_UnitHealth(_unit);
   local targetHealth = CP_UnitHealthMax(_unit) * CastPartyConfig.bandage_threshold;
   
   if health >= targetHealth then
      CastParty.Debug('healomatic', "not healing with bandage, actual health:"..health.." target health:"..targetHealth..
                      ' bandage threshhold:'..CastPartyConfig.bandage_threshold)
      return
   end

   CastParty.Debug('healomatic', "healing with bandage")

   local bandage = {};

   -- find a bandage
   for index, value in pairs(CastParty.Healomatic.bandages) do
      bagpos, item  = CastParty.GetContainerItem(value.name);
      if bagpos and item then
         table.insert( bandage, { heal = value.heal,
                                  name = value.name,
                                  bag = bagpos,
                                  slot = item, }
                     );

         CastParty.Debug( 'healomatic', 'Found bandage', value.name, bag, slot )
      end
   end

   -- should be in-order, may not need this
   -- table.sort(bandage, function(a,b) return a.heal < b.heal end)

   -- find the correct bandage
   -- this will do for now, but needs to be revamped
   local compareBandageAmmount = 0;
   local selectBandage         = nil;

   for index, value in pairs(bandage) do
      -- make sure were using the best one we can
      if health + value.heal > compareBandageAmmount then
         compareBandageAmmount = health + value.heal;
         selectBandage         = index;
      end

      -- no reason to over heal, so lets break early
      if compareBandageAmmount > targetHealth then
         break;
      end
   end

   -- did we find one?
   if selectBandage then

      -- event handler stuff for canceling spells, and spell interruptions, etc
      CastParty.State.spell_target_unit = _unit;
      CastParty.State.spell_being_cast  = bandage[selectBandage];

      CastParty.Debug('healomatic', 'Using', bandage[selectBandage].name, bandage[selectBandage].heal)

      CastParty_Retarget(_unit);

      UseContainerItem( bandage[selectBandage].bag, bandage[selectBandage].slot );
      SpellTargetUnit(_unit);

      CastParty_Retarget('SET');
   end
end

-- This is a local-access global-scope funnel function to display an error message for all heal types
local function CastParty_DoApplyHeal(unit, typename)
   CastParty.Debug(3, 'applying '..typename..' heal')

   if not unit or not typename then
      CastParty.Debug(3, 'applying heal failed', tostring(unit), tostring(typename))
      return
   end
   
   if not CastParty.Brain.heal_data[typename] then
      note(RED.. typename.." type spell is not present or not loaded. If you are supposed to be"..
           " able to cast this type of spell, please report this to the CastParty forums at "..
           "http://sourceforge.net/projects/wowcastparty/");
      note(YEL.."Also, please type " .. GRY .. "/script CastParty.DebugPrintSpells()" .. YEL ..
           " to check to see what spells CastParty can recognize. This list should have your "..
           "spell type listed...");
      return;
   end

   CastParty.Healomatic.ApplyHeal(unit, typename);
end

--***************************************************************************
-- These functions are called by keybindings
--

function CastParty.Healomatic.ApplyEfficientHeal(_unit)
   CastParty_DoApplyHeal(_unit, 'efficient')
end

function CastParty.Healomatic.ApplyEmergencyHeal(_unit)
   CastParty_DoApplyHeal(_unit, 'emergency')
end

function CastParty.Healomatic.ApplyInstantHeal(_unit)
   CastParty_DoApplyHeal(_unit, 'instant')
end

function CastParty.Healomatic.ApplyShield(_unit)
   CastParty_DoApplyHeal(_unit, 'shield')
end

function CastParty.Healomatic.ApplyGroupHeal(_unit)
   CastParty_DoApplyHeal(_unit, 'group')
end

function CastParty.Healomatic.ApplyResurrectionHeal(_unit)
   CastParty_DoApplyHeal(_unit, 'resurrection')
end


--***************************************************************************
--***************************************************************************
-- casts natures swiftness, THEN does the heal thing, for druids and shamans.
function CastParty.Healomatic.Panic(_unit)
   --as suggested by renbear.
   if CastParty.Brain.className == 'DRUID' or CastParty.Brain.className == 'SHAMAN' then
      SpellStopCasting();
      CastSpellByName(CASTPARTY_BUFF_NATURES_SWIFTNESS);
      SpellStopCasting();
      CastParty.Healomatic.ApplyEfficientHeal(_unit);
   else
      SpellStopCasting();
      CastParty.Healomatic.ApplyEmergencyHeal(_unit);
   end
end

--***************************************************************************
--***************************************************************************
-- casts natures swiftness, THEN does the heal thing, for druids and shamans.
-- as suggested by Poison Ivy
--TODO: nobody is actually calling this!!
function CastParty.Healomatic.PanicAdvanced(_unit)
	z=0;
	x=0;
	for i = 1, CastPartyConfig.max_buffs do
		t = UnitBuff(_unit, i)
		if (t and string.find(t,"Rejuvenation")) or (t and string.find(t,"ResistNature")) then
			z=1
		end
	end

   --TODO: fix with brain interaction
   local swiftMend = CastParty.Brain.best_spell_ids[CASTPARTY_BUFF_SWIFTMEND]
   local swiftNess = CastParty.Brain.best_spell_ids[CASTPARTY_BUFF_NATURES_SWIFTNESS]
	local start, duration = GetSpellCooldown(swiftMend.id ,BOOKTYPE_SPELL);  --swiftmend
	DEFAULT_CHAT_FRAME:AddMessage( 'start=' .. start .. ' duration=' .. duration )

   local startswiftness, durationswiftness = GetSpellCooldown(swiftNess.id ,BOOKTYPE_SPELL);  --natures swiftness
   DEFAULT_CHAT_FRAME:AddMessage( 'start=' .. startswiftness .. ' duration=' .. startswiftness )

   seconds = GetTime();
   DEFAULT_CHAT_FRAME:AddMessage( 'Current system uptime is: ' .. seconds )

   difference = seconds - startswiftness;
   DEFAULT_CHAT_FRAME:AddMessage( 'difference= ' .. difference )

	SpellStopCasting()
	if z==1 and duration == 0 then
      TargetUnit(_unit)
      CastSpellByName(CASTPARTY_BUFF_SWIFTMEND)

   elseif startswiftness == 0 then
      CastSpellByName(CASTPARTY_BUFF_NATURES_SWIFTNESS)
      SpellStopCasting()
      CastParty.Healomatic.ApplyEfficientHeal(_unit)

   elseif difference < 0.003 then
      CastParty.Healomatic.ApplyEfficientHeal(_unit)

   else
      CastSpellByName(CASTPARTY_HEAL_REGROWTH)
   end
end

--***************************************************************************
--***************************************************************************
-- This spell currently does not take into account druid cooldown for tranquility
-- That feature should be added at a later date to account for this, Druids
-- have special proccessing until extra checks can be added.  Also unsure about
-- shamans... do they have a cooldown?
--
-- Currently used by:
--    CastParty.Healomatic.DoTheRightThing
--
function TestGroupHeal()

   local group_heal_name
   local best_group_heal
   
   if CastParty.Brain then
     -- be sure the player has group heals
     if not ( CastParty.Brain.heal_data and CastParty.Brain.heal_data['group'] ) then
        return false;
     end
  
      group_heal_name = CastParty.Brain.heal_data['group'][1].name;
      best_group_heal = CastParty.Brain.best_spell_ids[group_heal_name];
   end

   if CastParty.Brain.className == 'DRUID' then
      -- Druid is a heal over time with 5 ticks, but is channeling
      -- assume we can only cast the spell for 1/2 the time
      best_group_heal.max_heal = best_group_heal.max_heal * 2.5;
      -- best_group_heal.max_heal = best_group_heal.max_heal
   end

   CastParty.State.healing_base = 0;
   if (BonusScanner and BonusScanner.active) then
      CastParty.State.healing_base = BonusScanner:GetBonus('HEAL');
   end

   best_group_heal.max_heal = best_group_heal.max_heal +
      (CastParty.State.healing_base * CastParty.Brain.healing_bonus_multipliers[group_heal_name][table.getn(CastParty.Brain.heal_data['group'])] );


   -- unsure if this if statment is needed, is this not another spell check?
   -- we just checked to see if we have group healing spells, should this
   -- not alway be returning true?
   if best_group_heal then

      -- see how much we could possible heal the party, group heal only works for
      -- the party, no reason to scan the raid group.  Don't count pets
      local total_heal = 0;
      for index = 0, 4 do
         if CastParty_UnitIsHealable(CP_units[index]) then
            -- range check maybe required to go here as well, this would give
            -- more accurate number. Unfortunatly I have no idea how to deal
            -- with Shamans Chain Heal bases off of a target?
            -- it may be possable by using:  also testing for range to target
            -- often doesn't work very well, different maps use different offsets
            -- as well as the feature does not work in instances, may need
            -- people to select a slot for the action test, for this function
            -- to work properly
            --
            total_heal = total_heal + math.min( CP_UnitHealthMax(CP_units[index]) - CP_UnitHealth(CP_units[index]), best_group_heal.max_heal );
         end
      end

      local efficient_heal_name
      local best_efficient_heal

      efficient_heal_name = CastParty.Brain.heal_data['efficient'][table.getn(CastParty.Brain.heal_data['efficient']) ].name;
      best_efficient_heal = CastParty.Brain.best_spell_ids[efficient_heal_name];

      if best_efficient_heal then
         best_efficient_heal.max_heal = best_efficient_heal.max_heal +
            (CastParty.State.healing_base * CastParty.Brain.healing_bonus_multipliers[group_heal_name][table.getn(CastParty.Brain.heal_data['group'])] );

         local group_heal_ratio     = total_heal / best_group_heal.mana;
         local efficient_heal_ratio = best_efficient_heal.max_heal / best_efficient_heal.mana;

         -- will lower debug count after testing
         if debug then debug( 4, 'Group total', total_heal, 'Group mana', best_group_heal.mana, 'Group ratio', group_heal_ratio) end;
         if debug then debug( 4, 'Single total', best_efficient_heal.max_heal, 'Single mana', best_efficient_heal.mana, 'Single ratio', efficient_heal_ratio) end;

         if group_heal_ratio >= efficient_heal_ratio * CastPartyConfig.auto_group_threshold then
            return best_group_heal;
         end
      end
   end

   return false;

end


--***************************************************************************
--***************************************************************************
-- key-bindable function that heals a party member based on percent health
--
-- Currently used by:
--    Keybindings
--
function CastParty.Healomatic.DoTheRightThing(_unit)

    -- is a friendly player targeted, if so heal them
   if _unit and not UnitIsFriend(_unit, 'player') then
      _unit = nil;
   end


   if _unit and UnitExists(_unit) then
      -- are we a healer?
      if not CastParty.Brain.heal_data then
         return;
      end

      -- there is no reason to test everthing if were a hunter
      if CastParty.Brain.className == 'HUNTER' then
         CastParty.Healomatic.ApplyHeal(_unit);
         return;
      end

      -- is a friendly player targeted, if so heal them
      if UnitIsUnit(_unit, 'target') then
         CastParty.Healomatic.ApplyHeal(_unit);
         return;
      end

      -- test group healing
      local group_heal_spell = TestGroupHeal()

      if group_heal_spell then
         CastParty.State.spell_target_unit  = 'party';
         CastParty.State.spell_being_cast   = group_heal_spell;
         CastParty.TriggerTrinket(group_heal_spell.name);
         CastSpell(group_heal_spell.id, SpellBookFrame.bookType);

         SpellTargetUnit(_unit);

         CastParty_TargetingClear();
         return
      end
   else
      -- Otherwise heal the party member with the lowest % health
      local group_status = {};
      local frame;

      -- check all members including raid members and pets but CP_ids will need changes
      -- like the changes to CastParty.State.ills
      for index = 0, 89 do

         CastParty.Debug(1, i, CP_units[index])

         unitHealth     = CP_UnitHealth(CP_units[index]);
         unitHealthMax  = CP_UnitHealthMax(CP_units[index]);
         frame          = getglobal('CastPartyMember' .. index);

         if CastParty_UnitIsHealable(CP_units[index]) and
            (frame and frame.range and frame.range < 50) then
            -- pet checks, some would rather not deal with pets, or consider
            -- pets of less or more value, check to see if were checking for
            -- pets.  pet data starts at the 45 entry in the new combined
            -- tables
   --         TargetUnit( CP_units[index] )
   --         if IsActionInRange(59) then
            if index >= 45 then
               unitHealth = unitHealth * CastPartyConfig.pet_health_value;
               if unitHealth >= unitHealthMax then
                  unitHealth = unitHealthMax;
               end
            end

            table.insert(group_status, { name = CP_units[index],
                                       health = unitHealth / unitHealthMax,
                                      current = unitHealth,
                                      maximum = unitHealthMax })
   --      else
   --         CastParty_Debug( 4, CP_units[index], 'is not in range' )
   --      end
         end
      end

      if not group_status then return end;

      table.sort(group_status, function(a,b) return a.health < b.health end);

      for i,j in pairs(group_status) do

         CastParty.Debug( 1, 'healing: ' .. j.name)

         -- this section of code is also used in CastParty_Apply heal
         -- may need to move this into a new function, to reduce
         -- repeated code
         if CastPartyConfig.advanced_spells then
            local tempFunction = CastParty.Brain.AdvancedHeal
            if tempFunction ~= nil and type(tempFunction) == 'function' then
               tempFunction(j.name, j.current, j.maximum );
            else
               CastParty.Healomatic.DefaultHeal(j.name, j.current, j.maximum );
            end
         else
            CastParty.Healomatic.DefaultHeal(j.name, j.current, j.maximum );
         end

         -- make sure we found a spell, if not we need to cycle to the next person
         -- in the list, until we run out of people, to try and heal
         if spell then
            CastParty.Debug(1, 'Spell chosen', spell.name)
            CastParty.State.spell_target_unit = j.name;
            CastParty.State.spell_being_cast = spell;

            CastParty_Retarget(j.name);
            CastParty.TriggerTrinket(spell.name);
            CastSpell(spell.id, SpellBookFrame.bookType);
            CastParty_Retarget('SET');

            if SpellCanTargetUnit(j.name) then
               SpellTargetUnit(j.name);
               break;
            else
               CastParty.Debug( 1, '<CastParty><DoTheRightThing> ERROR: Invalid Target' .. j.name)
               SpellStopCasting();
            end
         end
      end

      CastParty_TargetingClear();
   end
end



--***************************************************************************
--***************************************************************************
function CastParty.Healomatic.InitHealVars(_unit, _unitHealth, _unitHealthMax, _healType)
   local playerMana;
   local playerManaFraction;

   if DruidBarKey and (CastParty.Brain.className == 'DRUID') then
      playerMana         = DruidBarKey.keepthemana;
      playerManaFraction = playerMana / DruidBarKey.maxmana;
   else
      playerMana         = UnitMana('player');
      playerManaFraction = playerMana / UnitManaMax('player');
   end

   local unitLevel          = UnitLevel(_unit);
   local unitHealthModified = _unitHealthMax * CastPartyConfig.spell_choice_threshold;
   local unitHealthFraction = _unitHealth / unitHealthModified;

   CastParty.Debug( 'healomatic', YEL..'InitHealVars', GRY..'Player mana', playerMana, 'Player mana fraction', playerManaFraction)
   CastParty.Debug( 'healomatic', YEL..'InitHealVars', GRY..'unit level', unitLevel, 'unit health MODIFIED', unitHealthModified, 'unit health modified fraction', unitHealthFraction)

   return CastParty.Healomatic.CheckStates(_unit, _healType), playerMana, playerManaFraction, unitHealthModified, unitHealthFraction, unitLevel;
end

--***************************************************************************
--***************************************************************************
function CastParty.Healomatic.DoTheShield(_unit, _unitHealth, _unitHealthMax, _unitHealthFraction, _unitLevel, _mana, _unit, _healType)

   if CastParty.State.usingBattlefieldHeal then
      return nil, _healType;
   end

   local spell;

   -- check for a critical condtion and in combat, if so do something about it if we can
   if ( CastPartyConfig.shield_if_critical and _unitHealthFraction < CastPartyConfig.critical_health_percent and UnitAffectingCombat(_unit) and (_unit == 'player' or UnitInGroup(_unit)) )
       or _healType == 'shield' then
      if not CastParty.State.heal_states[CASTPARTY_DEBUFF_WS] and CastParty.Brain.heal_data['shield'] and not CastParty.State.heal_states['shields up'] then
         spell = GetSpell( 0, 0, 0, 'shield', _unitLevel, _mana, _unit, true );
      end
      if spell then
         if _unitHealthMax > 0 and _unitHealthFraction <= CastPartyConfig.message_health_percent and _unitHealth > 0 and CastPartyConfig.message_heals then
            CastParty_SpeakSpell(_unit, spell, 0);
         end
         CastParty.Debug( 2, YEL..'DoTheShield', 'should cast the shield')
         return spell, "";
      end

      -- this means we didn't cast a shield...
      CastParty.Debug( 2, YEL..'DoTheShield', 'cant cast the shield')
      return nil, "";
   end
   return nil, _healType;
end

--***************************************************************************
--***************************************************************************
function CastParty.Healomatic.DoInstantHeal( _healType, _unitHealth, _unitHealthModified, _unitHealthFraction, _unitHealthMax, _unitLevel, _mana, _unit)

   CastParty.Debug( 2, YEL..'DoInstantHeal', GRY..'heal type', _healType, CastParty.State.heal_states[_healType])

   local spell;

   -- check the target and see if the spell is already on them
   -- if so another casting of this spell only waste mana, skip
   -- it and move to the next spell
   --note: rejuv AND renew can be on the target at the same time.
   if not CastParty.State.heal_states[_healType] and CastParty.Brain.heal_data['instant'] then
      spell = GetSpell( _unitHealth, _unitHealthModified, _unitHealthMax, 'instant', _unitLevel, _mana, _unit, CastPartyConfig.max_rank_hot );
   end
   -- speak spell is normal triggerd when a spell starts casting
   -- but in this case, its an instant spell, so there is no event from what i can gather
   if spell then
      if _unitHealthMax > 0 and _unitHealthFraction <= CastPartyConfig.message_health_percent and CastPartyConfig.message_heals then
         CastParty_SpeakSpell(_unit, spell, 0);
      end
      CastParty.Debug( 2, YEL..'DoInstantHeal', GRY..'found instant cast spell')
      return spell, "";
   end

   -- this couldn't cast an instant spell, so we are here.
   -- this is because we want to make sure that the spell selection knows that we couldn't cast
   -- an instant. (most likely because there is already one going.) this is al used overheal protection
   CastParty.Debug( 2, YEL..'DoInstantHeal', GRY..'no instant cast')
   return nil, "";
end

--***************************************************************************
--***************************************************************************
-- do the emergency in combat thing here. this was messing up so i made it it's own function...
-- return the spell to cast, or nil. nil if you should cast an efficient one.
function CastParty.Healomatic.DoEmergencyInCombat(_spell, _healType, _unitHealthFraction, _playerManaFraction, _Adjusted_health, _unitHealthModified)
   -- check for being in combat and mana is above mana_conserve_threshold
   -- this is not a mana friendly spell and could use more mana then needed
   if CastPartyConfig.emergency_in_combat and _healType == "" then
      if (UnitAffectingCombat('player')
          and CastParty.State.last_player_damage
          and (GetTime() - CastParty.State.last_player_damage) < CastPartyConfig.emergency_damage_window) then

         -- check to see if an emergency heal will heal the target to targeted health? - 20%
         -- check to see if target is under a % of health if so always cast
         -- I have played with this a to some extent, should consider placing
         -- a variable in place in stead of the 20% rule also consider mana conservation
         if _Adjusted_health > ( _unitHealthModified - 0.2) and _playerManaFraction > CastPartyConfig.mana_conserve_threshold then
            CastParty.Debug( 2, YEL..'Emergency heal in combat!')
            return spell;
         end

         -- check to see if target is under a % of health if so always cast
         -- reguardless of mana remaining, it is an emergency, and time matters
         if _unitHealthFraction < CastPartyConfig.emergency_health_percent then
            --pop up an error message on the screen, for feedback purposes!!!
            UIErrorsFrame:AddMessage('Emergency health trigger in combat!', 1.0, 0.1, 0.1, 1.0, UIERRORS_HOLD_TIME);
            return spell;
         end

         -- check for being in combat and mana is above 45%, stop using it if your running
         -- out of mana
         if _playerManaFraction > CastPartyConfig.mana_conserve_threshold then
            if CastParty_Debug then CastParty_Debug( 2, YEL..'Emergency heal!') end;
            --pop up an error message on the screen, for feedback purposes!!!
            UIErrorsFrame:AddMessage('Mana conserve trigger in combat!', 1.0, 0.1, 0.1, 1.0, UIERRORS_HOLD_TIME);
            return spell;
         end

         -- no spell to cast while in combat! so cast an effcient one!
         return nil;
      end
   else
      if CastParty_Debug then CastParty_Debug( 2, YEL..'Emergency heal!') end;
      return _spell;
   end
end

--***************************************************************************
--***************************************************************************
-- This is the core function that is used to cast healing spells
--
-- Currently used by:
--    Keybindings
--

function CastParty.Healomatic.ApplyHeal(_unit, _healType)
   CastParty.Debug( 1, RED..'\n\nNew heal occuring!!!', _unit, _healType)

   local spell;
   local healType = _healType;

   if type(_healType) ~= 'string' or _healType == "" then
      healType = "";
   end

   if CastParty.State.usingBattlefieldHeal and healType == "" then
      return;
   end

   -- check and make sure there are heal types, this is not always checked for if the spell was not forced
   if not CastParty.Brain.heal_data then
      CastParty.Debug( 1, 'ApplyHeal failed, no brain heal_data in '..CastParty.Brain.className )
      healType = "";
      return;
   end

   -- when we have no target, OR if they are not our friend (demorgan's law on this one...)
   -- Then cast on self
   if not _unit or not UnitIsFriend(_unit, 'player') then
      _unit = 'player';
   end

   local targetHealth    = CP_UnitHealth(_unit);
   local targetHealthMax = CP_UnitHealthMax(_unit);

   -- make sure unit can benfit from healing
   if not CastParty_UnitIsHealable(_unit) then
      CastParty.Debug( 4, _unit, RED..'does not benfit from healing or cannot be rezzed' )
      return;
   end

   CastParty.Debug( 3, RED..CastParty.Brain.className, healType )
   if CastPartyConfig.advanced_spells then
      local tempFunction = CastParty.Brain.AdvancedHeal
      if tempFunction ~= nil and type(tempFunction) == 'function' then
         spell = tempFunction(_unit, targetHealth, targetHealthMax, healType);
      else
         CastParty.Debug(4, RED..'No advanced heal found in the '..CastParty.Brain.className..' brain');
         spell = CastParty.Healomatic.DefaultHeal( _unit, targetHealth, targetHealthMax, healType);
      end
   else
      spell = CastParty.Healomatic.DefaultHeal( _unit, targetHealth, targetHealthMax, healType);
   end


   -- if we found no spell return and do nothing
   if not spell or type(spell) ~= 'table' or not spell.id or not spell.name or not spell.rank then
      CastParty.Debug(1, RED..'NO SPELL FOUND!')
      return;
   end

   if spell.name and spell.rank then
      CastParty.Debug( 3, GRN..'Spell chosen', spell.name, spell.rank)
   else
      CastParty.Debug( 3, GRN..'Spell chosen', tostring(spell))
   end
   
   if not spell.id then
      CastParty.Debug( 4, RED.. 'Unable to cast spell with no id field', tostring(spell))
      return;
   end

   -- event handler stuff for canceling spells, and spell interruptions, etc
   CastParty.State.spell_target_unit = _unit;
   CastParty.State.spell_being_cast  = spell;

   CastParty_Retarget(_unit);

   CastParty.TriggerTrinket(spell.name);
   CastSpell(spell.id, SpellBookFrame.bookType);
   SpellTargetUnit(_unit);

   CastParty_Retarget('SET');
end


--[[***************************************************************************
*
*                S P E L L   S E L E C T I O N   F U N C T I O N S
*
***************************************************************************]]--


--***************************************************************************
--***************************************************************************
-- Helper function, test level, and spell, mana, and health
-- returns spell and adjusted health for further testing in
-- some cases where the spell does not heal enough.
--
-- This is the main guts of how castparty spell selection works
--
-- Currently used by:
--    AdvancedDRUIDHeal
--    AdvancedPRIESTHeal
--    AdvancedPALADINHeal
--    AdvancedSHAMANHeal
--    DefaultHeal
function GetSpell( _health, _healthTarget, _healthMax, _healType, _unitLevel, _mana, _unit, _maxSpellRank, _lowRank, _highRank )
   local spell;
   local value;
   local adjusted_health = 0;


   -- some spell types warrent the use of the max/best spell like instant
   -- problem is this is not mana efficient, why cast a max level regen if
   -- the target is only 100 health away from max?  to work around this
   -- as everyone's preferances are different, is the indroduction of
   -- of _max_healing combined with the spell selections that warrent
   -- the use of the max_spell.  These combined can override health checks
   -- on selected spells or not.  This is a conserve mana or not feature
   if _maxSpellRank or _healType == 'shield' then
      _healthTarget = _healthTarget + 10000;
   end

   -- entered a mana-less state? force health target to max, as it doesn't cost anything, and then
   -- force mana to max because it doesn't cost anything
   if CastParty.State.heal_states[CASTPARTY_CLEARCASTING]
   or CastParty.State.heal_states[CASTPARTY_BUFF_INNER_FOCUS]
   or CastParty.State.heal_states[CASTPARTY_BUFF_SPIRIT_OF_REDEMPTION] then
      _healthTarget = _healthTarget + 10000;
      _mana = _mana + 10000;
   end

   -- forums reported error 1074
   if _unitLevel == nil then
      CastParty.Debug( 5, 'CastParty has detected an error 1074.  Please' )
      CastParty.Debug( 5, 'report the following infomation to' )
      CastParty.Debug( 5, 'https://sourceforge.net/projects/wowcastparty/' )
      CastParty.Debug( 5, 'Unable to detect target level' )
      _unitLevel = UnitLevel('player');
      if _unitLevel == nil then
         CastParty.Debug( 5, 'Unable to detect the players level' )
         CastParty.Debug( 5, 'setting target to level 1' )
         _unitLevel = 1;
      else
         CastParty.Debug( 5, 'Target level is assumed to be', _unitLevel )
      end
   end

	local CP_healing_strength = 0;

   if (BonusScanner and BonusScanner.active) then
      CastParty.State.healing_base = BonusScanner:GetBonus('HEAL');
   end

   local lowRank = _lowRank or 1;
   local highRank = _highRank or table.getn(CastParty.Brain.heal_data[_healType]);

   local spellName;
   local buffLevel;
   
   CastParty.Debug(2, BLU.. 'searching for best heal type:', _healType)

   for index = lowRank, highRank do
      value = CastParty.Brain.heal_data[_healType][index];

      --CastParty.Debug( 2, YEL, _healType)
      CastParty.Debug( 2, BLU..'testing'..GRY, value.name, index, 'Heal', value.max_heal )
      CastParty.Debug( 1, GRY..'mana cost', value.mana, 'mana available', _mana)

      -- make sure we have the mana for it
      if value.mana <= _mana then
         -- check spell level here, make sure we use the correct level of spell
         spellName = value.name

         -- forums reported error 1072, CastParty_Debug line of code
         if value == nil or value == 0 then
            CastParty.Debug( 5, RED..'CastParty has detected an error 1072.  Please' )
            CastParty.Debug( 5, RED..'report the following infomation to' )
            CastParty.Debug( 5, RED..'http://www.sourceforge.net/projects/wowcastparty/' )
            CastParty.Debug( 5, RED..'Unable to index class buff levels, disabling' )
            CastParty.Debug( 5, RED..'spell rank restriction based on spell level' )
            CastParty.Debug( 5, RED..'for Class', CastParty.Brain.className )
            CastParty.Debug( 5, RED..'for Spell type', _healType )
            CastParty.Debug( 5, RED..'for Spell rank', index )
            CastParty.Debug( 5, RED..'for Spell', spellName )
            buffLevel = 1;
         elseif not CastParty.Brain.class_buffs[spellName] then
            CastParty.Debug( 5, RED..'CastParty does not know about class buff: '..spellName)
            CastParty.Debug( 5, RED..'Please report the following infomation to' )
            CastParty.Debug( 5, RED..'http://www.sourceforge.net/projects/wowcastparty/' )
            CastParty.Debug( 5, RED..'Unrecognized class buff:'..spellName )
            CastParty.Debug( 5, RED..'for Class', CastParty.Brain.className )
            CastParty.Debug( 5, RED..'for Spell type', _healType )
            CastParty.Debug( 5, RED..'for Spell rank', index )
            buffLevel = 1;
         else
            CastParty.Debug( 1, GRN.. 'Looking up class buff for '..CastParty.Brain.className, spellName, index)
            --CastParty.Debug( 1, GRN.. tostring(CastParty.Brain.class_buffs))
            --CastParty.Debug( 1, GRN.. tostring(CastParty.Brain.class_buffs[spellName]))
            --CastParty.Debug( 1, GRN.. tostring(CastParty.Brain.class_buffs[spellName][index]))
            buffLevel = CastParty.Brain.class_buffs[spellName][index];
         end

         CastParty.Debug( 1, 'Buff Level', buffLevel, 'Unit Level', _unitLevel )

         local bonusmult = CastParty.Brain.healing_bonus_multipliers
         if bonusmult and bonusmult[spellName] and bonusmult[spellName][index] then
            CP_healing_strength = bonusmult[spellName][index];
            CastParty.Debug( 1, 'Healing Bonus', CastParty.State.healing_base, '*', CP_healing_strength, '=', (CastParty.State.healing_base * CP_healing_strength) )
         end

         -- don't allow a heal much greator then the targets health deficit
         -- but not if we are in a manaless state.
         -- Only if we cast an efficient heal do we want this to happen.
         if CastPartyConfig.overheal_protection and not CastPartyConfig.useMaxRankNoMatterWhat and
         	not (CastParty.State.heal_states[CASTPARTY_CLEARCASTING]
            or CastParty.State.heal_states[CASTPARTY_BUFF_INNER_FOCUS]
            or CastParty.State.heal_states[CASTPARTY_BUFF_SPIRIT_OF_REDEMPTION])
            and _healType == 'efficient' then
            if value.max_heal > _healthMax - (_health * 1.2 * CastPartyConfig.spell_choice_threshold) then
               break;
            end
         end

         -- make sure the target is not to low to recive the heal
         if buffLevel and ( buffLevel - _unitLevel <= 10 )then
            spell = value;
            -- break the search early if we find a smaller sufficient heal
            adjusted_health = value.max_heal + _health + (CastParty.State.healing_base * CP_healing_strength);
            if CastParty_Debug then CastParty_Debug( 1, 'target health', _healthTarget, 'health + heal', adjusted_health) end;
            if adjusted_health >= _healthTarget then
               break;
            end
         end
      end
   end

	-- This way, if we can FORCE the highest rank of a particular spell (not the general category)
	-- i.e force the highest rank of lesser heal, etc.
	-- this is different that above in that this allows the normal spell selection to still occur, and
	-- then it will pick the best rank spell of whatever spell is returned.
	if CastPartyConfig.useMaxRankNoMatterWhat then
	   if CastParty_Debug then CastParty_Debug( 2, RED..'max rank forced enabled' ) end;
		spell = CastParty.Brain.best_spell_ids[value.name];
		adjusted_health = spell['max_heal'] + _health + (CastParty.State.healing_base * CP_healing_strength);
	end
   
   CastParty.Debug(2, RED.. 'returning:', spell, adjusted_health)

   return spell, adjusted_health
end


function CastParty.Healomatic.DoResurrect(_unit, unitLevel, playerMana)
   -- check to see if the unit is dead, then check and see if they have the spells
   -- to do something about it and if so, then make sure the options is selected
   -- may consider revising this to if (UnitIsDead(unit) and not UnitIsGhost(unit)
   -- as you can't target ghost to resurrect, you have to target the body
   -- NR: changed to UnitIsDead (has range issues when unit is a ghost)
   CastParty.Debug(2, RED..'unit is:', _unit);
   --wont work because they (as a ghost) could be right next to thier body.
   -- 20 sept 2006, patch 1.12, Blizz no longer allows targeting a ghost to resurrect, must click the body after release
   if UnitIsGhost(_unit) then
      --pop up an error message on the screen, for feedback purposes!!!
      UIErrorsFrame:AddMessage(CASTPARTY_GHOST_HEAL, 1.0, 0.1, 0.1, 1.0, UIERRORS_HOLD_TIME);
      return
   end

   if CastParty.Brain.heal_data['resurrection'] then
      CastParty.Debug( 1, 'made it to res selection' )

      if CastParty.Brain.className == 'DRUID' then
         local resRank = 0

         --TODO: I'm going to be adding cooldown checks here in the future, so if a spell is in cooldown
         --      then spit out an error/info thing onto the screen, so they know it's in cooldown...
         -- find a seed
         for _, value in pairs(brain.reagents) do
            bagpos, item  = CastParty.GetContainerItem(value.name);
            if bagpos and item then
               CastParty.Debug(2, 'Found', value.name, 'In Bag', bagpos, 'In Slot', item, 'For Rank', value.rank )
               if value.rank > resRank then
                  resRank = value.rank
               end
            end
         end

         -- if we need level checking will have to add it later, to cycle
         -- though the seeds we have and use the correct one based on level
         -- and then compare that seed with the spell level to be compared
         -- with the unit level

         -- choose the best one we have a seed for, may need to update
         -- the logic later to more selectively choose, and to account for havening seeds
         -- but not the spell

         local resSpell = CastParty.Brain.heal_data['resurrection'][resRank]
         if resSpell then
            if resSpell.mana < playerMana then
               --we have found the resurrection spell for the druid, now return that spell.
               CastParty.Debug( 2, 'Found res spell', resSpell.name, resSpell.rank)
               return resSpell;
            end
         end
      end
      return GetSpell( 0, 0, 0, 'resurrection', unitLevel, playerMana, _unit, true );
   end
end

--***************************************************************************
--***************************************************************************
-- determine what spell is most appropriate given the health and max health
-- takes into account the current state of affairs as well
--
-- Currently used by:
--    CastParty.Healomatic.ApplyHeal
--    CastParty.Healomatic.DoTheRightThing
--
function CastParty.Healomatic.DefaultHeal( _unit, _unitHealth, _unitHealthMax, _healType)
   CastParty.Debug( 2, GRN..'DefaultHeal triggered' )

   local spell;
   local playerMana;
   local playerManaFraction;
   local unitHealthModified;
   local unitHealthFraction;
   local unitLevel;

   local _healType, playerMana, playerManaFraction, unitHealthModified, unitHealthFraction, unitLevel = 
      CastParty.Healomatic.InitHealVars(_unit, _unitHealth, _unitHealthMax, _healType);

   if _healType and _healType == 'resurrection' then
      return CastParty.Healomatic.DoResurrect(_unit, unitLevel, playerMana)
   end

   -- always choose the highest heal when forced
   if _healType and _healType ~= "" and CastParty.Brain.heal_data[_healType] then
      CastParty.Debug( 3, 'heal specified', _healType)

      -- always choose the best instant and group heals based on mana
      if _healType == 'instant' or _healType == 'group' or _healType == 'shield' then
         return GetSpell( _unitHealth, unitHealthModified, _unitHealthMax, _healType, unitLevel, playerMana, _unit, true );
      end

      -- otherwise choose the best efficient/emergency based on mana and health
      return GetSpell( _unitHealth, unitHealthModified, _unitHealthMax, _healType, unitLevel, playerMana, _unit )
   end


   --this is so if we can cast a heal, return, otherwise we allow the program to move on...
   if _healType == 'emergency' then
      CastParty.Debug( 1, GRY..'begin emergency checks...')
      -- is a previous spell still running? if so skip and cast a normal spell
      -- no futher benfit can be gained from re-casting this spell, it would
      -- just be a waste of addtional mana, as the direct healing of the spell
      -- is rather minor
      if CastParty.Brain.heal_data['emergency'] then
         local Adjusted_health = 0;

         -- load up the spell for testing
         spell, Adjusted_health = GetSpell( _unitHealth, unitHealthModified, _unitHealthMax, 'emergency', unitLevel, playerMana, _unit );

         spell = CastParty.Healomatic.DoEmergencyInCombat(spell, _healType, unitHealthFraction, playerManaFraction, Adjusted_health, unitHealthModified);
      end

      --this is so if we can cast and emergency spell, do it, otherwise we allow the program to move on...
      if spell then
         return spell;
      end
   end

   -- otherwise try an efficient heal
   if CastParty.Brain.heal_data['efficient'] then
      return GetSpell( _unitHealth, unitHealthModified, _unitHealthMax, 'efficient', unitLevel, playerMana, _unit )
   end

end


-- The advanced heals have been moved to their respective brains.
-- TODO: move these comments somewhere useful
--
-- These funtions are used to replaces the default CastParty_DefaultHeal,
-- I belive the default is effective for those that would rather deal with
-- the details of a healing class.  But these versions enables the classes
-- to be less twitch based and more of a support class, keep in mind this
-- in no way makes the class a mindless drone, and players still need to play
-- their class correctly for these to even be effective there simply is no
-- logic currently in existance that will account for every situation.
-- Nor is it our desire to play the game though an addon what fun would
-- that be, if someone else plays your character for you removing all
-- of your choices?
--
-- How these functions differ from DefaultHeal is quite simple, it checks
-- for the presence of other spells, and then skips them if they're active
-- nothing more to it, basically the two differences are, in DefaultHeal
-- you request a spell to fire, if the spell doesn't fire then you have
-- to reselect another spell.  In the logic following it can skip certain
-- re-clicks and basically just avoid continued clicks nothing more.
-- The function will not play the class for you and you will still have
-- to know when an efficent heal or an instant heal is better just like
-- in the past.  The only intent of these functions is to make the class
-- not as twich based and allow for more selectablity in healing choices
-- A 60 level character just cannot have all his healing spells displayed
-- in the game, thus it normally results in having only a few of each
-- type of spell in the action windows, other wise the clutter would reduce
-- a player's ablity to play the game, also if Blizzard intended for us
-- to only select a few healing spells at each level of the character
-- they would simply replace each rank spells instead of allow access
-- to them, as they do with warrior.
--
-- one final note, these functions are subject to deletion as per
-- the remarks made by slouken at this location
--
-- NOTE: These remarks are gone now...
-- http://forums.worldofwarcraft.com/thread.aspx?fn=wow-interface-customization&t=161068&p=1&tmp=1
--
-- there simply is insuffienct data to make a judgement at this time.
--
