

--check NOT IN OPTIONS YET section for options still to be added to CP Options GUI.


function CastParty.Config.Load()
   -- if there is no config file, we need to specify what class it is, and what keybindings to default to.
   if CastPartyConfig == nil then
      DEFAULT_CHAT_FRAME:AddMessage(RED.. 'Resetting '..YEL.. BINDING_NAME_CASTPARTY_OPTIONS);
      CastPartyConfig = {};
   end

   --[[
      --*****************************************************************************
      --
      --  NOTE: This is where you can change your default keybinding settings per class.
      --
      --*****************************************************************************
      --
      -- if the value is a global function *name*, that function will be
      -- called with the unit id (e.g., 'player1') as the only argument.
      -- This can be a WoW defined function such as 'TargetUnit' or
      -- 'AssistUnit', or one defined by CastParty, see below:
      --
      --    CastParty_ApplyHeal
      --       picks a heal based on damage, mana and combat
      --
      --    CastParty_ApplyEfficientHeal
      --       picks a heal based on damage, mana and combat from *efficient* heals only,
      --       e.g. 'Greater Heal' when playing Priest.
      --
      --    CastParty_ApplyEmergencyHeal
      --       picks a heal based on damage, mana and combat from *emergency* heals only,
      --       e.g. 'Flash Heal' when playing a Priest.
      --
      --    CastParty_ApplyInstantHeal
      --       picks from *instant* heals only, usually a HoT (health over time) spell.
      --
      --    CastParty_ApplyRessurection
      --       resurrects a selected players, works with druid seeds as well.
      --
      --    CastParty_ApplyShield
      --       Used for paladins and Priest to apply a protective shield.
      --
      --    CastParty_DoTheRightThing
      --       Heals the target, if the target is not healable scan the party for the person with
      --       the least health.
      --
      --    CastParty.Healomatic.ApplyBandage
      --       picks the highest level bandage you have and uses it on your target.
      --
      --    CastParty_CureAny
      --       picks one of the right cures for whatever ails your target.
      --
      --    CureGroupMember
      --       Scan the party for the 1st cureable.
      --
      --    CastParty_StopSpell
      --       used as a spamming key to stop spells if the right condtions are meet
      --
      --    CastParty_Panic
      --       used to cast natures swiftness, then cast your most efficient heal
      --       with other classes, just casts an emergency heal
      --
      --
      -- if the value is a spell name and rank (e.g., 'Healing Wave(Rank 1)') then
      -- that spell will be cast on the player that receives the click.
      --
      -- if the value is a spell name with *no* rank (e.g., 'Healing Wave') then
      -- the most powerful version of that spell will be cast on the player that
      -- receives the click. note that for buffs this takes the level of the recipient
      -- into consideration.
      --
      -- power users can even set the value to a function. that function will
      -- be called with the unit id as the only argument
   --]]

   local class_key_bindings = CastParty.Brain.default_key_bindings
   
   if class_key_bindings then
      CastParty.Debug(4, 'Found default key bindings from brain: '..CastParty.Brain.className)
      --CastParty.DebugPrintBindings();
   else
      -- Couldn't get key bindings from the brain or brain base
      -- if we don't know WHAT you are... there are more important problems to be worried about.
      note(RED.. 'CastParty brain did not load before config data')
   end
   
   -- ------------------------------------------------------------------------------------
   -- DISPLAY SETTINGS
   -- The following options are related to the minimalistic party frame:
   -- ------------------------------------------------------------------------------------

   -- specify whether the castparty player frames should be shown or not
   -- true: castparty player frames are hidden
   -- false : castparty player frames are not hidden
   if CastPartyConfig.showPlayer == nil then
      CastPartyConfig.showPlayer = true;
   end


   -- specify whether the regular party frames should be shown or not
   -- true: regular party frames are hidden
   -- false : regular party frames are not hidden
   if CastPartyConfig.hide_party_frame == nil then
      CastPartyConfig.hide_party_frame = true;
   end

   -- specify whether the statusbar frame (border around cast party) should be shown or not
   -- true: statusbar frame is visible
   -- false: statusbar frame is not visible
   if CastPartyConfig.show_statusbar_frames == nil then
      CastPartyConfig.show_statusbar_frames = true;
   end

   -- specify whether you want to see the castpart party frames or not
   -- true: castparty party frames is  visible
   -- false: castparty party frames is not visible
   if CastPartyConfig.showParty == nil then
      CastPartyConfig.showParty = true;
   end

   -- specify whether cast party can move
   -- true: cast party cannot move
   -- false: cast party can move
   if CastPartyConfig.locked == nil then
      CastPartyConfig.locked = false;
   end

   -- specify manner in which buffs are displayed
   --  right: the buffs appear right of the party bars
   --   left: the buffs appear left of the party bars
   --  below: the buffs appear to below the party bars
   if CastPartyConfig.pet_alignment == nil then
      CastPartyConfig.pet_alignment = CASTPARTY_ALIGNMENT_BELOW;
   end

   -- specify whether you want to see castparty target frame or not
   -- true: castparty target frame is  visible
   -- false: castparty target frame is not visible
   if CastPartyConfig.showTarget == nil then
      CastPartyConfig.showTarget = true;
   end

   -- specify whether the regular target frames should be shown or not
   -- true: regular target frames are hidden
   -- false : regular target frames are not hidden
   if CastPartyConfig.hide_target_frame == nil then
      CastPartyConfig.hide_target_frame = false;
   end

   -- specify whether the regular player frames should be shown or not
   -- true: regular player frames are hidden
   -- false : regular player frames are not hidden
   if CastPartyConfig.hide_player_frame == nil then
      CastPartyConfig.hide_player_frame = false;
   end

   -- specify whether to allow castparty style clicking on other type of frames
   -- true: allow for range checking in the background
   -- false: don't allow it (default - also might be problematic for slow computers)
   if CastPartyConfig.update_range == nil then
      CastPartyConfig.update_range = true;
   end

   -- These settings are changed above, PER CLASS!!! Remember, these are just the
   -- default behavior, when this mod is run for the first time on a char.
   if CastPartyConfig.key_bindings == nil then
      CastPartyConfig.key_bindings = class_key_bindings;
   end


   -- -----------------------
   -- AURAS / BUFFS / DEBUFFS
   -- -----------------------

   -- specify whether debuffs should be shown or not
   -- true:  debuff icons are visible
   -- false: debuff icons are not visible
   if CastPartyConfig.show_debuffs == nil then
      CastPartyConfig.show_debuffs = true;
   end

   -- specify whether buffs should be shown or not
   -- true:  buff icons are visible
   -- false: buff icons are not visible
   if CastPartyConfig.show_buffs == nil then
      CastPartyConfig.show_buffs = true;
   end

   -- specify whether common buffs should always be shown or not
   -- true:  buff icons are always visible
   -- false: buff icons are not always visible
   if CastPartyConfig.show_common_buffs == nil then
      CastPartyConfig.show_common_buffs = false;
   end

   -- specify whether buffs should be filtered to only those that you can cast
   -- true:  filter buffs by the ones you can cast
   -- false: do not filter the buffs
   if CastPartyConfig.filter_buffs == nil then
      CastPartyConfig.filter_buffs = true;
   end

   -- specify whether debuffs should be filtered to only those that you can cure
   -- true:  filter by those you can cure
   -- false: so all the debuffs
   if CastPartyConfig.filter_debuffs == nil then
      CastPartyConfig.filter_debuffs = true;
   end

   -- specify whether tooltips should be shown when mousing over buffs and debuffs
   -- true:  show the tooltips for auras
   -- false: do not show the tooltips for auras
   if CastPartyConfig.show_aura_tooltips == nil then
      CastPartyConfig.show_aura_tooltips = true;
   end

   -- specify whether tooltips should be shown when mousing over players in your party and in combat
   -- true:  show the partymember tooltips
   -- false: do not show the tooltips
   if CastPartyConfig.hide_aura_tooltips_in_combat == nil then
      CastPartyConfig.hide_aura_tooltips_in_combat = false;
   end

   -- specify manner in which buffs are displayed
   --  right: the buffs appear right of the party bars
   --   left: the buffs appear left of the party bars
   --  below: the buffs appear to below the party bars
   if CastPartyConfig.buff_alignment == nil then
      CastPartyConfig.buff_alignment = CASTPARTY_ALIGNMENT_RIGHT;
   end

   -- specify manner in which debuffs are displayed
   --  right: the debuffs appear right of the party bars
   --   left: the debuffs appear left of the party bars
   --  below: the debuffs appear to below the party bars
   if CastPartyConfig.debuff_alignment == nil then
      CastPartyConfig.debuff_alignment = CASTPARTY_ALIGNMENT_RIGHT;
   end

   -- specify whether hots should be shown or not
   -- true:  hots icons are visible
   -- false: hots icons are not visible
   if CastPartyConfig.show_hots == nil then
      CastPartyConfig.show_hots = true;
   end

   -- specify manner in which buffs are displayed
   --  right: the buffs appear right of the party bars
   --   left: the buffs appear left of the party bars
   --  below: the buffs appear to below the party bars
   if CastPartyConfig.pet_buff_alignment == nil then
      CastPartyConfig.pet_buff_alignment = CASTPARTY_ALIGNMENT_RIGHT;
   end

   -- specify manner in which debuffs are displayed
   --  right: the debuffs appear right of the party bars
   --   left: the debuffs appear left of the party bars
   --  below: the debuffs appear to below the party bars
   if CastPartyConfig.pet_debuff_alignment == nil then
      CastPartyConfig.pet_debuff_alignment = CASTPARTY_ALIGNMENT_RIGHT;
   end

   -- sets how many auras you want visible at one time one each player. (global setting -- buffs/debuffs)
   if CastPartyConfig.max_auras == nil then
      CastPartyConfig.max_auras = 40;
   end


   -- ----
   -- BARS
   -- ----

   -- specify whether tooltips should be shown when mousing over players in your party
   -- true:  show the partymember tooltips
   -- false: do not show the tooltips
   if CastPartyConfig.show_party_tooltips == nil then
      CastPartyConfig.show_party_tooltips = true;
   end

   -- specify whether tooltips should be shown when mousing over players in your party and in combat
   -- true:  show the partymember tooltips
   -- false: do not show the tooltips
   if CastPartyConfig.hide_unit_tooltips_in_combat == nil then
      CastPartyConfig.hide_unit_tooltips_in_combat = false;
   end

   -- specify the height of your health bar
   if CastPartyConfig.health_bar_height == nil then
      CastPartyConfig.health_bar_height = 12;
   end

   -- specify the height of your mana bar
   if CastPartyConfig.mana_bar_height == nil then
      CastPartyConfig.mana_bar_height = 6;
   end

   -- specify the width of your health and mana bars
   if CastPartyConfig.bar_width == nil then
      CastPartyConfig.bar_width = 150;
   end

   -- specify whether to show rage/energy when available or not
   -- true:  show rage and energy
   -- false: do not show rage and energy
   if CastPartyConfig.show_power == nil then
      CastPartyConfig.show_power = true;
   end

   -- specify the format of the text on your party member bars
   if CastPartyConfig.unit_text_template == nil then
      CastPartyConfig.unit_text_template = '%n: %Dh';
   end

   if CastPartyConfig.target_text_template == nil then
      CastPartyConfig.target_text_template = '%l: %n: %h';
   end

   -- specify whether to allow castparty style clicking on other type of frames
   -- true: allow cp click style
   -- false: don't allow it (default)
   if CastPartyConfig.cp_clicks == nil then
      CastPartyConfig.cp_clicks = false;
   end

   -- specify whether to allow castparty style clicking on blizzard frames
   -- true: allow cp click style
   -- false: don't allow it (default)
   if CastPartyConfig.cp_blizz_clicks == nil then
      CastPartyConfig.cp_blizz_clicks = false;
   end

   -- specify whether cast party will show the player flag icons
   -- true: flag icons visible
   -- false: flag icons not visible
   if CastPartyConfig.show_flags == nil then
      CastPartyConfig.show_flags = true;
   end

   -- specify whether the frames will appear above of below you
   -- true: frames will appear above you
   -- false: frames will appear below you
   if CastPartyConfig.flow_up == nil then
      CastPartyConfig.flow_up = false;
   end

   -- specify whether if we want the names of people colored by class
   -- true:  color the names
   -- false: don't color the names (leave them white, or whatever color specified)
   if CastPartyConfig.color_names == nil then
      CastPartyConfig.color_names = false;
   end

   -- specify whether if we want the level of people colored by difficulty
   -- true:  color the level
   -- false: don't color the level (leave them white, or whatever color specified)
   if CastPartyConfig.color_level == nil then
      CastPartyConfig.color_level = false;
   end

   -- --------
   -- BEHAVIOR
   -- --------

   -- specify growth/coloring behavior of health bar
   --  true:  bars grow as pools shrink (CastParty style)
   --  false: bars shrink as pools shrink (default WoW client style)
   if CastPartyConfig.grow_with_depletion_health == nil then
      CastPartyConfig.grow_with_depletion_health = false;
   end

   -- specify growth/coloring behavior of mana bar
   --  true:  bars grow as pools shrink (CastParty style)
   --  false: bars shrink as pools shrink (default WoW client style)
   if CastPartyConfig.grow_with_depletion_mana == nil then
      CastPartyConfig.grow_with_depletion_mana = false;
   end

   -- specify growth/coloring behavior of rage bar
   --  true:  bars grow as pools shrink (CastParty style)
   --  false: bars shrink as pools shrink (default WoW client style)
   if CastPartyConfig.grow_with_depletion_rage == nil then
      CastPartyConfig.grow_with_depletion_rage = false;
   end

   -- specify growth/coloring behavior of focus bar
   --  true:  bars grow as pools shrink (CastParty style)
   --  false: bars shrink as pools shrink (default WoW client style)
   if CastPartyConfig.grow_with_depletion_focus == nil then
      CastPartyConfig.grow_with_depletion_focus = false;
   end

   -- specify growth/coloring behavior of energy bar
   --  true:  bars grow as pools shrink (CastParty style)
   --  false: bars shrink as pools shrink (default WoW client style)
   if CastPartyConfig.grow_with_depletion_energy == nil then
      CastPartyConfig.grow_with_depletion_energy = false;
   end

   -- Specifies the health percentage a given spell must satisfy to be chosen as the
   -- most appropriate heal. for example, a value of 0.90 indicates that if a heal
   -- gets the player to 90% health no higher heal will be tested. Note that if you
   -- enable the overheal_protection setting you won't be able to cast a heal on someone
   -- with a health higher than this treshold (aka no preemptive healing if treshold
   -- is set to less than 1.0).
   if CastPartyConfig.spell_choice_threshold == nil then
      CastPartyConfig.spell_choice_threshold = 0.90;
   end

   -- If the amount of healing that your group heal spell can do (assuming you have one)
   -- is this much more health/mana efficient than your best single target efficient heal
   -- the group heal will be favored. This affects the 'Do The Right Thing' key binding
   -- and do the right thing click binding.
   if CastPartyConfig.auto_group_threshold == nil then
      CastPartyConfig.auto_group_threshold = 1.5;
   end

   -- Specifies that emergency spells will be favored when the player is on a monster's
   -- hate list and has recenty taken damage.
   -- true:  enable faster spells at the price of mana
   -- false: do not allow faster spells to be cast
   if CastPartyConfig.emergency_in_combat == nil then
      CastPartyConfig.emergency_in_combat = true;
   end

   -- Specifies the health percentage at which the player will cast emergency heals in
   -- place of longer casting more efficient heals.
   if CastPartyConfig.emergency_health_percent == nil then
      CastPartyConfig.emergency_health_percent = 0.35;
   end

   -- number of seconds for which damage will be considered recent in the above.
   -- a value of 2 (and 1 for 'emergency_in_combat') means that the player must
   -- be in combat and taken damage in the last 2 seconds to automatically use
   -- emergency heals
   if CastPartyConfig.emergency_damage_window == nil then
      CastPartyConfig.emergency_damage_window = 2;
   end

   -- ------------------------------------------------------------------------------------
   -- HEALING CANCELLATION
   -- These options allow you to auto-cancel a heal shortly before it lands if the unit
   -- you were about to heal has been healed by someone else.
   -- ------------------------------------------------------------------------------------

      -- if the heal spell being cast is not going to hit for at least this fraction of its
      -- potential, it will cancel. So, if you set it to 0.6 (i.e., 60%) then a heal for
      -- 1000 health would cancel if the recipient needs less than 60% of this amount.
      -- The lower the treshold, the more mana you will waste. A treshold of 0.7
      -- will overheal the unit for a maximum of 30% (non-crit).
      -- Note to DEVs: the test for cancellation is made during UNIT_HEALTH updates.
   if CastPartyConfig.heal_cancel_threshold == nil then
      CastPartyConfig.heal_cancel_threshold = 0.7;
   end

   -- Specify whether to try to resurrect a dead player when you click on his bar
   -- true:  try to resurrect selected player
   -- false: do not try to resurrect the selected player
   if CastPartyConfig.resurrect_dead == nil then
      CastPartyConfig.resurrect_dead = true;
   end

   -- specify weather to cure a debuff when you click on it in the CP bars
   -- true:  clicking on an debuff aura will cure the debuff
   -- flase: turns off the clicking to cure the debuff
   if CastPartyConfig.click_to_debuff == nil then
      CastPartyConfig.click_to_debuff = true;
   end

   -- specify whether to rebuff a spell when you click on it in the CP bars
   -- true:  clicking on an aura will re-cast the spell
   -- false: turns off clicking to re-buff
   if CastPartyConfig.click_to_rebuff == nil then
      CastPartyConfig.click_to_rebuff = true;
   end

   -- specify whether to remove a spell when you right-click on it in the CP bars
   -- true:  right clicking on an aura will remove the spell
   -- false: turns off right clicking to remove
   if CastPartyConfig.rightclick_to_remove == nil then
      CastPartyConfig.rightclick_to_remove = true;
   end


   -- ------------------------------------------------------------------------------------
   -- MESSAGE SPAM SETTINGS
   -- CastParty can announce heals and other spells in a channel of your choice:
   -- ------------------------------------------------------------------------------------

   -- specify whether we should message about healing spells
   -- true:  send healing messages if health_percent is meet
   -- false: do not send healing messages
   if CastPartyConfig.message_heals == nil then
      CastPartyConfig.message_heals = false;
   end

   -- specify whether we should message about non-healing spells
   -- true:  send message about other spell other then healing spells
   -- false: do not send messages about other spells
   if CastPartyConfig.message_non_heals == nil then
      CastPartyConfig.message_non_heals = false;
   end

   -- specify whether we should message about failed healing spells
   -- true:  send message about other failed healing spells
   -- false: do not send messages about failed healing spells
   if CastPartyConfig.message_failures == nil then
      CastPartyConfig.message_failures = false;
   end

   -- specify whether we should message about resurrections
   -- true:  send res message
   -- false: do not send the res message
   if CastPartyConfig.message_res == nil then
      CastPartyConfig.message_res = true;
   end

   -- Specify where to send messages.  Setting to raid while not in a raid will send to party.
   -- If not in a party, setting to raid or party will not generate messages.
   -- 'player' = send messages to self
   -- 'whisper' = send messages to people not in your group
   -- 'party' = send messages to your party for people in your group
   -- 'raid' = send messages to you raid or party
   --
   -- this setting will see addional upgrades stay tunned, we are aware of changes needed
   if CastPartyConfig.message_channel == nil then
      CastPartyConfig.message_channel = 'raid';
   end

   -- specify the health percentage at which and below we should issue healing messages
   if CastPartyConfig.message_health_percent == nil then
      CastPartyConfig.message_health_percent = 0.35;
   end


   -- specify the template to say when casting spells (nil indicates no message)
   -- %c - target class
   -- %C - target class abbreviation
   -- %d - duration of cast in seconds
   -- %e - estamated lowest health amount healed on dilivery
   -- %h - health of target
   -- %H - health deficit
   -- %l - level of target
   -- %L - level of target (hide if 60)
   -- %m - health max of target
   -- %n - name of target
   -- %p - health percent of target
   -- %P - health percent deficit of target
   -- %r - spell rank
   -- %s - spell name
   -- %t - custom health message
   if CastPartyConfig.message_spell_text == nil then
      CastPartyConfig.message_spell_text = 'Casting %s (%r) on %n in %d seconds...';
   end

   -- specify the health percentage at which and below we should issue failure messages
   if CastPartyConfig.message_failure_percent == nil then
      CastPartyConfig.message_failure_percent = 0.5;
   end

   -- specifies to stop spamming if mana is below this amount
   -- set to 0.0 to spam all casting messages
   -- setting ot 1.0 will stop all messages
   if CastPartyConfig.message_disable_mana == nil then
      CastPartyConfig.message_disable_mana = 0.1;
   end

   -- ------
   -- COLORS
   -- ------

   -- specifies the rgb values for the party member text display
   if CastPartyConfig.text_rgb == nil then
      CastPartyConfig.text_rgb = { r = 1.00, g = 1.00, b = 1.00 };
   end

   -- specifies the rgb values for the party member text mana display
   if CastPartyConfig.text_mana_rgb == nil then
      CastPartyConfig.text_mana_rgb = { r = 0.1, g = 0, b = 0.6 };
   end

   -- specifies the rgb values for the party member text rage display
   if CastPartyConfig.text_rage_rgb == nil then
      CastPartyConfig.text_rage_rgb = { r = 1.00, g = 0, b = 0 };
   end

   -- specifies the rgb values for the party member text energy display
   if CastPartyConfig.text_energy_rgb == nil then
      CastPartyConfig.text_energy_rgb = { r = 1.00, g = 1.00, b = 0 };
   end

   -- specifies the sole rgb values for the mana bar background
   if CastPartyConfig.mana_bg_rgb == nil then
      CastPartyConfig.mana_bg_rgb   = { r = 0, g = 0, b = 0.3, a = 0.5 };
   end

   -- specifies the sole rgb values for the health bar background
   if CastPartyConfig.health_bg_rgb == nil then
      CastPartyConfig.health_bg_rgb = { r = 0, g = 0.3, b = 0, a = 0.5 };
   end

   -- Each bar can have a set color, or can change colors depending on how the
   -- table is filled out.  If the depleation and full setting are the same
   -- then there will be no color change.  If the depleation and full colors
   -- are not the same, then a displayed color will be chosen by, some percentage
   -- between the 2 colors.
   if CastPartyConfig.bar_colors == nil then
      CastPartyConfig.bar_colors = {
         -- mana, depleated color, then full color
         [0] = { r = 1.00, g = 0.00, b = 0.00 },
         [1] = { r = 0.00, g = 0.00, b = 1.00 },
         -- rage, depleated color, then full color
         [2] = { r = 1.00, g = 0, b = 0.00 },
         [3] = { r = 1.00, g = 0, b = 0.00 },
         -- focus, depleated color, then full color
         [4] = { r = 1.00, g = 0.50, b = 0.25 },
         [5] = { r = 1.00, g = 0.50, b = 0.25 },
         -- energy, depleated color, then full color
         [6] = { r = 1.00, g = 1.00, b = 0.00 },
         [7] = { r = 1.00, g = 1.00, b = 0.00 },
         -- happiness, not currently used
         [8] = nil,
         [9] = nil,
         -- health, depleated color, then full color
         [10] = { r =  1.00, g = 0.00, b = 0.00 },
         [11] = { r =  0.00, g = 1.00, b = 0.00 },
      };
   end


   -- --------
   -- ADVANCED
   -- --------
   -- specify how long to check the interval for range checking (in seconds)
   if CastPartyConfig.range_check_interval == nil then
      CastPartyConfig.range_check_interval = 2;
   end

   --determine to show the healing frame monitor frame or not.
   --true: shows it
   --false: hides it
   if CastPartyConfig.showMonitor == nil then
      CastPartyConfig.showMonitor = false;
   end

   -- ------------------------------------------------------------------------------------
   -- PRIEST AUTO-CASTING INNER FIRE
   -- This will allow CastParty to auto-cast inner fire if your target is not in an
   -- emergency and you are recieving damage.
   -- ------------------------------------------------------------------------------------
      -- Set true to cast Inner Fire if the caster is in direct combat, and there is
      -- not an emergency to attend to on the selected target.
      if CastPartyConfig.auto_cast_inner_fire == nil then
         CastPartyConfig.auto_cast_inner_fire = false;
      end

   -- Set to true to automatically cancel wand,etc auto-casting/shooting if you try to cast
   -- a spell while you are auto-casting/shooting.
   if CastPartyConfig.stop_autocast == nil then
      CastPartyConfig.stop_autocast = true;
   end

   -- specify the multiplier for the spacing inbetween the frames.
   -- this applies to spacing between players.
   if CastPartyConfig.frameSpacing == nil then
      CastPartyConfig.frameSpacing = 1.0;
   end

   -- ------------------------------------------------------------------------------------
   -- ADVANCED HEALING CONFIGURATION
   -- This section is also called the Healomatic code. It offers multiple ways of
   -- choosing the best spell while conserving mana when possible:'
   -- ------------------------------------------------------------------------------------

      -- This feature turns on the advanced spell selection when using
      -- ApplyHeal or DoTheRightThing. It will make spell decisions
      -- depending on your class. E.g. for priests it will use shield if the target is at low
      -- health and keep renew (healing over time) up. This feature is still in development
      -- stages, and may be faulty at times. Also note that there is no Shaman support yet.
      -- true:  uses advanced spell sequencing
      -- false: uses basic spell sequencing (as in CastParty 1.x)
      if CastPartyConfig.advanced_spells == nil then
         CastPartyConfig.advanced_spells = false;
      end

      -- This is an override feature, meaning if the unit health is not below
      -- spell_choice_threshold then we don't need to cast a healing spell.  In addtion
      -- it will not allow a heal greater then then the units max health to be selected
      -- usefull for thouse that set _spell_choice_threshold high to account for DPS or
      -- to force max healing spell rank base on mana only
      -- true:  turns addtional overheal protection on, by not healing thus saving mana
      -- false: turns the added protection off
      if CastPartyConfig.overheal_protection == nil then
         CastPartyConfig.overheal_protection = false;
      end

      -- When a shield is detected on the selected target, it controls wether or not
      -- to use an efficent heal.  If set to false it will use the targets health
      -- to choose the next spell, which could be an emergency spell depending on the
      -- health level.
      -- true:  force efficent heal when shield is detected
      -- false: allows normal spell selection to take place.
      if CastPartyConfig.shield_detection == nil then
         CastPartyConfig.shield_detection = true;
      end


      -- This allows a shield to be cast instead of a heal IF the target units health is
      -- critically low (see option below... it is ~15%). Some people are annoyed by this...
      -- true: automatically cast a shield instead of a heal if the target health is critical
      -- false: will not cast a shield if the target units health is critical
      if CastPartyConfig.shield_if_critical == nil then
         CastPartyConfig.shield_if_critical = true;
      end


      -- This is used to force Renew and Reju to cast the best regen spell based on mana
      -- and level of target, otherwise it will pick spells based on the previous two and
      -- include health of target.  Set it to true if you mainly use HoT spells on tanks,
      -- before a fight or when playing end game raids.
      -- true:  force highest rank spell for HoT's, based only on mana, and level of target
      -- false: base HoT's on health
      if CastPartyConfig.max_rank_hot == nil then
         CastPartyConfig.max_rank_hot = true;
      end

      -- This is used when casting faster firing spells while in combat when mana gets
      -- below this point we need to start saving mana before we run out of it completely,
      -- set to zero to disable this feature. If set at e.g. 0.45 CastParty will try to use
      -- mana efficient (usually slower) spells once your mana is below 45%.
      if CastPartyConfig.mana_conserve_threshold == nil then
         CastPartyConfig.mana_conserve_threshold = 0.45;
      end

      -- This sets the target heal threshold for bandages.  This is only useful for people
      -- with more then one type of bandage in their inventory - if you only have one type
      -- in your inventory it will use that one.  75% combind with a target natural regen
      -- seems about right.
      if CastPartyConfig.bandage_threshold == nil then
         CastPartyConfig.bandage_threshold = 0.75;
      end

      -- The setting only effects 'Do the right thing' and is a controling variable for
      -- pets by setting it to 1.2 the pet appear to have 20% more health then then it
      -- otherwise would, setting it at 0.8 would make a pet appear to have 20% less
      -- health. If you prefer healing pets, use a number <1, if you prefer healing players,
      -- keep it at >1.
      if CastPartyConfig.pet_health_value == nil then
         CastPartyConfig.pet_health_value = 1.2;
      end

   -- ------------------
   -- NOT IN OPTIONS YET
   -- ------------------

   --at least I don't think they are yet...

   --these values are specified HERE, then referenced in my code.
   if CastPartyConfig.max_debuffs == nil then
      CastPartyConfig.max_debuffs = 16;
   end
   if CastPartyConfig.max_buffs == nil then
      CastPartyConfig.max_buffs = 16;
   end

   -- Only used with advanced spells, this is used to control spells like Shields,
   -- nature's switfness when health is criticaly low, it will enable these spells
   -- into the spell selector.  Note this feature is not complete.
   if CastPartyConfig.critical_health_percent == nil then
      CastPartyConfig.critical_health_percent = 0.15;
   end


   -- ---------------------------------------------------------------------------
   -- NON CHANGED OPTIONS!
   -- These options are never changed by the CONFIG GUI
   -- ---------------------------------------------------------------------------

   --this is the default position of the minimap button. no need to be able to change this value
   -- as you can just unlock CP, and drag the button around the minimap! WAY COOL!
   if CastPartyConfig.MinimapButtonAngle == nil then
      CastPartyConfig.MinimapButtonAngle = -90;
   end

   if CastPartyConfig.base_healing == nil then
      CastPartyConfig.base_healing = 0;
   end

   --this is because WoW doesn't save our frame location!!
   if CastPartyConfig.Spot == nil then
      local centerOfScreen = { x=UIParent:GetRight()/2, y=UIParent:GetTop()/2 };
      CastPartyConfig.Spot = {};
      CastPartyConfig.Spot[0] = centerOfScreen;  --player frame
      CastPartyConfig.Spot[1] = centerOfScreen;  --party frame
      CastPartyConfig.Spot[2] = centerOfScreen;  --target frame
   end

   -- This toggles if I want to heal by clicking on people on the battlefield!!!
   if CastPartyConfig.useBattlefieldHeal == nil then
      CastPartyConfig.useBattlefieldHeal = false;
   end

  -- This toggles if I want to use the max rank of a heal NO MATTER WHAT!!!
   if CastPartyConfig.useMaxRankNoMatterWhat == nil then
      CastPartyConfig.useMaxRankNoMatterWhat = false;
   end


end