-- This file should contain the misc functions necessary in Healomatic as
-- well as the functions used to monitor the various actions/events that occur.


local auraBuffIndex
local untilCancelled

local auraDebuffTexture;
local auraDebuffApplications;

local fake_i
local auraDebuffName
local auraDebuffType

local auraBuffTexture
local auraBuffName

local oldTargetIsEmpty  = false;
local playerIsAttacking = false;
local retargetOldTarget = false;

local bagItemLink;

local frameName;
local auraFrameName;
local unit;
local size;

--[[***************************************************************************
*
*                  A U R A   U P D A T E   F U N C T I O N S
*
***************************************************************************]]--

--***************************************************************************
--***************************************************************************
-- called by UNIT_AURA
function CastParty_UpdateAuras(id)
   CastParty_Tooltip:SetOwner(UIParent, 'ANCHOR_NONE');  --must have this here. weird things happen to the auras if you don't...

   local unit = CP_units[id];

   if not UnitExists(unit) then
      CastParty.Debug('event', '<CastParty><UpdateAuras> no unit???', unit)
      return;
   end

   if not CastParty_Tooltip then
      CastParty.Debug('event', '<CastParty><UpdateAuras> no tooltip???')
      return;
   end

   CastParty.Debug('event', '<CastParty><UpdateAuras>', unit)

   --reset buff info...
   local size = table.getn(CastParty.State.ills[unit]);
   for index = size, 1, -1 do
      CastParty.Debug('event', '<UpdateAuras>Removing aura from ills', unit, index, CastParty.State.ills[unit][index])
      CastParty.State.ills[unit][index] = nil;
		table.setn(CastParty.State.ills[unit], size-1);
		size = size -1;
   end

   --reset buff info...
   size = table.getn(CP_auras[unit]);
   for index = size, 1, -1 do
      CastParty.Debug('event', '<UpdateAuras>Removing aura from CP_auras', unit, index, CP_auras[unit][index].name)
      CP_auras[unit][index] = nil;
		table.setn(CP_auras[unit], size-1);
		size = size -1;
   end

   for i = 1, CastPartyConfig.max_debuffs do
      if id == 0 then
         fake_i = i-1;
         auraBuffIndex, untilCancelled = GetPlayerBuff(fake_i, 'HARMFUL');
         auraDebuffTexture             = GetPlayerBuffTexture(auraBuffIndex);
         auraDebuffApplications        = GetPlayerBuffApplications(auraBuffIndex);
         auraDebuffType                = GetPlayerBuffDispelType(auraBuffIndex)
      else
         fake_i = i;
         auraDebuffTexture, auraDebuffApplications, auraDebuffType  = UnitDebuff(unit, fake_i);
      end
      if auraDebuffTexture then
         --note(id.. '##debuff ' .. i..auraDebuffTexture)
         CastParty_TooltipTextLeft1:SetText("");
         CastParty_TooltipTextRight1:SetText("");

         if id == 0 then
            CastParty_Tooltip:SetPlayerBuff(auraBuffIndex);
         else
            CastParty_Tooltip:SetUnitDebuff(unit, fake_i);
         end

         auraDebuffName = CastParty_TooltipTextLeft1:GetText();

         -- record ailments for curing purposes
         if auraDebuffType and CastParty.Brain.curables[auraDebuffType] then
            table.insert(CastParty.State.ills[unit], CastParty.Brain.curables[auraDebuffType]);
         end

         -- if CastParty_Debug then CastParty_Debug( 'event', '<UpdateAuras><Add Debuff>', unit, auraDebuffName) end;
         --here we add ALL buffs, so that we can scan all of them, instead of just the ones we can see.
         if auraDebuffName then
            table.insert(CP_auras[unit],
            {
              icon = auraDebuffTexture,
              debuff_index = fake_i,
              overlay = 'Interface\\Buttons\\UI-Debuff-Border',
              debuffCount = auraDebuffApplications,
              name = auraDebuffName,
              type = auraDebuffType,
              buffFilter = 'HARMFUL',
              show = false;
            })
         end
      end
   end

   for i = 1,CastPartyConfig.max_buffs do
      if id == 0 then
         fake_i = i-1;
         auraBuffIndex, untilCancelled = GetPlayerBuff(fake_i, 'HELPFUL');  -- removed |PASSIVE
         auraBuffTexture               = GetPlayerBuffTexture(auraBuffIndex);
         auraDebuffApplications        = GetPlayerBuffApplications(auraBuffIndex);
         auraDebuffType                = GetPlayerBuffDispelType(auraBuffIndex)
      else
         fake_i = i;
         auraBuffTexture, auraDebuffApplications, auraDebuffType = UnitBuff(unit, fake_i);
      end

      if auraBuffTexture then
         --note(id..'##buff ' .. i .. auraBuffTexture)

         if id == 0 then
            CastParty_Tooltip:SetPlayerBuff(auraBuffIndex);
         else
            CastParty_Tooltip:SetUnitBuff(unit, fake_i);
         end

         auraBuffName = CastParty_TooltipTextLeft1:GetText();

         -- if CastParty_Debug then CastParty_Debug( 'event', '<UpdateAuras><Add Buff>', unit, auraBuffName) end;
         --here we add ALL buffs, so that we can scan all of them, instead of just the ones we can see.

         -- record enemy buffs for curing purposes
         if auraDebuffType and CastParty.Brain.curables[auraDebuffType] and id == 90 then
            table.insert(CastParty.State.ills[unit], CastParty.Brain.curables[auraDebuffType]);
         end

         if auraBuffName then
            table.insert(CP_auras[unit],
            {
             icon = auraBuffTexture,
             overlay = 'Interface\\AddOns\\CastPartyGUI\\Images\\CastParty_Buff-Border',
             debuffCount = auraDebuffApplications,   --debuffApplications is ALSO used for innerfire.
             buff_index = fake_i,
             name = auraBuffName,
             type = auraDebuffType,
             buffFilter = 'HELPFUL',
             show = false;
            })
         end
      end
   end

end

--***************************************************************************
--***************************************************************************
-- Currently used by:
--    CastParty_ClickAura()
--    CastParty_ClickPetAura()
--

-- I can use this function to have my rank limiter on.
function CastParty_ChooseBuffRank(_unit, _buffName, _highRank, _lowRank)
   local lowRank;
   local highRank;

   if _lowRank ~= nil then
      lowRank = _lowRank;
   else
      lowRank = 1;
   end
   
   --[[
   if in class_buffs, use that one
   get highest rank of the spell, cast that one if target is self
   --]]

   if _highRank ~= nil then
      highRank = _highRank;
   else
      if CastParty.Brain.class_buffs[_buffName] then
        highRank = table.getn(CastParty.Brain.class_buffs[_buffName]);
      else
        -- try high rank
        highRank = string.gfind(CastParty.Brain.best_spell_ids[_buffName].rank, "%d+")()
      end
   end
   
   -- check to see if we have the buffs requested
   if not UnitExists(_unit)
   or not CastParty.Brain.class_buffs
   or not CastParty.Brain.class_buffs[_buffName] then
      CastParty.Debug(2, 'Unable to find buff rank:', 'u', _unit, 'buffName', _buffname, 'hi', _highRank, 'lo', _lowRank);
      return;
   end

   local unitLevel = UnitLevel(_unit);
   local spellRank;

   local spellFullName;
   local spellBuffLevel;

   -- this tests for the spell, starting at highest rank and see if it will work.
   -- then it works it's way down to rank 1.
   -- if none are found, then it just returns the buff name (ie, there is no rank)
   for i = highRank, lowRank, -1 do
      spellRank = i;

      spellFullName  = _buffName .. '(' .. CASTPARTY_RANK .. ' ' .. spellRank .. ')';
      spellBuffLevel = CastParty.Brain.class_buffs[_buffName][i];

      CastParty.Debug( 1, RED..'Checking spell rank vs target unit level', spellFullName, 'bufflvl', spellBuffLevel, 'unitlvl', unitLevel )
      -- if we find the spell (it actually exists) then we return the highest rank, as found here.
      if CastParty.Brain.spell_ids[spellFullName] and spellBuffLevel and (spellBuffLevel - unitLevel <= 10) then
         CastParty.Debug( 1, ORN, 'passed: ', spellFullName, spellBuffLevel, spellBuffLevel - unitLevel <= 10 )
         return _buffName .. '(' .. CASTPARTY_RANK .. ' ' .. spellRank .. ')';
      elseif CastParty.Brain.spell_ids[spellFullName] then
         CastParty.Debug(1, ORN, 'failed: ', CastParty.Brain.spell_ids[spellFullName].name, CastParty.Brain.spell_ids[spellFullName].rank)
      end
   end
   
   CastParty.Debug( 1, RED..'Failed spell rank checks, selected buff: ', _buffName)
   return _buffName;
end


--***************************************************************************
--***************************************************************************
-- abstracted from CastParty_ClickAura() because this function can be used more
-- than just once...
function CastParty_CastBuff(_buffName, _unit, _unitID)
   local spellFullName = CastParty_ChooseBuffRank(_unit, _buffName);

   CastParty.Debug( 3, ORN..'Using spell', spellFullName )

   CastParty.State.spell_being_cast = nil;

   CastParty.State.spell_being_cast = CastParty.Brain.spell_ids[spellFullName];

   -- Some buffs (e.g. Salvation) do not have a rank.
  if not CastParty.State.spell_being_cast and CastParty.Brain.best_spell_ids[_buffName] then
     CastParty.State.spell_being_cast = CastParty.Brain.best_spell_ids[_buffName];
  end

   -- spell doesn't exist...so return
   if not CastParty.State.spell_being_cast
   or not CastParty.State.spell_being_cast.id then
      CastParty.Debug( 4, ORN..'Failed to cast buff', _buffName, _unit, _unitID, spellFullName )
      return;
   end

   if CastParty_UpdateUnitPing then
      CastParty_UpdateUnitPing(1, _unitID);
   end

   CastParty_Retarget(_unit);

   CastSpell(CastParty.State.spell_being_cast.id, SpellBookFrame.bookType);
   SpellTargetUnit(_unit);

   CastParty_Retarget('SET');
   return
end


--***************************************************************************
--***************************************************************************
-- abstracted from a lot of code. this is to make it's behavior consistant
-- this function is called when you cast a spell, or bandage.

-- to solve the autoattack isssues, we DON'T change our target UNLESS our target is friendly AND we click on someone NOT our target!
-- this is because... ALL spells that can be cast on a friendly, CANNOT be cast on a enemy!
-- (except dispell magic and devour magic and...??) in which case you must switch targets first.

function CastParty_Retarget(_unit)

   if _unit ~= 'SET' then
      retargetOldTarget   = false;
      oldTargetIsEmpty    = false;

      if CastParty.State.usingBattlefieldHeal and UnitIsUnit(_unit, 'target') then
         CastParty.Debug(3, GRY..'BATTLEFIELD heal used!')
         retargetOldTarget = true;
      end

      if not UnitIsUnit(_unit, 'target') then
         if UnitIsEnemy('target', 'player') then
            if CastParty.State.spell_being_cast and CastParty.State.spell_being_cast.name and
                  CastParty.State.spell_being_cast.name == CASTPARTY_CURE_MAGIC_1 or
                  CastParty.State.spell_being_cast.name == CASTPARTY_CURE_MAGIC_2 or
                  CastParty.State.spell_being_cast.name == CASTPARTY_HEAL_HOLY_SHOCK
            then
               --if the unit we clicked on is not the target AND the spell being cast is dispel/devour magic.
               retargetOldTarget = true;
               if CastParty.State.isMeleeAttacking then CastParty.State.wasMeleeAttack = true end;
               TargetUnit(_unit);
            end
         else   --target is NOT the enemey
            if UnitExists('target') then
               --we have a friendly/neutral unit as target, and we heal/do something on someone else.
               retargetOldTarget = true;
               if not UnitExists('target') then oldTargetIsEmpty = true end;  --also, the case where the target can be nothing.
               if CastParty.State.isMeleeAttacking then CastParty.State.wasMeleeAttack = true end;
               TargetUnit(_unit);
            end
         end
      end

      if CastParty_Debug then CastParty_Debug(3, GRY..'GETTING Unit', _unit, 'Retarget Old Target', retargetOldTarget ) end;

   else
      if CastParty_Debug then CastParty_Debug(3, GRY..'SETTING prev unit') end;

      if oldTargetIsEmpty then
         ClearTarget();
      elseif retargetOldTarget then
         -- we clicked on someone that wasn't the target with a spell that MUST target them to cast...
         TargetLastTarget();
      end
   end

   CastParty_TargetingClear();
end


function CastParty_TargetingClear()
   if SpellIsTargeting() then
      local tempSpell = CastParty.State.spell_being_cast;
      SpellStopTargeting();
      CastParty.State.spell_being_cast = tempSpell;
   end
end


--***************************************************************************
--***************************************************************************
-- This is a gernal function to get item loctions from inventory
-- could be used in checking for seeds for druid res so they
-- are properly selected
--
-- Currently used by:
--    CastParty.Healomatic.ApplyBandage
--
function CastParty.GetContainerItem(_name)
   -- Search for the item link from name

   for bag = 0, 4, 1 do
      for item = 1, GetContainerNumSlots(bag), 1 do
         bagItemLink = GetContainerItemLink(bag, item);
         if not bagItemLink then
            --CastParty_Debug( 1, 'found non-link bag item in bag',bag,' item', item)
         end
         if bagItemLink then
            -- need to try string.find might be a direct compare, instead of a partial match
            if bagItemLink == _name or string.find(bagItemLink, _name) then
               CastParty.Debug(1, 'found item in bag', _name, bag, item)
               
               -- issues with == not always finding correct banagaes had to resort to strfind
               -- would like to revist this later on after a few more patches from blizzard
               -- if string.find( bagItemLink, 'Heavy' ) then
                  -- if string.find( _name, 'Heavy' ) then
                     -- return bag, item;
                  -- end
               -- else
                  return bag, item;
               -- end
            end
         end
      end
   end

   return nil, nil;
end


-- this allows you to use a trinket or other item BY NAME.. :)
-- as per the suggestions in the forum
-- this is for use in click keybindings.
function CastParty_UseEquipmentByName(_itemName, _unit)
   local equipmentName;
   for slotId = 0, 23 do -- 0 to 23 slots are equipment
      equipmentName = GetItemInfo(GetInventoryItemLink('player',slotId));
      if _itemName == equipmentName then
         if GetInventoryItemCooldown('player', slotId) == 0 then
            CastParty_Retarget(_unit);
            UseInventoryItem(slotId);
            CastParty_Retarget("SET");
         end
      end
   end
end


--***************************************************************************
--***************************************************************************
-- used in keybinding to automatically buff everyone in your group and/or target.
-- NOTE: only does one buff per keystroke.
function CastParty_Buffomatic()
   for i=0,90 do
      frameName = getglobal('CastPartyMember'..i);
      auraFrameName = getglobal('CastPartyMember'..i..'Aura1');
      unit = CP_units[i];
      if UnitExists(unit) and auraFrameName and auraFrameName.watching and
         (frameName.range and frameName.range < 50) then
         CastParty_CastBuff(auraFrameName.name, unit, i)
      end
   end
end

--***************************************************************************
--***************************************************************************
-- This is a global test, on the target that determins if the targeted
-- unit is healable, also the unitneedsheal function has been incoperated
-- into this function as well, if was found that in most cases these two
-- functions were used together very offten, and didn't warrent making
-- the program any more complex by having more functions to call
--
-- Currently used by:
--    DoTheRightThing
--    AdvancedHeals
--    CastParty_TestGroupHeal
--    CastParty_UI_ERROR
--
function CastParty_UnitIsHealable(_unit)

   if not _unit then
      return false;
   end

   if not UnitExists(_unit) then
      if CastParty_Debug then CastParty_Debug(2, RED.._unit, 'does not exist') end;
      return false;
   end

   if not UnitIsFriend('player', _unit) then
      if CastParty_Debug then CastParty_Debug(2, RED.._unit, 'is not a friend') end;
      return false;
   end

   if UnitIsEnemy(_unit, 'player') then
      if CastParty_Debug then CastParty_Debug(2, RED.._unit, 'is an enemy') end;
      return false;
   end

   if UnitCanAttack(_unit, 'player') then
      if CastParty_Debug then CastParty_Debug(2, RED.._unit, 'can attack player') end;
      return false;
   end

   if UnitCanAttack('player', _unit) then
      if CastParty_Debug then CastParty_Debug(2, RED.._unit, 'can be attacked by player') end;
      return false;
   end

   if not _unit == 'player' and not UnitCanCooperate('player', _unit) then
      if CastParty_Debug then CastParty_Debug(2, RED.._unit, 'cannot cooperate with player') end;
      return false;
   end

   if not UnitIsConnected(_unit) then
      if CastParty_Debug then CastParty_Debug(2, RED.._unit, 'is not connected') end;
      return false;
   end

   if CastParty_Debug then CastParty_Debug(1, GRN.._unit, 'is healable or can be resurrected') end;
   return true;

end

--***************************************************************************
--***************************************************************************
-- scan through the list and check for a valid member of party or raid
-- this should also work for pets
--
function UnitInGroup(_unit)
   if _unit == 'player' then
      if GetNumPartyMembers() > 0 or GetNumRaidMembers() > 0 then
         --if CastParty_Debug then CastParty_Debug(4, 'check if unit in group: TRUE') end;
         return true;
      else
         --if CastParty_Debug then CastParty_Debug(4, 'check if unit in group: FALSE') end;
         return false;
      end
   elseif UnitInParty(_unit) or UnitInRaid(_unit) or string.find(_unit, 'pet') or string.find(_unit, 'minion') or
   (_unit == 'mouseover' and (GetNumPartyMembers() > 0 or GetNumRaidMembers() > 0)) then
      --if CastParty_Debug then CastParty_Debug(4, 'check if unit in group: TTRUE') end;
      return true;
   end
   --if CastParty_Debug then CastParty_Debug(4, 'check if unit in group: FFALSE') end;
   return false;
end