-- This file contains information for the base class and construction of all player types
--DEFAULT_CHAT_FRAME:AddMessage("loading brain.lua")

CastParty.BrainFactory = {}
CastParty.BrainBase = 
{
  className = "Undefined", -- subclasses should set this field
  heal_types = {}, -- mapping of healing spell lines to casting categories
  
  --TODO: index this table by {n="name",r="rank"} instead of "name ('Rank' rank)"
  spell_ids = {},
  heal_data = {}, -- storage for spell data used in determining appropriateness
  heal_multipliers = {},
  -- set spells that do not require level checks. Level 1 default so the level check will always pass
  class_buffs = {}, -- things you can cast on others
  best_spell_ids = {},
  healing_bonus_multipliers = {},
  curables = {},
  cures = {}, -- table of cure spells to a set of what they cure
  
  spell_parsers = {}, -- mapping of healing spell lines to heal amount parsers
  reagents = {},
  
  -- Default key bindings. Specific brains may override.
  default_key_bindings = {
     LeftButton  = {
        None              = 'TargetUnit',
        Shift             = 'AssistUnit',
        },
     RightButton = {
        None              = 'CastParty_PartyDropDown',
        },
     MiddleButton  = {
        None              = 'ApplyBandage',
        },
     Button4 = {
        None              = 'TargetUnit',
        Shift             = 'CastParty_PartyDropDown',
        },
     Button5 = {
        None              = 'AssistUnit',
        },
  }

   --[[
      Here is the default template used, in case you really want to change things.
      *****************************************************************************
      
      default_key_bindings = {
            LeftButton  = {
               None              = nil,
               Alt               = nil,
               AltCtrl           = nil,
               AltCtrlShift      = nil,
               AltShift          = nil,
               Ctrl              = nil,
               CtrlShift         = nil,
               Shift             = nil,
               },
      
            RightButton = {
               None              = nil,
               Alt               = nil,
               AltCtrl           = nil,
               AltCtrlShift      = nil,
               AltShift          = nil,
               Ctrl              = nil,
               CtrlShift         = nil,
               Shift             = nil,
               },
      
            MiddleButton  = {
               None              = nil,
               Alt               = nil,
               AltCtrl           = nil,
               AltCtrlShift      = nil,
               AltShift          = nil,
               Ctrl              = nil,
               CtrlShift         = nil,
               Shift             = nil,
               },
      
            Button4 = {
               None              = nil,
               Alt               = nil,
               AltCtrl           = nil,
               AltCtrlShift      = nil,
               AltShift          = nil,
               Ctrl              = nil,
               CtrlShift         = nil,
               Shift             = nil,
               },
      
            Button5 = {
               None              = nil,
               Alt               = nil,
               AltCtrl           = nil,
               AltCtrlShift      = nil,
               AltShift          = nil,
               Ctrl              = nil,
               CtrlShift         = nil,
               Shift             = nil,
               },
          }
   ]]
}

-- functions for parsing healing info from tooltip
-- unfortunately lua requires functions be defined before referring to them
function CastParty.parse_number(_text, _index)
   if not _text then return -1 end

   local i = 1
   for number in string.gfind(_text, '%d+[.,]?%d*') do
      if i == _index then
         return tonumber(number) or 0;  -- in case the number doesn't exist.
      end
      i = i + 1
   end

end

function CastParty.BrainFactory.new(className)
  local brain = nil
  
  if CastParty.BrainFactory[className] and CastParty.BrainFactory[className].new then
    brain = CastParty.BrainFactory[className].new()
  end
  
  if brain then
    -- add default versions for any brain that doesn't override functions
    setmetatable(brain, {__index = CastParty.BrainBase})
    
    if not brain.GetHealAmount then
      brain.GetHealAmount = function (spellName)
        CastParty.BrainBase.GetHealAmount(brain, spellName)
      end
    end
    return brain
  end
  
  --note("unable to find: CastParty.BrainFactory."..className..".new()")
  
  CastParty.Debug(1, "CastParty doesn't know how to make a brain for "..className.. ", using default logic");
  for key in CastParty.BrainFactory do
    CastParty.Debug(1, key)
  end
  return nil
end

function CastParty.BrainBase.GetHealAmount(spellEntry, brain)
  --CastParty.Debug(1, "calling base.GetHealAmount")
  local spellName = spellEntry.name
  if not brain then
    brain = CastParty.BrainBase
  end
  
  if not brain.spell_parsers or not brain.spell_parsers[spellName] then
    Castparty.Debug(4, "Call to "..brain.className..".GetHealAmount had no amount for: "..spellName)
    return 0
  end
  
   --HACK: whatever classes use this spell, this should be in their brain somehow
   --      Spells with reagents, when reading spell data, the reagent will be tooltip 4 and
   --      the spell description text will be tooltip 5
   local text = CastParty_TooltipTextLeft4:GetText()
   if spellName == CASTPARTY_RES_REBIRTH then
      text = CastParty_TooltipTextLeft5:GetText();
   end
   
  local which = brain.spell_parsers[spellName];
  local spellHealAmount
  if which < 10 and which > 0 then
     spellHealAmount = CastParty.parse_number(text, which);
  elseif which > 0 and which < 100 then
     local ten = floor(which / 10)
     local one = which - (ten*10)
     CastParty.Debug(1, "parsed double-digit spell ", spellName, " ten:", ten, " one:", one)
     
     -- add the number of the tens digit with the one of the ones digit
     spellHealAmount = CastParty.parse_number(text, ten) + CastParty.parse_number(text, one)
  else
     note(RED.. "Unrecognized spell parser data for ", spellName, ": ", which);
  end

  if brain.heal_multipliers[spellName] then
    return spellHealAmount * brain.heal_multipliers[spellName];
  end
  return spellHealAmount
end

function CastParty.BrainBase.ReloadSpellData(brain)
   if not CastParty.Debug then
     CastParty.Debug = function()end
   end
   CastParty.Debug(1, GRY.."calling BrainBase.reloadSpellData")
   
   if not brain then
     CastParty.Debug(1, RED.."No brain provided to reload spell data")
     return false
   end
   
   --i have to initialize the tooltip here (1.10 stuff...)
   CastParty_Tooltip:SetOwner(UIParent, 'ANCHOR_NONE');
   
   -- Clear the tables before reload so they don't just keep growing
   brain.heal_data = {}
   brain.spell_ids = {}
   brain.best_spell_ids = {}

   local i = 1

   CastParty.Debug(2, RED..'CastPary SpellBook Parser')
   CastParty.Debug(2, RED..'Loading heal data...')
   
   -- Load spell names
   local spells = {}
   while true do
      local spellName, spellRank = GetSpellName(i, SpellBookFrame.bookType)

      if not spellName then
         --if CastParty.Debug then CastParty.Debug( 2, RED..'No spell Name!! Most likely the end of the list!') end;
         break;
      end
      
      --TODO: need to parse the spell rank here and use everywhere else as an int
      --      also, parse the word Rank and assign to the localized "Rank" string to skip specifying it
      --spellRank = string.gfind(spellRank, "%d+")
      --CastParty.Debug(2, YEL..'found spell '..spellName..":"..spellRank)
      spells[i] = {name=spellName, rank = spellRank, id = i }
      i = i + 1
   end
   
   --if CastParty.Debug then CastParty.Debug( 2, RED..'Loaded No spell Name!! Most likely the end of the list!') end;
   CastParty.Debug(3, "Loaded spells, iterating through")
   
   local spellEntry
   for index,spellEntry in ipairs(spells) do
      --CastParty.Debug(2, GRY.." processing spell: "..spellEntry.name.."("..spellEntry.rank..')['..spellEntry.id..']')
      
      local spell_ID_key = spellEntry.name
      
      if spellEntry.rank == "" then
        spellEntry.rank = 0
      else
        spell_ID_key = spellEntry.name .. '(' .. spellEntry.rank .. ')'
      end
      
      brain.spell_ids[spell_ID_key] = spellEntry;
      -- The last one for the given name is the best, replace previous mappings
      brain.best_spell_ids[spellEntry.name] = spellEntry;

      -- get healing data
      local spellType = brain.heal_types[spellEntry.name]
      if not spellType then
        --CastParty.Debug(1, GRN..'unknown spell type for spell ', spellEntry.name)
      end
      if spellType then
        --CastParty.Debug( 1, YEL..'processing healing spell', spellEntry.id, spellEntry.name, spellEntry.rank, spellType )
        CastParty_Tooltip:SetSpell(spellEntry.id, SpellBookFrame.bookType)
        local spellManaCost = CastParty_TooltipTextLeft2:GetText()
        --CastParty.Debug(3, BLU..'spell mana:'..spellEntry.name..':'..tostring(spellManaCost))
        --local spellHealAmount = 1

        if not spellManaCost then
           CastParty.Debug(2, RED..'Spell had no mana', spellEntry.name )
           return false;
        end

        if string.find(spellManaCost, 'Mana') then
           _,_,spellManaCost = string.find(spellManaCost, '(%d+)');
        else
           spellManaCost = false;
        end

        local spellHealAmount = brain.GetHealAmount(spellEntry)
        
        if not brain.heal_data[spellType] then
           brain.heal_data[spellType] = { };
        end
        
        if not spellHealAmount then
          spellEntry.max_heal = tonumber(spellHealAmount)
          spellEntry.mana = tonumber(spellManaCost)
        else
          spellEntry.max_heal = math.floor(tonumber(spellHealAmount));
          spellEntry.mana = tonumber(spellManaCost)
        end

        -- if spells fail to load, don't keep trying, return false and let the event handler
        -- re-try loading the spells.  This function is being called on player entering world
        -- we can't use ADDON_LOAD that only calls once, becuase data is sometimes cleared on
        -- entering instances/zoning.  Sometimes this is being called before spell TooltipText
        -- is initialized, the OnEvent function will try again on each event until loaded
        -- correctly

        if not spellEntry.mana or spellEntry.max_heal == -1 then
           CastParty.Debug( 2, RED..'Spell loading failed : there is no mana data OR there is no heal data')
           CastParty.Debug( 2, RED..spellEntry.max_heal, spellEntry.name , spellEntry.rank, 'mana', spellEntry.mana )
           return false;
        else
           --CastParty.Debug( 2, GRY..'       Loaded', spellEntry.name, spellEntry.rank )
           table.insert(brain.heal_data[spellType], spellEntry);
        end

        brain.spell_ids[spell_ID_key]   = spellEntry;
        brain.best_spell_ids[spellEntry.name] = spellEntry;
      end
   end
   
   brain:SortSpells()
   brain:LoadCurables()
   
   return true;
end

function CastParty.BrainBase.SortSpells(brain)
   -- sort the heal spells by mana cost within each spell type
   local key1, value1, key2, value2
   for key1, value1 in brain.heal_data do
     CastParty.Debug(2, RED..key1)
     for key2, value2 in value1 do
        if value2.name == CASTPARTY_RES_ANCESTRAL_SPIRIT or
           value2.name == CASTPARTY_RES_RESURRECTION or
           value2.name == CASTPARTY_RES_REBIRTH or
           value2.name == CASTPARTY_RES_REDEMPTION then
           table.sort(value1, function(a,b) return a.rank < b.rank end);
        else
           table.sort(value1, function(a,b) return a.mana < b.mana end);
        end
        --CastParty.Debug(2, WHT..value2.name , value2.rank, BLU.. '  Mana', value2.mana , GRN..'  Heal Amt',value2.max_heal )
     end
   end
   
   CastParty.Debug(1, GRN.. 'done sorting')
end

function CastParty.BrainBase.LoadCurables(brain)
   -- fill in my curables as well
   brain.curables = {}

   for _, value1 in pairs(brain.cures) do
      cureName   = value1.spell
      cures      = value1.cures

      if brain.best_spell_ids[cureName] then
         CastParty.Debug(2, RED..'Cure', GRY..cureName)

         for _, value2 in pairs(cures) do
            brain.curables[value2] = cureName
            CastParty.Debug(2, RED..'       ', GRY..value2)
         end
      end
   end

   CastParty.Debug( 3, 'Spells loaded by: ', brain.className )
end
