-- Make a class for Druid-based constants and funcitonality
-- load this class when castparty starts and detects the player's class as druid
-- use an instance to call desired overloaded methods

local DRUID_NAMESPACE = {} -- declare the namespace
CastParty.BrainFactory.DRUID = DRUID_NAMESPACE

--DEFAULT_CHAT_FRAME:AddMessage("loading brainDruid.lua")

function DRUID_NAMESPACE.new()
  CastParty.Debug(4, "making a new druid brain")
  
  DRUID_NAMESPACE.className = 'DRUID'
  
  DRUID_NAMESPACE.heal_types = {
      [CASTPARTY_HEAL_HEALING_TOUCH]          = 'efficient',
      [CASTPARTY_HEAL_REJUVENATION]           = 'instant',
      [CASTPARTY_HEAL_REGROWTH]               = 'emergency',
      [CASTPARTY_HEAL_TRANQUILITY]            = 'group',
      [CASTPARTY_RES_REBIRTH]                 = 'resurrection',
  }
  
  DRUID_NAMESPACE.class_buffs = {
      [CASTPARTY_HEAL_HEALING_TOUCH]          = {1,1,1,1,1,1,1,1,1,1,1},
      [CASTPARTY_HEAL_REJUVENATION]           = {4,10,26,22,28,34,40,46,52,58,60},
      [CASTPARTY_HEAL_REGROWTH]               = {12,18,24,30,36,42,48,54,60},
      [CASTPARTY_HEAL_TRANQUILITY]            = {1,1,1,1},
      [CASTPARTY_RES_REBIRTH]                 = {1,1,1,1,1},
      
      -- non-heal/shield buffs
      [CASTPARTY_BUFF_MOTW]                   = {1,10,20,30,40,50,60},
      [CASTPARTY_BUFF_THORNS]                 = {6,14,24,34,44,54},
      [CASTPARTY_BUFF_SWIFTMEND]              = {1},
  }
  
  DRUID_NAMESPACE.heal_multipliers = {
  }
  
  DRUID_NAMESPACE.spell_parsers = {
      [CASTPARTY_HEAL_HEALING_TOUCH]          = 2,
      [CASTPARTY_HEAL_REJUVENATION]           = 1,
      [CASTPARTY_HEAL_REGROWTH]               = 13, --first_plus_third_number,
      [CASTPARTY_HEAL_TRANQUILITY]            = 1,
      [CASTPARTY_RES_REBIRTH]                 = 2,
      [CASTPARTY_BUFF_SWIFTMEND]              = 2,
  }
  
  if GetLocale() == 'deDE' then
     DRUID_NAMESPACE.spell_parsers[CASTPARTY_HEAL_REJUVENATION] = 2
  end
  
  DRUID_NAMESPACE.healing_bonus_multipliers = {
      [CASTPARTY_HEAL_HEALING_TOUCH]          = {.107,.286,.536,.857,1,1,1,1,1,1,1},
      [CASTPARTY_HEAL_TRANQUILITY]            = {.333,.333,.333,.333},
      [CASTPARTY_HEAL_REJUVENATION]           = {.4,.6,.8,.8,.8,.8,.8,.8,.8,.8,.8},
      [CASTPARTY_HEAL_REGROWTH]               = {.262,.524,.786,.786,.786,.786,.786,.786,.786},
  }
  
   DRUID_NAMESPACE.cures = {
      { spell = CASTPARTY_CURE_POISON_1,  cures = { CASTPARTY_AILMENT_POISON  } },
      { spell = CASTPARTY_CURE_POISON_2,  cures = { CASTPARTY_AILMENT_POISON  } },
   }
   
   DRUID_NAMESPACE.reagents = {
      { name = CASTPARTY_REA_MAPLE_SEED,         rank = 1 },
      { name = CASTPARTY_REA_STRANGLETHORN_SEED, rank = 2 },
      { name = CASTPARTY_REA_ASHWOOD_SEED,       rank = 3 },
      { name = CASTPARTY_REA_HORNBEAM_SEED,      rank = 4 },
      { name = CASTPARTY_REA_IRONWOOD_SEED,      rank = 5 },
   }
   
   DRUID_NAMESPACE.default_key_bindings = {
      LeftButton  = {
         None              = 'ApplyHeal',
         Alt               = 'ApplyEfficientHeal',
         AltCtrl           = 'TargetUnit',
         Ctrl              = 'ApplyInstantHeal',
         CtrlShift         = 'AssistUnit',
         Shift             = 'ApplyEmergencyHeal',
         },
      RightButton = {
         None              = 'Mark of the Wild',
         Ctrl              = 'Thorns',
         CtrlShift         = 'CureAny',
         Shift             = 'CureGroupMember',
         },
      MiddleButton  = {
         None              = 'DoTheRightThing',
         Shift             = 'ApplyBandage',
         },
      Button4 = {
         None              = 'TargetUnit',
         Shift             = 'CastParty_PartyDropDown',
         },
      Button5 = {
         None              = 'AssistUnit',
         },
   }

   return DRUID_NAMESPACE
end

--@Override
function DRUID_NAMESPACE.ReloadSpellData()
  CastParty.Debug(1, "calling druid.ReloadSpellData")
  return CastParty.BrainBase.ReloadSpellData(DRUID_NAMESPACE)
end

--@Override
function DRUID_NAMESPACE.GetHealAmount(spellEntry)
  --CastParty.Debug(1, 'calling druid.GetHealAmount')
  return CastParty.BrainBase.GetHealAmount(spellEntry, DRUID_NAMESPACE);
end

function DRUID_NAMESPACE.AdvancedHeal(_unit, _unitHealth, _unitHealthMax, _healType )
   local spell;
   local playerMana;
   local playerManaFraction;
   local unitHealthModified;
   local unitHealthFraction;
   local unitLevel;

   local _healType, playerMana, playerManaFraction, unitHealthModified, unitHealthFraction, unitLevel = CastParty.Healomatic.InitHealVars(_unit, _unitHealth, _unitHealthMax, _healType);

   if _healType and _healType == 'resurrection' then
      return CastParty.Healomatic.DoResurrect(_unit, unitLevel, playerMana)
   end
   
   -- make sure instant cast is running 1st, while instant cast is
   -- a minor heal, it is farly effecient, and is a low argo spell
   if ( _healType == 'instant' or _healType == "" ) then
      spell, _healType  = CastParty.Healomatic.DoInstantHeal(CASTPARTY_HEAL_REJUVENATION, _unitHealth, unitHealthModified, unitHealthFraction, _unitHealthMax, unitLevel, playerMana, _unit);
   end

   --this is so if we can cast a instant-cast, stop, otherwise we allow the program to move on...
   if spell then
      return spell;
   end

   if _healType == 'emergency' or _healType == "" then
      if CastParty_Debug then CastParty_Debug( 1, GRY..'begin emergency checks...') end;
      -- is a previous spell still running? if so skip and cast a normal spell
      -- no futher benfit can be gained from re-casting this spell, it would
      -- just be a waste of addtional mana, as the direct healing of the spell
      -- is rather minor
      if not CastParty.State.heal_states[CASTPARTY_HEAL_REGROWTH] and CastParty.Brain.heal_data['emergency'] then
         local Adjusted_health = 0;

         -- load up the spell for testing
         spell, Adjusted_health = GetSpell( _unitHealth, unitHealthModified, _unitHealthMax, 'emergency', unitLevel, playerMana, _unit );

         spell = CastParty.Healomatic.DoEmergencyInCombat(spell, _healType, unitHealthFraction, playerManaFraction, Adjusted_health, unitHealthModified);
      end

      --this is so if we can cast and emergency spell, do it, otherwise we allow the program to move on...
      if spell then
         return spell;
      end
   end

   -- always choose the best group heals based on mana
   if healType == 'group' and CastParty.Brain.heal_data[healType] then
      if CastParty_Debug then CastParty_Debug( 2, YEL..'Group heal!') end;
      return GetSpell( _unitHealth, unitHealthModified, _unitHealthMax, 'group', unitLevel, playerMana, _unit, true );
   end

   -- ok this is it, its time for the big guns of healing we don't have an
   -- instant to cast and dont have an emergency to cast, this is the last one
   if CastParty.Brain.heal_data['efficient'] then
      if CastParty_Debug then CastParty_Debug( 2, YEL..'Efficient heal!') end;
      return GetSpell( _unitHealth, unitHealthModified, _unitHealthMax, 'efficient', unitLevel, playerMana, _unit);
   end
end
