-- Make a class for Paladin-based constants and funcitonality
-- load this class when castparty starts and detects the player's class as Paladin
-- use an instance to call desired overloaded methods

local PALADIN_NAMESPACE = {} -- declare the namespace
CastParty.BrainFactory.PALADIN = PALADIN_NAMESPACE

--DEFAULT_CHAT_FRAME:AddMessage("loading brainPaladin.lua")

function PALADIN_NAMESPACE.new()
   CastParty.Debug(4, "making a new paladin brain")
   
   PALADIN_NAMESPACE.className = 'PALADIN'
   
   PALADIN_NAMESPACE.heal_types = {
       [CASTPARTY_HEAL_HOLY_LIGHT]             = 'efficient',
       [CASTPARTY_HEAL_FLASH_OF_LIGHT]         = 'emergency',
       [CASTPARTY_HEAL_HOLY_SHOCK]             = 'instant',
       [CASTPARTY_RES_REDEMPTION]              = 'resurrection',
       [CASTPARTY_BUFF_BLESS_OF_PROTECTION]    = 'shield',
       [CASTPARTY_BUFF_DIVINE_PROTECTION]      = 'shield',
       [CASTPARTY_BUFF_DIVINE_SHIELD]          = 'shield',
   }
   
   PALADIN_NAMESPACE.class_buffs = {
       [CASTPARTY_HEAL_HOLY_LIGHT]             = {1,1,1,1,1,1,1,1,1},
       [CASTPARTY_HEAL_FLASH_OF_LIGHT]         = {1,1,1,1,1,1},
       [CASTPARTY_HEAL_HOLY_SHOCK]             = {1,1,1},
       [CASTPARTY_RES_REDEMPTION]              = {1,1,1,1,1},
       
       [CASTPARTY_BUFF_BLESS_OF_PROTECTION]    = {10,24,38},
       [CASTPARTY_BUFF_DIVINE_PROTECTION]      = {6,18},
       [CASTPARTY_BUFF_DIVINE_SHIELD]          = {34,50},
       
       -- non-heal/shield buffs
       [CASTPARTY_BUFF_BLESS_OF_MIGHT]         = {4,12,22,32,42,52,60},
       [CASTPARTY_BUFF_BLESS_OF_WISDOM]        = {14,24,34,44,54,60},
       [CASTPARTY_BUFF_BLESS_OF_FREEDOM]       = {18},
       [CASTPARTY_BUFF_BLESS_OF_SALVATION]     = {26},
       [CASTPARTY_BUFF_BLESS_OF_KINGS]         = {20},
       [CASTPARTY_BUFF_BLESS_OF_LIGHT]         = {40,50,60},
       [CASTPARTY_BUFF_BLESS_OF_SANCTUARY]     = {20,30,40,50,60},
       [CASTPARTY_BUFF_BLESS_OF_SACRIFICE]     = {46,54},
       [CASTPARTY_BUFF_GREAT_BLESS_OF_LIGHT]   = {60},
       [CASTPARTY_BUFF_GREAT_BLESS_OF_SANCT]   = {60},
       [CASTPARTY_BUFF_GREAT_BLESS_OF_KINGS]   = {60},
       [CASTPARTY_BUFF_GREAT_BLESS_OF_MIGHT]   = {52,60},
       [CASTPARTY_BUFF_GREAT_BLESS_OF_WISDOM]  = {54,60},
       [CASTPARTY_BUFF_GREAT_BLESS_OF_SALV]    = {60},
   }
   
   PALADIN_NAMESPACE.spell_parsers = {
       [CASTPARTY_HEAL_HOLY_LIGHT]             = 2,
       [CASTPARTY_HEAL_FLASH_OF_LIGHT]         = 2,
       [CASTPARTY_HEAL_HOLY_SHOCK]             = 2,
       [CASTPARTY_RES_REDEMPTION]              = 2,
       [CASTPARTY_BUFF_BLESS_OF_PROTECTION]    = 2,
       [CASTPARTY_BUFF_DIVINE_PROTECTION]      = 2,
       [CASTPARTY_BUFF_DIVINE_SHIELD]          = 2,
   }
   
   PALADIN_NAMESPACE.healing_bonus_multipliers = {
       [CASTPARTY_HEAL_HOLY_LIGHT]             = {.238,.476,.714,.714,.714,.714,.714,.714,.714},
       [CASTPARTY_HEAL_FLASH_OF_LIGHT]         = {.429,.429,.428,.429,.429,.429},
       [CASTPARTY_HEAL_HOLY_SHOCK]             = {.333,.333,.333},
   }
   
   PALADIN_NAMESPACE.cures = {
       { spell = CASTPARTY_CURE_PURIFY_1,  cures = { CASTPARTY_AILMENT_DISEASE, CASTPARTY_AILMENT_POISON } },
       { spell = CASTPARTY_CURE_CLEANSE_1, cures = { CASTPARTY_AILMENT_DISEASE, CASTPARTY_AILMENT_POISON, CASTPARTY_AILMENT_MAGIC } },
   }
   
   PALADIN_NAMESPACE.default_key_bindings = {
     LeftButton  = {
        None              = 'ApplyHeal',
        Alt               = 'ApplyEfficientHeal',
        AltCtrl           = 'TargetUnit',
        Ctrl              = 'Blessing of Sacrifice',
        CtrlShift         = 'AssistUnit',
        Shift             = 'ApplyEmergencyHeal',
        },
     RightButton = {
        None              = 'CureAny',
        Alt               = 'Blessing of Protection',
        AltCtrl           = 'Blessing of Freedom',
        AltCtrlShift      = 'DoTheRightThing',
        Ctrl              = 'CastParty_PartyDropDown',
        Shift             = 'CureGroupMember',
        },
     MiddleButton  = {
        None              = 'Blessing of Might',
        Alt               = 'Blessing of Wisdom',
        Ctrl              = 'Blessing of Salvation',
        Shift             = 'ApplyBandage',
        },
     Button4 = {
        None              = 'TargetUnit',
        Shift             = 'CastParty_PartyDropDown',
        },
     Button5 = {
        None              = 'AssistUnit',
        },
   }   

   return PALADIN_NAMESPACE
end

--@Override
function PALADIN_NAMESPACE.ReloadSpellData()
  CastParty.Debug(1, "calling Paladin.ReloadSpellData")
  return CastParty.BrainBase.ReloadSpellData(PALADIN_NAMESPACE)
end

--@Override
function PALADIN_NAMESPACE.GetHealAmount(spellEntry)
   CastParty.Debug(1, 'calling Paladin.GetHealAmount', spellEntry.name, spellEntry.rank)
   if (spellEntry.name == CASTPARTY_BUFF_BLESS_OF_PROTECTION or
       spellEntry.name == CASTPARTY_BUFF_DIVINE_PROTECTION or
       spellEntry.name == CASTPARTY_BUFF_DIVINE_SHIELD)
   then
     CastParty.Debug(2, RED.. spellEntry.name, spellEntry.rank, tostring(spellEntry.mana), tostring(spellEntry.max_heal))
     CastParty.Debug(2, RED.. spellEntry.rank)
     CastParty.Debug(2, RED.. CastParty.parse_number(spellEntry.rank, 1))
     -- for some odd reason, gfind returns a function, so call it
     return (string.gfind(spellEntry.rank, "%d+")() or 0)* 1000;
   end
   
   CastParty.Debug(1, 'delegating Paladin.GetHealAmount to BrainBase')
   return CastParty.BrainBase.GetHealAmount(spellEntry, PALADIN_NAMESPACE)
end

-- Paladins only have two healing spells, and the flash heal is
-- more effecinet so in rare cases we can use flash heal to
-- save mana, not only that it is faster, so we can get back to
-- Slaying Dragons sooner :)
--
-- TODO: include the use of paladin shields, at critical times
--
function PALADIN_NAMESPACE.AdvancedHeal(_unit, _unitHealth, _unitHealthMax, _healType )
   local spell;
   local playerMana;
   local playerManaFraction;
   local unitHealthModified;
   local unitHealthFraction;
   local unitLevel;

   local _healType, playerMana, playerManaFraction, unitHealthModified, unitHealthFraction, unitLevel = CastParty.Healomatic.InitHealVars(_unit, _unitHealth, _unitHealthMax, _healType);

   if _healType and _healType == 'resurrection' then
      return CastParty.Healomatic.DoResurrect(_unit, unitLevel, playerMana)
   end
   
   --this will select the shield spell IF it's supposed to!
   spell, _healType = CastParty.Healomatic.DoTheShield(_unit, unitHealthModified, _unitHealthMax, unitHealthFraction, unitLevel, playerMana, _unit, _healType);

   --this is so if we can cast a shield, stop, otherwise we allow the program to move on...
   if spell then
      return spell;
   end

   -- paladin's only have 2 healing spells, and there smaller one is more
   -- mana efficient, and is faster to cast, so check to see if we can use
   -- it or not
   if _healType == 'emergency' or _healType == "" then
      if CastParty_Debug then CastParty_Debug( 1, GRY..'begin emergency checks...') end;
      -- is a previous spell still running? if so skip and cast a normal spell
      -- no futher benfit can be gained from re-casting this spell, it would
      -- just be a waste of addtional mana, as the direct healing of the spell
      -- is rather minor
      if CastParty.Brain.heal_data['emergency']then
         local Adjusted_health = 0;

         -- load up the spell for testing
         spell, Adjusted_health = GetSpell( _unitHealth, unitHealthModified, _unitHealthMax, 'emergency', unitLevel, playerMana, _unit );

         spell = CastParty.Healomatic.DoEmergencyInCombat(spell, _healType, unitHealthFraction, playerManaFraction, Adjusted_health, unitHealthModified);
      end

      --this is so if we can cast and emergency spell, do it, otherwise we allow the program to move on...
      if spell then
         return spell;
      end
   end

   -- ok this is it, its time for the big guns of healing we don't have any
   -- more spells to try, this is the last one
   if CastParty.Brain.heal_data['efficient'] then
      return GetSpell( _unitHealth, unitHealthModified, _unitHealthMax, 'efficient', unitLevel, playerMana, _unit);
   end

end
