-- Make a class for Priest-based constants and funcitonality
-- load this class when castparty starts and detects the player's class as priest
-- use an instance to call desired overloaded methods

local PRIEST_NAMESPACE = {} -- declare the namespace
CastParty.BrainFactory.PRIEST = PRIEST_NAMESPACE

--DEFAULT_CHAT_FRAME:AddMessage("loading brainPriest.lua")

function PRIEST_NAMESPACE.new()
  CastParty.Debug(4, "making a new priest brain")
  
  PRIEST_NAMESPACE.className = 'PRIEST'
  
  PRIEST_NAMESPACE.heal_types = {
      [CASTPARTY_HEAL_LESSER_HEAL]            = 'efficient',
      [CASTPARTY_HEAL_HEAL]                   = 'efficient',
      [CASTPARTY_HEAL_GREATER_HEAL]           = 'efficient',
      
      [CASTPARTY_HEAL_FLASH_HEAL]             = 'emergency',
      [CASTPARTY_HEAL_RENEW]                  = 'instant',
      [CASTPARTY_HEAL_PRAYER_OF_HEALING]      = 'group',
      
      [CASTPARTY_BUFF_POWER_WORD_SHIELD]      = 'shield',
      
      [CASTPARTY_BUFF_INNER_FIRE]             = 'inner fire',
      [CASTPARTY_RES_RESURRECTION]            = 'resurrection',
  }
  
  PRIEST_NAMESPACE.class_buffs = {
      -- need to make sure we offset the efficient heals for priest
      -- as there are 3 spell names to that group, the level checker
      -- needs to see the spells at the correct offsets
      -- NR: indexing of tables normally starts at 0, but for these
      -- it starts at 1.
      [CASTPARTY_HEAL_LESSER_HEAL]            = {1,1,1},
      [CASTPARTY_HEAL_HEAL]                   = {0,0,0,1,1,1,1},
      [CASTPARTY_HEAL_GREATER_HEAL]           = {0,0,0,0,0,0,0,1,1,1,1,1},
      
      [CASTPARTY_HEAL_FLASH_HEAL]             = {1,1,1,1,1,1,1},
      [CASTPARTY_HEAL_RENEW]                  = {8,14,20,26,32,38,44,50,56,60},
      [CASTPARTY_HEAL_PRAYER_OF_HEALING]      = {1,1,1,1,1},
      
      [CASTPARTY_BUFF_POWER_WORD_SHIELD]      = {6,12,18,24,30,36,42,48,54,60},
      [CASTPARTY_BUFF_INNER_FIRE]             = {1,1,1,1,1,1},
      [CASTPARTY_RES_RESURRECTION]            = {1,1,1,1,1},
      
      -- non-healing buffs
      [CASTPARTY_BUFF_POWER_WORD_FORTITUDE]   = {1,12,24,36,48,60},
      [CASTPARTY_BUFF_SHADOW_PROTECTION]      = {30,42,56},
      [CASTPARTY_BUFF_DIVINE_SPIRIT]          = {30,40,50,60},
  }
  
  PRIEST_NAMESPACE.heal_multipliers = {
      [CASTPARTY_HEAL_PRAYER_OF_HEALING]      = 1.75,
  }
  
  PRIEST_NAMESPACE.spell_parsers = {
      [CASTPARTY_HEAL_LESSER_HEAL]            = 2,
      [CASTPARTY_HEAL_HEAL]                   = 2,
      [CASTPARTY_HEAL_GREATER_HEAL]           = 2,
      [CASTPARTY_HEAL_FLASH_HEAL]             = 2,
      [CASTPARTY_HEAL_RENEW]                  = 1,
      [CASTPARTY_HEAL_PRAYER_OF_HEALING]      = 2,
      [CASTPARTY_BUFF_POWER_WORD_SHIELD]      = 1,
      [CASTPARTY_BUFF_INNER_FIRE]             = 2,
      [CASTPARTY_RES_RESURRECTION]            = 1,
  }
  
  PRIEST_NAMESPACE.healing_bonus_multipliers = {
      -- need to make sure we offset the efficient heals for priest
      -- as there are 3 spell names to that group, the level checker
      -- needs to see the spells at the correct offsets
      [CASTPARTY_HEAL_LESSER_HEAL]            = {.086,.229,.429},
      [CASTPARTY_HEAL_HEAL]                   = {0,0,0,.686,.857,.857,.857},
      [CASTPARTY_HEAL_GREATER_HEAL]           = {0,0,0,0,0,0,0,.857,.857,.857,.857,.857},
      [CASTPARTY_HEAL_FLASH_HEAL]             = {.429,.429,.429,.429,.429,.429,.429},
      [CASTPARTY_HEAL_PRAYER_OF_HEALING]      = {.333,.333,.333,.333,.333},
      [CASTPARTY_HEAL_RENEW]                  = {.333,.667,1,1,1,1,1,1,1,1},
  }
  
  PRIEST_NAMESPACE.cures = {
      { spell = CASTPARTY_CURE_DISEASE_1, cures = { CASTPARTY_AILMENT_DISEASE } },
      { spell = CASTPARTY_CURE_DISEASE_2, cures = { CASTPARTY_AILMENT_DISEASE } },
      { spell = CASTPARTY_CURE_MAGIC_1,   cures = { CASTPARTY_AILMENT_MAGIC   } },
  }
  
  PRIEST_NAMESPACE.default_key_bindings = {
     LeftButton  = {
        None              = 'ApplyHeal',
        Alt               = 'ApplyEfficientHeal',
        Ctrl              = 'ApplyInstantHeal',
        Shift             = 'ApplyEmergencyHeal',
        },
     RightButton = {
        None              = 'ApplyShield',
        Shift             = 'Power Word: Fortitude',
        Ctrl              = 'Shadow Protection',
        Alt               = 'Divine Spirit',
        },
     MiddleButton  = {
        None              = 'CureAny',
        Shift             = 'ApplyBandage',
        },
     Button4 = {
        None              = 'TargetUnit',
        Shift             = 'CastParty_PartyDropDown',
        },
     Button5 = {
        None              = 'AssistUnit',
        },
   }
  
  return PRIEST_NAMESPACE
end

--@Override
function PRIEST_NAMESPACE.ReloadSpellData()
  CastParty.Debug(1, "calling priest.ReloadSpellData")
  return CastParty.BrainBase.ReloadSpellData(PRIEST_NAMESPACE)
end

--@Override
function PRIEST_NAMESPACE.GetHealAmount(spellEntry)
  --CastParty.Debug(1, 'calling priest.GetHealAmount')
  return CastParty.BrainBase.GetHealAmount(spellEntry, PRIEST_NAMESPACE);
end

-- todo's remaining
--    - check for player in combat and target is not in emergeny health
--         status, then cast barkskin if there is no cooldown
--    - check for target less then CastPartyConfig.critical_health_percent if so
--         cast nauture's swiftness if there is no cooldown
function PRIEST_NAMESPACE.AdvancedHeal(_unit, _unitHealth, _unitHealthMax, _healType)
   local spell;
   local playerMana;
   local playerManaFraction;
   local unitHealthModified;
   local unitHealthFraction;
   local unitLevel;
   
   CastParty.Debug(4, 'Using advanded heal for '.. PRIEST_NAMESPACE.className)

   _healType,
      playerMana,
      playerManaFraction,
      unitHealthModified,
      unitHealthFraction,
      unitLevel = CastParty.Healomatic.InitHealVars(_unit, _unitHealth, _unitHealthMax, _healType);

   if _healType and _healType == 'resurrection' then
      return CastParty.Healomatic.DoResurrect(_unit, unitLevel, playerMana)
   end
   
   --this will select the shield spell IF it's supposed to!
   spell, _healType = CastParty.Healomatic.DoTheShield(_unit, unitHealthModified, _unitHealthMax, unitHealthFraction, unitLevel, playerMana, _unit, _healType);

   --this is so if we can cast a shield, stop, otherwise we allow the program to move on...
   if spell then
      return spell;
   end

   -- AUTO INNER FIRE
   -- if the target is not in emergency status or worse then check for inner fire
   if _healType == "" and unitHealthFraction > CastPartyConfig.emergency_health_percent
        and not CastParty.State.heal_states[CASTPARTY_BUFF_INNER_FIRE] and CastParty.Brain.heal_data['inner fire']
        and CastPartyConfig.auto_cast_inner_fire then
      -- at least make sure were in combat or its wasted mana
      if (CastPartyConfig.emergency_in_combat
           and UnitAffectingCombat('player')
           and CastParty.State.last_player_damage
           and (GetTime() - CastParty.State.last_player_damage) < CastPartyConfig.emergency_damage_window) then
         -- add to health, to force the best inner fire we have mana for
         spell = GetSpell( _unitHealth, unitHealthModified, _unitHealthMax, 'inner fire', unitLevel, playerMana, _unit, true );

         if spell then
            -- put caster is under attack announce here
            CastParty_SpeakFailure(_unit, 'HUA');
            return spell;
         end
      end
   end
   -- END AUTO INNER FIRE

   -- make sure instant cast is running 1st, while instant cast is
   -- a minor heal, it is farly effecient, and is a low argo spell
   if ( _healType == 'instant' or _healType == "" ) then
      spell, _healType  = CastParty.Healomatic.DoInstantHeal(CASTPARTY_HEAL_RENEW, _unitHealth, unitHealthModified, unitHealthFraction, _unitHealthMax, unitLevel, playerMana, _unit);
   end

   --this is so if we can cast a instant-cast, stop, otherwise we allow the program to move on...
   if spell then
      return spell;
   end

   -- I did not orignally want to do this, as the addon was about
   -- conserving mana, but I am but one, and many have stated to
   -- me personally that they would rather use flash heal in this
   -- way even know it uses more mana, there main reason is instances
   -- in instances it is more desired to have a faster heal then
   -- a slow efficent one.  This of course can be changed back rather
   -- quickly to follow the normal emergency heal rules
   if _healType == 'emergency' or _healType == "" then
      local Adjusted_health = 0;

      -- load up the spell for testing
      if CastParty.Brain.heal_data['emergency'] then
         spell, Adjusted_health = GetSpell( _unitHealth, unitHealthModified, _unitHealthMax, 'emergency', unitLevel, playerMana, _unit );

         spell = CastParty.Healomatic.DoEmergencyInCombat(spell, _healType, unitHealthFraction, playerManaFraction, Adjusted_health, unitHealthModified);
      end

      --this is so if we can cast and emergency spell, do it, otherwise we allow the program to move on...
      if spell then
         return spell;
      end
   end

   -- always choose the best group heals based on mana
   -- only cast a group spell if specified to cast a group spell however...????
   if _healType == 'group' and CastParty.Brain.heal_data['group'] then
      spell = GetSpell( _unitHealth, unitHealthModified, _unitHealthMax, 'group', unitLevel, playerMana, _unit )
      if spell then
         CastParty.Debug( 2, YEL..'Emergency heal!')
         return spell;
      end
   end

   -- ok this is it, its time for the big guns of healing we don't have an
   -- instant to cast and dont have an emergency to cast, this is the last one
   if CastParty.Brain.heal_data['efficient'] then
      return GetSpell( _unitHealth, unitHealthModified, _unitHealthMax, 'efficient', unitLevel, playerMana, _unit );
   end
end

