-- Make a class for Shaman-based constants and funcitonality
-- load this class when castparty starts and detects the player's class as shaman
-- use an instance to call desired overloaded methods

local SHAMAN_NAMESPACE = {} -- declare the namespace
CastParty.BrainFactory.SHAMAN = SHAMAN_NAMESPACE

--DEFAULT_CHAT_FRAME:AddMessage("loading brainShaman.lua")

function SHAMAN_NAMESPACE.new()
   CastParty.Debug(4, "making a new shaman brain")
   
   SHAMAN_NAMESPACE.className = 'SHAMAN'
   
   SHAMAN_NAMESPACE.heal_types = {
       [CASTPARTY_HEAL_HEALING_WAVE]           = 'efficient',
       [CASTPARTY_HEAL_LESSER_HEALING_WAVE]    = 'emergency',
       [CASTPARTY_HEAL_CHAIN_HEAL]             = 'group',
       [CASTPARTY_RES_ANCESTRAL_SPIRIT]        = 'resurrection',
   }
   
   SHAMAN_NAMESPACE.class_buffs = {
       [CASTPARTY_HEAL_HEALING_WAVE]           = {1,1,1,1,1,1,1,1,1,1},
       [CASTPARTY_HEAL_LESSER_HEALING_WAVE]    = {1,1,1,1,1,1},
       [CASTPARTY_HEAL_CHAIN_HEAL]             = {1,1,1},
       [CASTPARTY_RES_ANCESTRAL_SPIRIT]        = {1,1,1,1,1},
       
       --TODO: add non heal/shield buffs
   }
   
   SHAMAN_NAMESPACE.heal_multipliers = {
       [CASTPARTY_HEAL_CHAIN_HEAL]             = 1.75,
   }
   
   SHAMAN_NAMESPACE.spell_parsers = {
       [CASTPARTY_HEAL_HEALING_WAVE]           = 2,
       [CASTPARTY_HEAL_LESSER_HEALING_WAVE]    = 2,
       [CASTPARTY_HEAL_CHAIN_HEAL]             = 1,
       [CASTPARTY_RES_ANCESTRAL_SPIRIT]        = 2,
   }
   
   SHAMAN_NAMESPACE.healing_bonus_multipliers = {
       [CASTPARTY_HEAL_HEALING_WAVE]           = {.086,.229,.429,.686,.857,.857,.857,.857,.857,.857},
       [CASTPARTY_HEAL_LESSER_HEALING_WAVE]    = {.429,.429,.429,.429,.429,.429},
       [CASTPARTY_HEAL_CHAIN_HEAL]             = {.333,.333,.333},
   }
   
   SHAMAN_NAMESPACE.cures = {
       { spell = CASTPARTY_CURE_PURGE,     cures = { CASTPARTY_AILMENT_MAGIC   } },
   }
   
   SHAMAN_NAMESPACE.default_key_bindings = {
      LeftButton  = {
         None              = 'ApplyHeal',
         Alt               = 'ApplyEfficientHeal',
         AltCtrl           = 'TargetUnit',
         Ctrl              = 'ApplyInstantHeal',
         CtrlShift         = 'AssistUnit',
         Shift             = 'ApplyEmergencyHeal',
         },
      RightButton = {
         None              = 'CureAny',
         AltCtrlShift      = 'DoTheRightThing',
         Ctrl              = 'CastParty_PartyDropDown',
         Shift             = 'CureGroupMember',
         },
      MiddleButton  = {
         None              = 'ApplyBandage',
         },
      Button4 = {
         None              = 'TargetUnit',
         Shift             = 'CastParty_PartyDropDown',
         },
      Button5 = {
        None              = 'AssistUnit',
        },
   }

   return SHAMAN_NAMESPACE
end

--@Override
function SHAMAN_NAMESPACE.ReloadSpellData()
  CastParty.Debug(1, "calling shaman.ReloadSpellData")
  return CastParty.BrainBase.ReloadSpellData(SHAMAN_NAMESPACE)
end

--@Override
function SHAMAN_NAMESPACE.GetHealAmount(spellEntry)
  --CastParty.Debug(1, 'calling shaman.GetHealAmount')
  return CastParty.BrainBase.GetHealAmount(spellEntry, SHAMAN_NAMESPACE);
end

--
-- Shaman.AdvancedHeal *anyone play a shaman?* that could tell me what their normal heal cycle is?
--
function SHAMAN_NAMESPACE.AdvancedHeal( _unit, _unitHealth, _unitHealthMax, _healType )
   local spell;
   local playerMana;
   local playerManaFraction;
   local unitHealthModified;
   local unitHealthFraction;
   local unitLevel;

   local _healType, playerMana, playerManaFraction, unitHealthModified, unitHealthFraction, unitLevel = CastParty.Healomatic.InitHealVars(_unit, _unitHealth, _unitHealthMax, _healType);

   if _healType and _healType == 'resurrection' then
      return CastParty.Healomatic.DoResurrect(_unit, unitLevel, playerMana)
   end
   
   -- shamans's smaller one is more  mana efficient, and is faster to cast,
   -- so check to see if we can use it or not
   if _healType == 'emergency' or _healType == "" then
      if CastParty_Debug then CastParty_Debug( 1, GRY..'begin emergency checks...') end;
      -- is a previous spell still running? if so skip and cast a normal spell
      -- no futher benfit can be gained from re-casting this spell, it would
      -- just be a waste of addtional mana, as the direct healing of the spell
      -- is rather minor
      if CastParty.Brain.heal_data['emergency']then
         local Adjusted_health = 0;

         -- load up the spell for testing
         spell, Adjusted_health = GetSpell( _unitHealth, unitHealthModified, _unitHealthMax, 'emergency', unitLevel, playerMana, _unit );

         spell = CastParty.Healomatic.DoEmergencyInCombat(spell, _healType, unitHealthFraction, playerManaFraction, Adjusted_health, unitHealthModified);
      end

      --this is so if we can cast and emergency spell, do it, otherwise we allow the program to move on...
      if spell then
         return spell;
      end
   end

   -- always choose the best group heals based on mana
   -- only cast a group spell if specified to cast a group spell however...????
   if _healType == 'group' and CastParty.Brain.heal_data['group'] then
      return GetSpell( _unitHealth, unitHealthModified, _unitHealthMax, 'group', unitLevel, playerMana, _unit )
   end

   -- ok this is it, its time for the big guns of healing we don't have any
   -- more spells to try, this is the last one
   if CastParty.Brain.heal_data['efficient'] then
      return GetSpell( _unitHealth, unitHealthModified, _unitHealthMax, 'efficient', unitLevel, playerMana, _unit);
   end

end
