
local KEY_BINDINGS_TYPES_DDLIST = {
   CASTPARTY_KEYBINDINGS_NONE,
   'ApplyHeal',
   'ApplyEfficientHeal',
   'ApplyEmergencyHeal',
   'ApplyInstantHeal',
   'ApplyGroupHeal',
   'ApplyResurrectionHeal',
   'ApplyShield',
   'ApplyBandage',
   'Panic',
   'CureAny',
   'CureGroupMember',
   'DoTheRightThing',
   'CastParty_WoWDefaultClick',
   'CastParty_PartyDropDown',
   'TargetUnit',
   'AssistUnit',
   CASTPARTY_KEYBINDINGS_OTHER,
}

local BUTTON_TYPES = {
  'LeftButton',
  'RightButton',
  'MiddleButton',
  'Button4',
  'Button5',
}
local MODIFIERS_LIST = {};

local cpo_buttonType = 'LeftButton';
local dropdown_selected;
local OtherButton;


-- Setup and methods associated with the key binding dropdown by
-- Nathan Robinson for CastParty



--***************************************************************************
--***************************************************************************
function KeyBindingsDDButton_OnClick()
   local id = this:GetID();

   local button = string.gsub(dropdown_selected, '.*Button*4*5*', "")

   UIDropDownMenu_SetSelectedID(getglobal('CastPartyKeyBindingsFrameDDButton' .. button), id);

   if (id == 1) then
      CastPartyConfig.key_bindings[cpo_buttonType][button] = nil
   elseif (id == table.getn(KEY_BINDINGS_TYPES_DDLIST)) then
      CastPartyConfigureOther:Show();
      CastPartyKeyBindingsFrame:Hide();
   else
      CastPartyConfig.key_bindings[cpo_buttonType][button] = KEY_BINDINGS_TYPES_DDLIST[id];
   end

end


--***************************************************************************
--***************************************************************************
-- OnEnterPressed of 'Other...' dialog
function SetKeyBindingOther(spell_name)
   if spell_name ~= "" then
      CastPartyConfig.key_bindings[cpo_buttonType][OtherButton] = spell_name
   else
      CastPartyConfig.key_bindings[cpo_buttonType][OtherButton] = nil
   end
   CastParty_SetDropdowns();
end

--***************************************************************************
--***************************************************************************
-- OnShow of the 'Other...' dialog
function CastPartySetOtherText()

   OtherButton = string.gsub(dropdown_selected, '.*Button*4*5*', "")

   local name = CastPartyConfig.key_bindings[cpo_buttonType][OtherButton]

   if (name) then
      CastPartyConfigureOtherEBSpellName:SetText(name);
   else
      CastPartyConfigureOtherEBSpellName:SetText("");
   end
end

--***************************************************************************
--***************************************************************************
-- Gets the index of an item in a list
function GetDropDownID(list, value)
   for index in pairs(list) do
      if (list[index] == value) then
         return index;
      end
   end
   return -1;
end

--***************************************************************************
--***************************************************************************
--button clicked (to select which button is being configured)
function CastPartySetButton(button)
   cpo_buttonType = button;
   CloseMenus();
   CastParty_SetDropdowns();

   MODIFIERS_LIST = {
     'None',
     'Alt',
     'AltCtrl',
     'AltCtrlShift',
     'AltShift',
     'Ctrl',
     'CtrlShift',
     'Shift'
   }

   for i=1, 8 do
      -- Set FontString text
      local text = AddChars(MODIFIERS_LIST[i], ' + ');
      if (i ~= 1) then
         text = text ..' + '..button..':';
      else
         text = button..':';
      end

      getglobal('CastPartyKeyBindingsFrameLeft'..MODIFIERS_LIST[i]):SetText(text);
   end

   CastPartyKeyBindingsFrameLeftButton:Enable();
   CastPartyKeyBindingsFrameRightButton:Enable();
   CastPartyKeyBindingsFrameMiddleButton:Enable();
   CastPartyKeyBindingsFrameButton4:Enable();
   CastPartyKeyBindingsFrameButton5:Enable();
end

--***************************************************************************
--***************************************************************************
--Sets the value of a dropdown to the corresponding value.
function SetDropdownValue(dropdown, value)
   local id;
   if (value) then
      id = GetDropDownID(KEY_BINDINGS_TYPES_DDLIST, value);
   else
      id = 1; -- Set to None
   end

   if (id == -1) then
      id = table.getn(KEY_BINDINGS_TYPES_DDLIST);  -- Set to Other
   end

   -- Set the value of that dropdown to our variable
   UIDropDownMenu_SetSelectedID(dropdown, id);
end



--***************************************************************************
--***************************************************************************
-- Sets the KeyBindings dropdowns to the corresponding values.
function CastParty_SetDropdowns()
   SetDropdownValue(CastPartyKeyBindingsFrameDDButtonNone, CastPartyConfig.key_bindings[cpo_buttonType].None)
   SetDropdownValue(CastPartyKeyBindingsFrameDDButtonAlt, CastPartyConfig.key_bindings[cpo_buttonType].Alt)
   SetDropdownValue(CastPartyKeyBindingsFrameDDButtonAltCtrl, CastPartyConfig.key_bindings[cpo_buttonType].AltCtrl)
   SetDropdownValue(CastPartyKeyBindingsFrameDDButtonAltCtrlShift, CastPartyConfig.key_bindings[cpo_buttonType].AltCtrlShift)
   SetDropdownValue(CastPartyKeyBindingsFrameDDButtonAltShift, CastPartyConfig.key_bindings[cpo_buttonType].AltShift)
   SetDropdownValue(CastPartyKeyBindingsFrameDDButtonCtrl, CastPartyConfig.key_bindings[cpo_buttonType].Ctrl)
   SetDropdownValue(CastPartyKeyBindingsFrameDDButtonCtrlShift, CastPartyConfig.key_bindings[cpo_buttonType].CtrlShift)
   SetDropdownValue(CastPartyKeyBindingsFrameDDButtonShift, CastPartyConfig.key_bindings[cpo_buttonType].Shift);
end

--***************************************************************************
--***************************************************************************
-- Uses AlternateCapitalization to find the endings of words,
-- and adds the specified chars after each word
function AddChars(str, chars)
   local retVal = "";
   local index = 1;
   local lower_str = strlower(str);
   for i=2, strlen(str), 1 do
     if (strbyte(str, i) ~= strbyte(lower_str, i)) then
        retVal = retVal .. strsub(str, index, i-1) .. chars;
        index = i;
     end
   end
   retVal = retVal .. strsub(str, index, i);
   return retVal;
end

--***************************************************************************
--***************************************************************************
-- Dropdowns Initializers
function KeyBindingsDDButtonNone_Initialize()
   for i = 1, table.getn(KEY_BINDINGS_TYPES_DDLIST), 1 do
      local info = { };
      info.text = KEY_BINDINGS_TYPES_DDLIST[i];
      info.func = function() dropdown_selected = cpo_buttonType .. 'None'; KeyBindingsDDButton_OnClick(); end
      UIDropDownMenu_AddButton(info);
   end
end
function KeyBindingsDDButtonAlt_Initialize()
   for i = 1, table.getn(KEY_BINDINGS_TYPES_DDLIST), 1 do
      local info = { };
      info.text = KEY_BINDINGS_TYPES_DDLIST[i];
      info.func = function() dropdown_selected = cpo_buttonType .. 'Alt'; KeyBindingsDDButton_OnClick(); end
      UIDropDownMenu_AddButton(info);
   end
end
function KeyBindingsDDButtonAltCtrl_Initialize()
   for i = 1, table.getn(KEY_BINDINGS_TYPES_DDLIST), 1 do
      local info = { };
      info.text = KEY_BINDINGS_TYPES_DDLIST[i];
      info.func = function() dropdown_selected = cpo_buttonType .. 'AltCtrl'; KeyBindingsDDButton_OnClick(); end
      UIDropDownMenu_AddButton(info);
   end
end
function KeyBindingsDDButtonAltCtrlShift_Initialize()
   for i = 1, table.getn(KEY_BINDINGS_TYPES_DDLIST), 1 do
      local info = { };
      info.text = KEY_BINDINGS_TYPES_DDLIST[i];
      info.func = function() dropdown_selected = cpo_buttonType .. 'AltCtrlShift'; KeyBindingsDDButton_OnClick(); end
      UIDropDownMenu_AddButton(info);
   end
end
function KeyBindingsDDButtonAltShift_Initialize()
   for i = 1, table.getn(KEY_BINDINGS_TYPES_DDLIST), 1 do
      local info = { };
      info.text = KEY_BINDINGS_TYPES_DDLIST[i];
      info.func = function() dropdown_selected = cpo_buttonType .. 'AltShift'; KeyBindingsDDButton_OnClick(); end
      UIDropDownMenu_AddButton(info);
   end
end
function KeyBindingsDDButtonCtrl_Initialize()
   for i = 1, table.getn(KEY_BINDINGS_TYPES_DDLIST), 1 do
      local info = { };
      info.text = KEY_BINDINGS_TYPES_DDLIST[i];
      info.func = function() dropdown_selected = cpo_buttonType .. 'Ctrl'; KeyBindingsDDButton_OnClick(); end
      UIDropDownMenu_AddButton(info);
   end
end
function KeyBindingsDDButtonCtrlShift_Initialize()
   for i = 1, table.getn(KEY_BINDINGS_TYPES_DDLIST), 1 do
      local info = { };
      info.text = KEY_BINDINGS_TYPES_DDLIST[i];
      info.func = function() dropdown_selected = cpo_buttonType .. 'CtrlShift'; KeyBindingsDDButton_OnClick(); end
      UIDropDownMenu_AddButton(info);
   end
end
function KeyBindingsDDButtonShift_Initialize()
   for i = 1, table.getn(KEY_BINDINGS_TYPES_DDLIST), 1 do
      local info = { };
      info.text = KEY_BINDINGS_TYPES_DDLIST[i];
      info.func = function() dropdown_selected = cpo_buttonType .. 'Shift'; KeyBindingsDDButton_OnClick(); end
      UIDropDownMenu_AddButton(info);
   end
end

--***************************************************************************
--***************************************************************************
-- Initialize options on load
function CastPartyOptions_SetOptionValuesBindings()
   local DD_WIDTH = 200;

   UIDropDownMenu_Initialize(CastPartyKeyBindingsFrameDDButtonNone, KeyBindingsDDButtonNone_Initialize);
   UIDropDownMenu_SetWidth(DD_WIDTH, CastPartyKeyBindingsFrameDDButtonNone);

   UIDropDownMenu_Initialize(CastPartyKeyBindingsFrameDDButtonAlt, KeyBindingsDDButtonAlt_Initialize);
   UIDropDownMenu_SetWidth(DD_WIDTH, CastPartyKeyBindingsFrameDDButtonAlt);

   UIDropDownMenu_Initialize(CastPartyKeyBindingsFrameDDButtonAltCtrl, KeyBindingsDDButtonAltCtrl_Initialize);
   UIDropDownMenu_SetWidth(DD_WIDTH, CastPartyKeyBindingsFrameDDButtonAltCtrl);

   UIDropDownMenu_Initialize(CastPartyKeyBindingsFrameDDButtonAltCtrlShift, KeyBindingsDDButtonAltCtrlShift_Initialize);
   UIDropDownMenu_SetWidth(DD_WIDTH, CastPartyKeyBindingsFrameDDButtonAltCtrlShift);

   UIDropDownMenu_Initialize(CastPartyKeyBindingsFrameDDButtonAltShift, KeyBindingsDDButtonAltShift_Initialize);
   UIDropDownMenu_SetWidth(DD_WIDTH, CastPartyKeyBindingsFrameDDButtonAltShift);

   UIDropDownMenu_Initialize(CastPartyKeyBindingsFrameDDButtonCtrl, KeyBindingsDDButtonCtrl_Initialize);
   UIDropDownMenu_SetWidth(DD_WIDTH, CastPartyKeyBindingsFrameDDButtonCtrl);

   UIDropDownMenu_Initialize(CastPartyKeyBindingsFrameDDButtonCtrlShift, KeyBindingsDDButtonCtrlShift_Initialize);
   UIDropDownMenu_SetWidth(DD_WIDTH, CastPartyKeyBindingsFrameDDButtonCtrlShift);

   UIDropDownMenu_Initialize(CastPartyKeyBindingsFrameDDButtonShift, KeyBindingsDDButtonShift_Initialize);
   UIDropDownMenu_SetWidth(DD_WIDTH, CastPartyKeyBindingsFrameDDButtonShift);

   CastParty_SetDropdowns()
end